@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2018 Edgewall Software
+# Copyright (C) 2008-2020 Edgewall Software
 # Copyright (C) 2008 Noah Kantrowitz <noah@coderanger.net>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 # Trac documentation build configuration file, created by
 # sphinx-quickstart on Wed May 14 09:05:13 2008.
@@ -31,6 +31,12 @@ from datetime import datetime
 
 from trac.util import get_pkginfo
 
+try:
+    from pygments.lexers.templates import DjangoLexer
+    html_jinja_lexer = DjangoLexer()
+except ImportError:
+    html_jinja_lexer = None
+
 pkg_info = get_pkginfo(sys.modules['trac'])
 
 # General substitutions.
@@ -68,6 +74,7 @@ extensions = []
 
 extensions.append('sphinx.ext.autodoc')
 
+autodoc_default_flags = ['members', 'show-inheritance']
 autoclass_content = 'both'
 autodoc_member_order = 'bysource'
 
@@ -252,13 +259,13 @@ latex_documents = [
 # The options element is a dictionary that lets you override
 # this config per-document.
 # For example,
-# ('index', u'MyProject', u'My Project', u'Author Name',
+# ('index', 'MyProject', 'My Project', 'Author Name',
 #  dict(pdf_compressed = True))
 # would mean that specific document would be compressed
 # regardless of the global pdf_compressed setting.
 
 pdf_documents = [
-    ('index', 'trac_dev', project, u'The Trac Team'),
+    ('index', 'trac_dev', project, 'The Trac Team'),
 ]
 
 # A comma-separated list of custom stylesheets (latest has higher precedence)
@@ -368,3 +375,5 @@ def setup(app):
 
     # ifconfig variables
     app.add_config_value('devel', True, '')
+    if html_jinja_lexer:
+        app.add_lexer('html+jinja', html_jinja_lexer)
