@@ -1,27 +1,28 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2018 Edgewall Software
+# Copyright (C) 2012-2020 Edgewall Software
 # Copyright (C) 2012 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 """Trac API doc checker
 
 Verify that all symbols belonging to modules already documented in the
 doc/api Sphinx sources are referenced.
 
-See http://trac.edgewall.org/wiki/TracDev/ApiDocs
+See https://trac.edgewall.org/wiki/TracDev/ApiDocs
 
 """
 
 import fnmatch
+import importlib
 import os
 import re
 import sys
@@ -73,7 +74,7 @@ def main(argv):
 def check_api_doc(basename, verbose, only_documented, has_submodules):
     module_name = basename.replace('_', '.')
     try:
-        module = __import__(module_name, globals(), {}, ['__all__'])
+        module = importlib.import_module(module_name)
     except ImportError as e:
         print("Skipping %s (%s)" % (basename, e))
         return
@@ -84,7 +85,7 @@ def check_api_doc(basename, verbose, only_documented, has_submodules):
         all = get_default_symbols(module, only_documented, has_submodules)
     no_apidoc = getattr(module, '__no_apidoc__', None)
     if no_apidoc:
-        if isinstance(no_apidoc, basestring):
+        if isinstance(no_apidoc, str):
             no_apidoc = [s.strip() for s in no_apidoc.split()]
         all = list(set(all) - set(no_apidoc))
     symbols, keywords = get_sphinx_documented_symbols(basename + '.rst')
@@ -97,13 +98,14 @@ def check_api_doc(basename, verbose, only_documented, has_submodules):
             value = getattr(module, symbol)
             cls = getattr(value, '__class__', None)
             keyword = 'data'
-            if cls.__name__ in ('function', 'instancemethod'):
-                keyword = 'function'
-            elif cls.__name__ == 'module':
-                keyword = 'module'
-            else:
-                keyword = 'class'
-            print(" * .. %14s :: %s" % ('auto' + keyword, symbol))
+            if cls:  # old-style class objects have not __class__ attribute
+                if cls.__name__ in ('function', 'instancemethod'):
+                    keyword = 'function'
+                elif cls.__name__ == 'module':
+                    keyword = 'module'
+                else:
+                    keyword = 'class'
+                print(" * .. %14s :: %s" % ('auto' + keyword, symbol))
 
 
 sphinx_doc_re = re.compile(r'''
@@ -112,7 +114,7 @@ sphinx_doc_re = re.compile(r'''
 ''', re.MULTILINE | re.VERBOSE)
 
 def get_sphinx_documented_symbols(rst):
-    with open(os.path.join(api_doc, rst)) as f:
+    with open(os.path.join(api_doc, rst), encoding='utf-8') as f:
         doc = f.read()
     symbols, keywords = [], []
     for k, s in sphinx_doc_re.findall(doc):
@@ -128,7 +130,7 @@ def get_default_symbols(module, only_documented, has_submodules):
     all = []
     for symbol in public:
         try:
-            __import__(symbol)
+            importlib.import_module(symbol)
         except ImportError:
             all.append(symbol)
     # only keep symbols having a docstring
@@ -143,7 +145,7 @@ def get_default_symbols(module, only_documented, has_submodules):
     return all
 
 def get_public_symbols(m):
-    return set(symbol for symbol in dir(m) if not symbol.startswith('_'))
+    return {symbol for symbol in dir(m) if not symbol.startswith('_')}
 
 import_from_re = re.compile(r'''
 ^ \s* from \s+ ([\w\.]+) \s+ import \s+   # module
@@ -159,20 +161,21 @@ def get_imported_symbols(module, has_submodules):
     src_filename = module.__file__.replace('\\', '/').replace('.pyc', '.py')
     if src_filename.endswith('/__init__.py') and not has_submodules:
         return set()
-    with open(src_filename) as f:
+    with open(src_filename, encoding='utf-8') as f:
         src = f.read()
     imported = set()
     for mod, symbol_list in import_from_re.findall(src):
         symbol_list = symbol_list.strip()
         if symbol_list == '*':
             try:
-                imported_module = __import__(mod, globals(), {}, ['__all__'])
-                symbols = set(getattr(imported_module, '__all__', None) or
-                              get_public_symbols(imported_module))
+                imported_module = importlib.import_module(mod)
             except ImportError:
                 print("Warning: 'from %s import *' couldn't be resolved"
                       % mod)
                 continue
+            else:
+                symbols = set(getattr(imported_module, '__all__', None) or
+                              get_public_symbols(imported_module))
         else:
             if symbol_list and symbol_list[0] == '(' and symbol_list[-1] == ')':
                 symbol_list = symbol_list[1:-1]
