@@ -1,21 +1,19 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2018 Edgewall Software
+# Copyright (C) 2010-2020 Edgewall Software
 # Copyright (C) 2010 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 # Simple wrapper script needed to run epydoc
 
-from __future__ import print_function
-
 import sys
 
 try:
