@@ -1,24 +1,24 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2018 Edgewall Software
+# Copyright (C) 2007-2020 Edgewall Software
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 """Example macro."""
 
 from trac.util.translation import cleandoc_
 from trac.wiki.macros import WikiMacroBase
 
-revision = "$Rev: 16771 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.2-stable/sample-plugins/HelloWorld.py $"
+revision = "$Rev$"
+url = "$URL$"
 
 
 class HelloWorldMacro(WikiMacroBase):
@@ -46,7 +46,9 @@ class HelloWorldMacro(WikiMacroBase):
           Wiki processor syntax and will be `None` if called using the
           macro syntax.
         """
-        return 'Hello World, content = ' + unicode(content)
+        return 'Hello World, content = ' + str(content)
 
-    # Note that there's no need to HTML escape the returned data,
-    # as the template engine (Genshi) will do it for us.
+    # Note that there's no need to HTML escape the returned data, as
+    # the template engine (Jinja2) will do it for us.  To prevent
+    # escaping, return a Markup instance or use the tag builder API
+    # from trac.util.html.
