@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2018 Edgewall Software
+# Copyright (C) 2009-2020 Edgewall Software
 # Copyright (C) 2009 Remy Blank <remy.blank@pobox.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 import re
 
@@ -20,8 +20,8 @@ from trac.resource import ResourceNotFound
 from trac.ticket.api import IMilestoneChangeListener
 from trac.ticket.model import Version
 
-revision = "$Rev: 16771 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.2-stable/sample-plugins/milestone_to_version.py $"
+revision = "$Rev$"
+url = "$URL$"
 
 
 class MilestoneToVersion(Component):
@@ -36,10 +36,11 @@ class MilestoneToVersion(Component):
 
     pattern = Option('milestone_to_version', 'pattern',
                      r'(?i)(?:v(?:er)?\.?|version)?\s*(?P<version>\d.*)',
-        """A regular expression to match the names of milestones that should be
-        made into versions when they are completed. The pattern must include
-        one named group called 'version' that matches the version number
-        itself.""")
+        """A regular expression to match the names of milestones that
+        should be made into versions when they are completed. The pattern
+        must include one named group called 'version' that matches the
+        version number itself.
+        """)
 
     def milestone_created(self, milestone):
         pass
@@ -56,6 +57,14 @@ class MilestoneToVersion(Component):
             return
         try:
             version = Version(self.env, version_name)
+        except ResourceNotFound:
+            version = Version(self.env)
+            version.name = version_name
+            version.time = milestone.completed
+            version.insert()
+            self.log.info('New version "%s" created from completed milestone '
+                          '"%s".', version.name, milestone.name)
+        else:
             if not version.time:
                 version.time = milestone.completed
                 version.update()
@@ -66,13 +75,6 @@ class MilestoneToVersion(Component):
                 self.log.info('Version "%s" already exists.  No new version '
                               'created from milestone "%s"', version.name,
                               milestone.name)
-        except ResourceNotFound:
-            version = Version(self.env)
-            version.name = version_name
-            version.time = milestone.completed
-            version.insert()
-            self.log.info('New version "%s" created from completed milestone '
-                          '"%s".', version.name, milestone.name)
 
     def milestone_deleted(self, milestone):
         pass
