@@ -1,25 +1,25 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2018 Edgewall Software
+# Copyright (C) 2009-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 
 from trac.perm import PermissionSystem
-from trac.tests.functional import FunctionalTwillTestCaseSetup, tc
+from trac.tests.functional import FunctionalTestCaseSetup, tc
 from trac.util.text import unicode_to_base64
 
 
-class AuthorizationTestCaseSetup(FunctionalTwillTestCaseSetup):
+class AuthorizationTestCaseSetup(FunctionalTestCaseSetup):
     def test_authorization(self, href, perms, h2_text):
         """Check permissions required to access an administration panel.
 
@@ -31,7 +31,7 @@ class AuthorizationTestCaseSetup(FunctionalTwillTestCaseSetup):
         self._tester.go_to_front()
         self._tester.logout()
         self._tester.login('user')
-        if isinstance(perms, basestring):
+        if isinstance(perms, str):
             perms = (perms, )
 
         h2 = r'<h2>[ \t\n]*%s[ \t\n]*' \
@@ -62,7 +62,7 @@ class AuthorizationTestCaseSetup(FunctionalTwillTestCaseSetup):
             self._tester.login('admin')
 
 
-class TestBasicSettings(FunctionalTwillTestCaseSetup):
+class TestBasicSettings(FunctionalTestCaseSetup):
     def runTest(self):
         """Check basic settings."""
         self._tester.go_to_admin()
@@ -91,7 +91,7 @@ class TestBasicSettingsAuthorization(AuthorizationTestCaseSetup):
                                 "Basic Settings")
 
 
-class TestDefaultHandler(FunctionalTwillTestCaseSetup):
+class TestDefaultHandler(FunctionalTestCaseSetup):
     def runTest(self):
         """Set default handler from the Basic Settings page."""
 
@@ -119,8 +119,8 @@ class TestDefaultHandler(FunctionalTwillTestCaseSetup):
                                      'disabled')
             self._tester.go_to_admin("Basic Settings")
             tc.find(r'<option value="TimelineModule">TimelineModule</option>')
-            tc.find(r'<span class="hint">TimelineModule is not a valid '
-                    r'IRequestHandler or is not enabled.</span>')
+            tc.find(r'<span class="hint">\s*TimelineModule is not a valid '
+                    r'IRequestHandler or is not enabled.\s*</span>')
             tc.go(self._tester.url)
             tc.find(r'<h1>Configuration Error</h1>')
             tc.find(r'Cannot find an implementation of the '
@@ -137,14 +137,14 @@ class TestDefaultHandler(FunctionalTwillTestCaseSetup):
             self._tester.go_to_admin("Basic Settings")
             tc.find(r'<option value="BatchModifyModule">BatchModifyModule'
                     r'</option>')
-            tc.find(r'<span class="hint">BatchModifyModule is not a valid '
-                    r'IRequestHandler or is not enabled.</span>')
+            tc.find(r'<span class="hint">\s*BatchModifyModule is not a valid '
+                    r'IRequestHandler or is not enabled.\s*</span>')
             tc.formvalue('modbasic', 'default_handler', 'BatchModifyModule')
             tc.submit()  # Invalid value should not be replaced on submit
             tc.find(r'<option value="BatchModifyModule">BatchModifyModule'
                     r'</option>')
-            tc.find(r'<span class="hint">BatchModifyModule is not a valid '
-                    r'IRequestHandler or is not enabled.</span>')
+            tc.find(r'<span class="hint">\s*BatchModifyModule is not a valid '
+                    r'IRequestHandler or is not enabled.\s*</span>')
             tc.go(self._tester.url)
             tc.find(r'<h1>Configuration Error</h1>')
             tc.find(r'<code>BatchModifyModule</code> is not a valid default '
@@ -153,7 +153,7 @@ class TestDefaultHandler(FunctionalTwillTestCaseSetup):
             self._testenv.set_config('trac', 'default_handler', 'WikiModule')
 
 
-class TestLoggingNone(FunctionalTwillTestCaseSetup):
+class TestLoggingNone(FunctionalTestCaseSetup):
     def runTest(self):
         """Turn off logging."""
         # For now, we just check that it shows up.
@@ -161,7 +161,7 @@ class TestLoggingNone(FunctionalTwillTestCaseSetup):
         tc.find('trac.log')
         tc.formvalue('modlog', 'log_type', 'none')
         tc.submit()
-        tc.find('selected="selected">None</option')
+        tc.find('selected="selected" value="none">None</option')
 
 
 class TestLoggingAuthorization(AuthorizationTestCaseSetup):
@@ -171,7 +171,7 @@ class TestLoggingAuthorization(AuthorizationTestCaseSetup):
                                 "Logging")
 
 
-class TestLoggingToFile(FunctionalTwillTestCaseSetup):
+class TestLoggingToFile(FunctionalTestCaseSetup):
     def runTest(self):
         """Turn logging back on."""
         # For now, we just check that it shows up.
@@ -181,12 +181,12 @@ class TestLoggingToFile(FunctionalTwillTestCaseSetup):
         tc.formvalue('modlog', 'log_file', 'trac.log2')
         tc.formvalue('modlog', 'log_level', 'INFO')
         tc.submit()
-        tc.find('selected="selected">File</option')
+        tc.find('selected="selected" value="file">File</option')
         tc.find('id="log_file".*value="trac.log2"')
         tc.find('selected="selected">INFO</option>')
 
 
-class TestLoggingToFileNormal(FunctionalTwillTestCaseSetup):
+class TestLoggingToFileNormal(FunctionalTestCaseSetup):
     def runTest(self):
         """Setting logging back to normal."""
         # For now, we just check that it shows up.
@@ -195,7 +195,7 @@ class TestLoggingToFileNormal(FunctionalTwillTestCaseSetup):
         tc.formvalue('modlog', 'log_file', 'trac.log')
         tc.formvalue('modlog', 'log_level', 'DEBUG')
         tc.submit()
-        tc.find('selected="selected">File</option')
+        tc.find('selected="selected" value="file">File</option')
         tc.find('id="log_file".*value="trac.log"')
         tc.find('selected="selected">DEBUG</option>')
 
@@ -208,7 +208,7 @@ class TestPermissionsAuthorization(AuthorizationTestCaseSetup):
                                 "Manage Permissions and Groups")
 
 
-class TestCreatePermissionGroup(FunctionalTwillTestCaseSetup):
+class TestCreatePermissionGroup(FunctionalTestCaseSetup):
     def runTest(self):
         """Create a permissions group"""
         self._tester.go_to_admin("Permissions")
@@ -221,7 +221,21 @@ class TestCreatePermissionGroup(FunctionalTwillTestCaseSetup):
         tc.find('%s:%s' % (somegroup, REPORT_CREATE))
 
 
-class TestAddUserToGroup(FunctionalTwillTestCaseSetup):
+class TestRemovePermissionGroup(FunctionalTestCaseSetup):
+    def runTest(self):
+        """Remove a permissions group"""
+        self._tester.go_to_admin("Permissions")
+        tc.find('Manage Permissions')
+        somegroup = unicode_to_base64('somegroup')
+        REPORT_CREATE = unicode_to_base64('REPORT_CREATE')
+        tc.find('%s:%s' % (somegroup, REPORT_CREATE))
+        tc.formvalue('revokeform', 'sel', '%s:%s' % (somegroup, REPORT_CREATE))
+        tc.submit(formname='revokeform')
+        tc.notfind('%s:%s' % (somegroup, REPORT_CREATE))
+        tc.notfind(somegroup)
+
+
+class TestAddUserToGroup(FunctionalTestCaseSetup):
     def runTest(self):
         """Add a user to a permissions group"""
         self._tester.go_to_admin("Permissions")
@@ -246,19 +260,19 @@ class TestAddUserToGroup(FunctionalTwillTestCaseSetup):
             tc.formvalue('addsubj', 'sg_subject', 'someuser')
             tc.formvalue('addsubj', 'sg_group', 'authenticated')
             tc.submit()
-            tc.find("The subject someuser was not added to the "
-                    "group authenticated because the group has "
-                    "TICKET_CHGPROP permission and users cannot "
-                    "grant permissions they don't possess.")
+            tc.find("The subject <strong>someuser</strong> was not added "
+                    "to the group <strong>authenticated</strong>. The group "
+                    "has <strong>TICKET_CREATE</strong> permission and you "
+                    "cannot grant permissions you don't possess.")
         finally:
             self._tester.login('admin')
             self._tester.go_to_admin("Permissions")
             tc.formvalue('revokeform', 'sel', revoke_checkbox)
-            tc.submit()
+            tc.submit(formname='revokeform')
             tc.notfind(revoke_checkbox)
 
 
-class TestRemoveUserFromGroup(FunctionalTwillTestCaseSetup):
+class TestRemoveUserFromGroup(FunctionalTestCaseSetup):
     def runTest(self):
         """Remove a user from a permissions group"""
         self._tester.go_to_admin("Permissions")
@@ -267,28 +281,14 @@ class TestRemoveUserFromGroup(FunctionalTwillTestCaseSetup):
         somegroup = unicode_to_base64('somegroup')
         tc.find('%s:%s' % (authenticated, somegroup))
         tc.formvalue('revokeform', 'sel', '%s:%s' % (authenticated, somegroup))
-        tc.submit()
+        tc.submit(formname='revokeform')
         tc.notfind('%s:%s' % (authenticated, somegroup))
 
 
-class TestRemovePermissionGroup(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Remove a permissions group"""
-        self._tester.go_to_admin("Permissions")
-        tc.find('Manage Permissions')
-        somegroup = unicode_to_base64('somegroup')
-        REPORT_CREATE = unicode_to_base64('REPORT_CREATE')
-        tc.find('%s:%s' % (somegroup, REPORT_CREATE))
-        tc.formvalue('revokeform', 'sel', '%s:%s' % (somegroup, REPORT_CREATE))
-        tc.submit()
-        tc.notfind('%s:%s' % (somegroup, REPORT_CREATE))
-        tc.notfind(somegroup)
-
-
-class TestCopyPermissions(FunctionalTwillTestCaseSetup):
+class TestCopyPermissions(FunctionalTestCaseSetup):
     def runTest(self):
         """Tests for the Copy Permissions functionality
-        added in http://trac.edgewall.org/ticket/11099."""
+        added in https://trac.edgewall.org/ticket/11099."""
         checkbox_value = lambda s, p: '%s:%s' % (unicode_to_base64(s),
                                                  unicode_to_base64(p))
         grant_msg = "The subject %s has been granted the permission %s\."
@@ -418,7 +418,7 @@ class TestCopyPermissions(FunctionalTwillTestCaseSetup):
             self._tester.login('admin')
 
 
-class TestPluginSettings(FunctionalTwillTestCaseSetup):
+class TestPluginSettings(FunctionalTestCaseSetup):
     def runTest(self):
         """Check plugin settings."""
         self._tester.go_to_admin("Plugins")
@@ -433,9 +433,9 @@ class TestPluginsAuthorization(AuthorizationTestCaseSetup):
                                 "Manage Plugins")
 
 
-class RegressionTestTicket10752(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket10752(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/10752
+        """Test for regression of https://trac.edgewall.org/ticket/10752
         Permissions on the web admin page should be greyed out when they
         are no longer defined.
         """
@@ -457,9 +457,9 @@ class RegressionTestTicket10752(FunctionalTwillTestCaseSetup):
                    '[^>]+>')
 
 
-class RegressionTestTicket11069(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket11069(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11069
+        """Test for regression of https://trac.edgewall.org/ticket/11069
         The permissions list should only be populated with permissions that
         the user can grant."""
         self._tester.go_to_front()
@@ -484,8 +484,8 @@ class RegressionTestTicket11069(FunctionalTwillTestCaseSetup):
             self._tester.login('admin')
 
 
-class RegressionTestTicket11095(FunctionalTwillTestCaseSetup):
-    """Test for regression of http://trac.edgewall.org/ticket/11095
+class RegressionTestTicket11095(FunctionalTestCaseSetup):
+    """Test for regression of https://trac.edgewall.org/ticket/11095
     The permission is truncated if it overflows the available space (CSS)
     and the full permission name is shown in the title on hover.
     """
@@ -495,8 +495,8 @@ class RegressionTestTicket11095(FunctionalTwillTestCaseSetup):
         tc.find('<span title="WIKI_VIEW">WIKI_VIEW</span>')
 
 
-class RegressionTestTicket11117(FunctionalTwillTestCaseSetup):
-    """Test for regression of http://trac.edgewall.org/ticket/11117
+class RegressionTestTicket11117(FunctionalTestCaseSetup):
+    """Test for regression of https://trac.edgewall.org/ticket/11117
     Hint should be shown on the Basic Settings admin panel when pytz is not
     installed.
     """
@@ -510,8 +510,8 @@ class RegressionTestTicket11117(FunctionalTwillTestCaseSetup):
             tc.notfind(pytz_hint)
 
 
-class RegressionTestTicket11257(FunctionalTwillTestCaseSetup):
-    """Test for regression of http://trac.edgewall.org/ticket/11257
+class RegressionTestTicket11257(FunctionalTestCaseSetup):
+    """Test for regression of https://trac.edgewall.org/ticket/11257
     Hints should be shown on the Basic Settings admin panel when Babel is not
     installed.
     """
@@ -523,7 +523,7 @@ class RegressionTestTicket11257(FunctionalTwillTestCaseSetup):
         catalog_hint = "Message catalogs have not been compiled."
         language_select = '<select name="default_language">'
         disabled_language_select = \
-            '<select name="default_language" disabled="disabled" ' \
+            '<select disabled="disabled" name="default_language" ' \
             'title="Translations are currently unavailable">'
 
         self._tester.go_to_admin("Basic Settings")
@@ -556,9 +556,9 @@ def functionalSuite(suite=None):
     suite.addTest(TestLoggingToFileNormal())
     suite.addTest(TestPermissionsAuthorization())
     suite.addTest(TestCreatePermissionGroup())
+    suite.addTest(TestRemovePermissionGroup())
     suite.addTest(TestAddUserToGroup())
     suite.addTest(TestRemoveUserFromGroup())
-    suite.addTest(TestRemovePermissionGroup())
     suite.addTest(TestCopyPermissions())
     suite.addTest(TestPluginSettings())
     suite.addTest(TestPluginsAuthorization())
