@@ -1,225 +1,27 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import copy
 import os
 import unittest
 
-import trac.tests.compat
 from trac.config import ConfigurationError
 from trac.db.api import DatabaseManager, get_column_names, \
-                        parse_connection_uri, with_transaction
+                        parse_connection_uri
 from trac.db_default import (schema as default_schema,
                              db_version as default_db_version)
 from trac.db.schema import Column, Table
-from trac.test import EnvironmentStub, Mock
-from trac.util.concurrency import ThreadLocal
-
-
-class Connection(object):
-
-    committed = False
-    rolledback = False
-
-    def commit(self):
-        self.committed = True
-
-    def rollback(self):
-        self.rolledback = True
-
-
-class Error(Exception):
-    pass
-
-
-def make_env(get_cnx):
-    from trac.core import ComponentManager
-    return Mock(ComponentManager, components={DatabaseManager:
-             Mock(get_connection=get_cnx,
-                  _transaction_local=ThreadLocal(wdb=None, rdb=None))})
-
-
-class WithTransactionTest(unittest.TestCase):
-
-    def test_successful_transaction(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        @with_transaction(env)
-        def do_transaction(db):
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(db.committed and not db.rolledback)
-
-    def test_failed_transaction(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        try:
-            @with_transaction(env)
-            def do_transaction(db):
-                self.assertTrue(not db.committed and not db.rolledback)
-                raise Error()
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(not db.committed and db.rolledback)
-
-    def test_implicit_nesting_success(self):
-        env = make_env(Connection)
-        dbs = [None, None]
-        @with_transaction(env)
-        def level0(db):
-            dbs[0] = db
-            @with_transaction(env)
-            def level1(db):
-                dbs[1] = db
-                self.assertTrue(not db.committed and not db.rolledback)
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(dbs[0].committed and not dbs[0].rolledback)
-
-    def test_implicit_nesting_failure(self):
-        env = make_env(Connection)
-        dbs = [None, None]
-        try:
-            @with_transaction(env)
-            def level0(db):
-                dbs[0] = db
-                try:
-                    @with_transaction(env)
-                    def level1(db):
-                        dbs[1] = db
-                        self.assertTrue(not db.committed and not db.rolledback)
-                        raise Error()
-                    self.fail()
-                except Error:
-                    self.assertTrue(not db.committed and not db.rolledback)
-                    raise
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(not dbs[0].committed and dbs[0].rolledback)
-
-    def test_explicit_success(self):
-        db = Connection()
-        env = make_env(lambda: None)
-        @with_transaction(env, db)
-        def do_transaction(idb):
-            self.assertTrue(idb is db)
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(not db.committed and not db.rolledback)
-
-    def test_explicit_failure(self):
-        db = Connection()
-        env = make_env(lambda: None)
-        try:
-            @with_transaction(env, db)
-            def do_transaction(idb):
-                self.assertTrue(idb is db)
-                self.assertTrue(not db.committed and not db.rolledback)
-                raise Error()
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(not db.committed and not db.rolledback)
-
-    def test_implicit_in_explicit_success(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        dbs = [None, None]
-        @with_transaction(env, db)
-        def level0(db):
-            dbs[0] = db
-            @with_transaction(env)
-            def level1(db):
-                dbs[1] = db
-                self.assertTrue(not db.committed and not db.rolledback)
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(not dbs[0].committed and not dbs[0].rolledback)
-
-    def test_implicit_in_explicit_failure(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        dbs = [None, None]
-        try:
-            @with_transaction(env, db)
-            def level0(db):
-                dbs[0] = db
-                @with_transaction(env)
-                def level1(db):
-                    dbs[1] = db
-                    self.assertTrue(not db.committed and not db.rolledback)
-                    raise Error()
-                self.fail()
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(not dbs[0].committed and not dbs[0].rolledback)
-
-    def test_explicit_in_implicit_success(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        dbs = [None, None]
-        @with_transaction(env)
-        def level0(db):
-            dbs[0] = db
-            @with_transaction(env, db)
-            def level1(db):
-                dbs[1] = db
-                self.assertTrue(not db.committed and not db.rolledback)
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(dbs[0].committed and not dbs[0].rolledback)
-
-    def test_explicit_in_implicit_failure(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        dbs = [None, None]
-        try:
-            @with_transaction(env)
-            def level0(db):
-                dbs[0] = db
-                @with_transaction(env, db)
-                def level1(db):
-                    dbs[1] = db
-                    self.assertTrue(not db.committed and not db.rolledback)
-                    raise Error()
-                self.fail()
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(not dbs[0].committed and dbs[0].rolledback)
-
-    def test_invalid_nesting(self):
-        env = make_env(Connection)
-        try:
-            @with_transaction(env)
-            def level0(db):
-                @with_transaction(env, Connection())
-                def level1(db):
-                    raise Error()
-                raise Error()
-            raise Error()
-        except AssertionError:
-            pass
+from trac.test import EnvironmentStub, get_dburi
 
 
 class ParseConnectionStringTestCase(unittest.TestCase):
@@ -313,32 +115,43 @@ class StringsTestCase(unittest.TestCase):
         self.env.reset_db()
 
     def test_insert_unicode(self):
-        self.env.db_transaction(
-                "INSERT INTO system (name,value) VALUES (%s,%s)",
-                ('test-unicode', u'ünicöde'))
-        self.assertEqual([(u'ünicöde',)], self.env.db_query(
-            "SELECT value FROM system WHERE name='test-unicode'"))
+        with self.env.db_transaction as db:
+            quoted = db.quote('system')
+            db("INSERT INTO " + quoted + " (name,value) VALUES (%s,%s)",
+               ('test-unicode', 'ünicöde'))
+        self.assertEqual([('ünicöde',)], self.env.db_query(
+            "SELECT value FROM " + quoted + " WHERE name='test-unicode'"))
 
     def test_insert_empty(self):
         from trac.util.text import empty
-        self.env.db_transaction(
-                "INSERT INTO system (name,value) VALUES (%s,%s)",
-                ('test-empty', empty))
-        self.assertEqual([(u'',)], self.env.db_query(
-            "SELECT value FROM system WHERE name='test-empty'"))
+        with self.env.db_transaction as db:
+            quoted = db.quote('system')
+            db("INSERT INTO " + quoted + " (name,value) VALUES (%s,%s)",
+               ('test-empty', empty))
+        self.assertEqual([('',)], self.env.db_query(
+            "SELECT value FROM " + quoted + " WHERE name='test-empty'"))
 
     def test_insert_markup(self):
-        from genshi.core import Markup
-        self.env.db_transaction(
-                "INSERT INTO system (name,value) VALUES (%s,%s)",
-                ('test-markup', Markup(u'<em>märkup</em>')))
-        self.assertEqual([(u'<em>märkup</em>',)], self.env.db_query(
-            "SELECT value FROM system WHERE name='test-markup'"))
+        from trac.util.html import Markup
+        with self.env.db_transaction as db:
+            quoted = db.quote('system')
+            query = "INSERT INTO {} (name,value) VALUES (%s,%s)".format(quoted)
+            db(query, ('test-markup', Markup('<em>märkup</em>')))
+            db.executemany(query, [('test-markup.%d' % i,
+                                    Markup('<em>märkup.%d</em>' % i))
+                                   for i in range(3)])
+        values = dict(self.env.db_query(
+            "SELECT name, value FROM {} WHERE name LIKE %s".format(quoted),
+            ('test-markup%',)))
+        self.assertEqual({'test-markup': '<em>märkup</em>',
+                          'test-markup.0': '<em>märkup.0</em>',
+                          'test-markup.1': '<em>märkup.1</em>',
+                          'test-markup.2': '<em>märkup.2</em>'}, values)
 
     def test_quote(self):
         with self.env.db_query as db:
             cursor = db.cursor()
-            cursor.execute('SELECT 1 AS %s' % \
+            cursor.execute('SELECT 1 AS %s' %
                            db.quote(r'alpha\`\"\'\\beta``gamma""delta'))
             self.assertEqual(r'alpha\`\"\'\\beta``gamma""delta',
                              get_column_names(cursor)[0])
@@ -356,37 +169,38 @@ class StringsTestCase(unittest.TestCase):
                 self.assertEqual(name, get_column_names(cursor)[0])
                 cursor.execute('SELECT %s AS ' + db.quote(name), (42,))
                 self.assertEqual(name, get_column_names(cursor)[0])
-                cursor.executemany("UPDATE system SET value=%s WHERE "
-                                   "1=(SELECT 0 AS " + db.quote(name) + ")",
-                                   [])
-                cursor.executemany("UPDATE system SET value=%s WHERE "
-                                   "1=(SELECT 0 AS " + db.quote(name) + ")",
-                                   [('42',), ('43',)])
+                stmt = """
+                    UPDATE {0} SET value=%s WHERE 1=(SELECT 0 AS {1})
+                    """.format(db.quote('system'), db.quote(name))
+                cursor.executemany(stmt, [])
+                cursor.executemany(stmt, [('42',), ('43',)])
 
         test()
         test(True)
 
     def test_prefix_match_case_sensitive(self):
         with self.env.db_transaction as db:
-            db.executemany("INSERT INTO system (name,value) VALUES (%s,1)",
-                           [('blahblah',), ('BlahBlah',), ('BLAHBLAH',),
-                            (u'BlähBlah',), (u'BlahBläh',)])
+            db.executemany("""
+                INSERT INTO {0} (name,value) VALUES (%s,1)
+                """.format(db.quote('system')),
+                [('blahblah',), ('BlahBlah',), ('BLAHBLAH',), ('BlähBlah',),
+                 ('BlahBläh',)])
 
         with self.env.db_query as db:
             names = sorted(name for name, in db(
-                "SELECT name FROM system WHERE name %s"
-                % db.prefix_match(),
+                "SELECT name FROM {0} WHERE name {1}"
+                .format(db.quote('system'), db.prefix_match()),
                 (db.prefix_match_value('Blah'),)))
         self.assertEqual('BlahBlah', names[0])
-        self.assertEqual(u'BlahBläh', names[1])
+        self.assertEqual('BlahBläh', names[1])
         self.assertEqual(2, len(names))
 
     def test_prefix_match_metachars(self):
         def do_query(prefix):
             with self.env.db_query as db:
-                return [name for name, in db(
-                    "SELECT name FROM system WHERE name %s "
-                    "ORDER BY name" % db.prefix_match(),
+                return [name for name, in db("""
+                    SELECT name FROM {0} WHERE name {1} ORDER BY name
+                    """.format(db.quote('system'), db.prefix_match()),
                     (db.prefix_match_value(prefix),))]
 
         values = ['foo*bar', 'foo*bar!', 'foo?bar', 'foo?bar!',
@@ -394,8 +208,10 @@ class StringsTestCase(unittest.TestCase):
                   'foo%bar', 'foo%bar!', 'foo_bar', 'foo_bar!',
                   'foo/bar', 'foo/bar!', 'fo*ob?ar[fo]ob%ar_fo/obar']
         with self.env.db_transaction as db:
-            db.executemany("INSERT INTO system (name,value) VALUES (%s,1)",
-                           [(value,) for value in values])
+            db.executemany("""
+                INSERT INTO {0} (name,value) VALUES (%s,1)
+                """.format(db.quote('system')),
+                [(value,) for value in values])
 
         self.assertEqual(['foo*bar', 'foo*bar!'], do_query('foo*'))
         self.assertEqual(['foo?bar', 'foo?bar!'], do_query('foo?'))
@@ -463,6 +279,41 @@ class ConnectionTestCase(unittest.TestCase):
         self.assertEqual((1, 'author1', 'comment one'), data[0])
         self.assertEqual((2, 'author2', 'comment two'), data[1])
 
+    def test_rollback_transaction_on_exception(self):
+        """Transaction is rolled back when an exception occurs in the
+        transaction context manager.
+        """
+        insert_sql = "INSERT INTO blog (bid, author) VALUES (42, 'anonymous')"
+        try:
+            with self.env.db_transaction as db:
+                db(insert_sql)
+                db(insert_sql)
+        except self.env.db_exc.IntegrityError:
+            pass
+
+        for _, in self.env.db_query("""
+                SELECT author FROM blog WHERE bid=42
+                """):
+            self.fail("Transaction was not rolled back")
+
+    def test_rollback_nested_transaction_on_exception(self):
+        """Transaction is rolled back when an exception occurs in the
+        inner transaction context manager.
+        """
+        sql = "INSERT INTO blog (bid, author) VALUES (42, 'anonymous')"
+        try:
+            with self.env.db_transaction as db_outer:
+                db_outer(sql)
+                with self.env.db_transaction as db_inner:
+                    db_inner(sql)
+        except self.env.db_exc.IntegrityError:
+            pass
+
+        for _, in self.env.db_query("""
+                SELECT author FROM blog WHERE bid=42
+                """):
+            self.fail("Transaction was not rolled back")
+
     def test_get_last_id(self):
         q = "INSERT INTO report (author) VALUES ('anonymous')"
         with self.env.db_transaction as db:
@@ -537,6 +388,12 @@ class ConnectionTestCase(unittest.TestCase):
                 self.assertEqual(column_names,
                                  db.get_column_names(table.name))
 
+    def test_get_column_names_non_existent_table(self):
+        with self.assertRaises(self.env.db_exc.OperationalError) as cm:
+            self.dbm.get_column_names('blah')
+        self.assertIn(str(cm.exception), ('Table "blah" not found',
+                                          'Table `blah` not found'))
+
 
 class DatabaseManagerTestCase(unittest.TestCase):
 
@@ -549,11 +406,16 @@ class DatabaseManagerTestCase(unittest.TestCase):
 
     def test_destroy_db(self):
         """Database doesn't exist after calling destroy_db."""
-        self.env.db_query("SELECT name FROM system")
+        with self.env.db_query as db:
+            db("SELECT name FROM " + db.quote('system'))
         self.assertIsNotNone(self.dbm._cnx_pool)
         self.dbm.destroy_db()
         self.assertIsNone(self.dbm._cnx_pool)  # No connection pool
-        self.assertFalse(self.dbm.db_exists())
+        scheme, params = parse_connection_uri(get_dburi())
+        if scheme != 'postgres' or params.get('schema', 'public') != 'public':
+            self.assertFalse(self.dbm.db_exists())
+        else:
+            self.assertEqual([], self.dbm.get_table_names())
 
     def test_get_column_names(self):
         """Get column names for the default database."""
@@ -573,6 +435,16 @@ class DatabaseManagerTestCase(unittest.TestCase):
         self.assertEqual(sorted(table.name for table in default_schema),
                          sorted(self.dbm.get_table_names()))
 
+    def test_has_table(self):
+        self.assertIs(True, self.dbm.has_table('system'))
+        self.assertIs(True, self.dbm.has_table('wiki'))
+        self.assertIs(False, self.dbm.has_table('trac'))
+        self.assertIs(False, self.dbm.has_table('blah.blah'))
+
+    def test_no_database_version(self):
+        """False is returned when entry doesn't exist"""
+        self.assertFalse(self.dbm.get_database_version('trac_plugin_version'))
+
     def test_set_default_database_version(self):
         """Set database version for the default entry named
         `database_version`.
@@ -580,6 +452,8 @@ class DatabaseManagerTestCase(unittest.TestCase):
         new_db_version = default_db_version + 1
         self.dbm.set_database_version(new_db_version)
         self.assertEqual(new_db_version, self.dbm.get_database_version())
+        self.assertEqual([('INFO', 'Upgraded database_version from 45 to 46')],
+                         self.env.log_messages)
 
         # Restore the previous version to avoid destroying the database
         # on teardown
@@ -590,12 +464,26 @@ class DatabaseManagerTestCase(unittest.TestCase):
         """Get and set database version for an entry with an
         arbitrary name.
         """
-        name = 'a_trac_plugin_version'
+        name = 'trac_plugin_version'
         db_ver = 1
 
-        self.assertFalse(self.dbm.get_database_version(name))
         self.dbm.set_database_version(db_ver, name)
+        self.assertEqual([], self.env.log_messages)
         self.assertEqual(db_ver, self.dbm.get_database_version(name))
+        # DB update will be skipped when new value equals database version
+        self.dbm.set_database_version(db_ver, name)
+        self.assertEqual([], self.env.log_messages)
+
+    def test_get_sequence_names(self):
+        sequence_names = []
+        if self.dbm.connection_uri.startswith('postgres'):
+            for table in default_schema:
+                for column in table.columns:
+                    if column.name == 'id' and column.auto_increment:
+                        sequence_names.append(table.name)
+            sequence_names.sort()
+
+        self.assertEqual(sequence_names, self.dbm.get_sequence_names())
 
 
 class ModifyTableTestCase(unittest.TestCase):
@@ -623,9 +511,14 @@ class ModifyTableTestCase(unittest.TestCase):
         self.new_schema = copy.deepcopy([self.schema[0], self.schema[2]])
         self.new_schema[0].remove_columns(('col2',))
         self.new_schema[1].columns.append(Column('col4'))
+        self.new_schema.append(
+            Table('table4')[
+                Column('col1'),
+            ]
+        )
 
     def tearDown(self):
-        self.dbm.drop_tables(self.schema)
+        self.dbm.drop_tables(['table1', 'table2', 'table3', 'table4'])
         self.env.reset_db()
 
     def _insert_data(self):
@@ -664,6 +557,12 @@ class ModifyTableTestCase(unittest.TestCase):
         self.assertEqual((10,), data[0])
         self.assertEqual((13,), data[1])
 
+    def test_drop_columns_non_existent_table(self):
+        with self.assertRaises(self.env.db_exc.OperationalError) as cm:
+            self.dbm.drop_columns('blah', ('col1',))
+        self.assertIn(str(cm.exception), ('Table "blah" not found',
+                                          'Table `blah` not found'))
+
     def test_upgrade_tables_have_new_schema(self):
         """The upgraded tables have the new schema."""
         self.dbm.upgrade_tables(self.new_schema)
@@ -712,7 +611,6 @@ def test_suite():
     suite.addTest(unittest.makeSuite(ParseConnectionStringTestCase))
     suite.addTest(unittest.makeSuite(StringsTestCase))
     suite.addTest(unittest.makeSuite(ConnectionTestCase))
-    suite.addTest(unittest.makeSuite(WithTransactionTest))
     suite.addTest(unittest.makeSuite(DatabaseManagerTestCase))
     suite.addTest(unittest.makeSuite(ModifyTableTestCase))
     return suite
