@@ -1,30 +1,27 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2018 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
+import collections
 import doctest
+import io
 import unittest
-from StringIO import StringIO
 
-from genshi import Stream, Namespace
-from genshi.core import Attrs, TEXT, START, END
-from genshi.input import HTMLParser
-
-import trac.tests.compat
 from trac.core import Component, implements
 from trac.test import EnvironmentStub, MockRequest
 from trac.mimeview import api
-from trac.mimeview.api import get_mimetype, IContentConverter, Mimeview, \
-                              _group_lines
+from trac.mimeview.api import (IContentConverter, Mimeview, RenderingContext,
+                               get_mimetype)
+from trac.resource import Resource
 from trac.web.api import RequestDone
 
 
@@ -36,7 +33,7 @@ class GetMimeTypeTestCase(unittest.TestCase):
 
     def test_from_suffix_using_mimetypes(self):
         accepted = ('image/png', 'image/x-png')
-        self.assertTrue(get_mimetype('doc/trac_logo.png', None) in accepted)
+        self.assertIn(get_mimetype('doc/trac_logo.png', None), accepted)
 
     def test_from_content_using_CONTENT_RE(self):
         self.assertEqual('text/x-python',
@@ -104,111 +101,6 @@ class MimeviewTestCase(unittest.TestCase):
         self.assertEqual(Converter1(self.env), conversions[1].converter)
         self.assertEqual(Converter2(self.env), conversions[2].converter)
 
-class GroupLinesTestCase(unittest.TestCase):
-
-    def test_empty_stream(self):
-        # FIXME: this currently fails
-        lines = list(_group_lines([]))
-        self.assertEqual(len(lines), 0)
-
-    def test_text_only_stream(self):
-        input = [(TEXT, "test", (None, -1, -1))]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), 1)
-        self.assertIsInstance(lines[0], Stream)
-        self.assertEqual(lines[0].events, input)
-
-    def test_text_only_stream2(self):
-        input = [(TEXT, "test\n", (None, -1, -1))]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), 1)
-        self.assertIsInstance(lines[0], Stream)
-        self.assertEqual(lines[0].events, [(TEXT, "test", (None, -1, -1))])
-
-    def test_simplespan(self):
-        input = HTMLParser(StringIO(u"<span>test</span>"), encoding=None)
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), 1)
-        self.assertIsInstance(lines[0], Stream)
-        for (a, b) in zip(lines[0], input):
-            self.assertEqual(a, b)
-
-    def test_empty_text_stream(self):
-        """
-        http://trac.edgewall.org/ticket/4336
-        """
-        input = [(TEXT, "", (None, -1, -1))]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), 0)
-
-    def test_newline_stream(self):
-        input = [(TEXT, "\n", (None, -1, -1))]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), 1)
-
-    def test_newline_stream2(self):
-        input = [(TEXT, "\n\n\n", (None, -1, -1))]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), 3)
-
-    def test_empty_text_in_span(self):
-        """
-        http://trac.edgewall.org/ticket/4336
-        """
-        ns = Namespace('http://www.w3.org/1999/xhtml')
-        input = [(START, (ns.span, Attrs([])), (None, -1, -1)),
-                 (TEXT, "", (None, -1, -1)),
-                 (END, ns.span, (None, -1, -1)),
-                ]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), 0)
-
-    def test_newline(self):
-        """
-        If the text element does not end with a newline, it's not properly
-        closed.
-        """
-        input = HTMLParser(StringIO(u'<span class="c">a\nb</span>'),
-            encoding=None)
-        expected = ['<span class="c">a</span>',
-                    '<span class="c">b</span>',
-                   ]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), len(expected))
-        for a, b in zip(lines, expected):
-            self.assertEqual(a.render('html'), b)
-
-    def test_newline2(self):
-        """
-        Same as test_newline above, but make sure it behaves properly wrt
-        the trailing \\n, especially given it's inside an element.
-        """
-        input = HTMLParser(StringIO(u'<span class="c">a\nb\n</span>'),
-            encoding=None)
-        expected = ['<span class="c">a</span>',
-                    '<span class="c">b</span>',
-                   ]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), len(expected))
-        for a, b in zip(lines, expected):
-            self.assertEqual(a.render('html'), b)
-
-    def test_multinewline(self):
-        """
-        ditto.
-        """
-        input = HTMLParser(StringIO(u'<span class="c">\n\n\na</span>'),
-            encoding=None)
-        expected = ['<span class="c"></span>',
-                    '<span class="c"></span>',
-                    '<span class="c"></span>',
-                    '<span class="c">a</span>',
-                   ]
-        lines = list(_group_lines(input))
-        self.assertEqual(len(lines), len(expected))
-        for a, b in zip(lines, expected):
-            self.assertEqual(a.render('html'), b)
-
 
 class TestMimeviewConverter(Component):
 
@@ -223,18 +115,18 @@ class TestMimeviewConverter(Component):
     def convert_content(self, req, mimetype, content, key):
         if content == 'iterable-bytes':
             def fn_bytes():
-                for idx in xrange(256):
-                    yield 'b' * 256
+                for idx in range(256):
+                    yield b'c' * 256
             return fn_bytes(), 'text/plain'
         if content == 'iterable-unicode':
             def fn_unicode():
-                for idx in xrange(0x10000):
-                    yield u'ü'
+                for idx in range(0x10000):
+                    yield 'ü'
             return fn_unicode(), 'text/plain'
         if content == 'bytes':
-            return 'a' * 0x10000, 'text/plain'
+            return b'a' * 0x10000, 'text/plain'
         if content == 'unicode':
-            return u'Ü' * 0x10000, 'text/plain'
+            return 'Ü' * 0x10000, 'text/plain'
 
 
 class MimeviewConverterTestCase(unittest.TestCase):
@@ -252,38 +144,41 @@ class MimeviewConverterTestCase(unittest.TestCase):
         output = mimeview.convert_content(MockRequest(self.env),
                                           self.in_mimetype,
                                           content, 'text', iterable=iterable)
+        content, ctype, ext = output
         if iterable:
-            self.assertNotIn(type(output[0]), (str, unicode))
-            self.assertEqual(expected, ''.join(output[0]))
+            self.assertNotIsInstance(content, (str, bytes))
+            self.assertIsInstance(content, collections.abc.Iterable)
+            join = ''.join if isinstance(expected, str) else b''.join
+            self.assertEqual(expected, join(content))
         else:
-            self.assertEqual(type(expected), type(output[0]))
-            self.assertEqual(expected, output[0])
-        self.assertEqual('text/plain', output[1])
-        self.assertEqual('txt', output[2])
+            self.assertEqual(type(expected), type(content))
+            self.assertEqual(expected, content)
+        self.assertEqual('text/plain', ctype)
+        self.assertEqual('txt', ext)
 
     def test_convert_content_iterable_bytes(self):
-        self._test_convert_content('b' * 0x10000, 'iterable-bytes', False)
+        self._test_convert_content(b'c' * 0x10000, 'iterable-bytes', False)
 
     def test_convert_content_iterable_unicode(self):
-        self._test_convert_content(u'ü' * 0x10000, 'iterable-unicode', False)
+        self._test_convert_content('ü' * 0x10000, 'iterable-unicode', False)
 
     def test_convert_content_bytes(self):
-        self._test_convert_content('a' * 0x10000, 'bytes', False)
+        self._test_convert_content(b'a' * 0x10000, 'bytes', False)
 
     def test_convert_content_unicode(self):
-        self._test_convert_content(u'Ü' * 0x10000, 'unicode', False)
+        self._test_convert_content('Ü' * 0x10000, 'unicode', False)
 
     def test_convert_content_iterable_bytes_iterable(self):
-        self._test_convert_content('b' * 0x10000, 'iterable-bytes', True)
+        self._test_convert_content(b'c' * 0x10000, 'iterable-bytes', True)
 
     def test_convert_content_iterable_unicode_iterable(self):
-        self._test_convert_content(u'ü' * 0x10000, 'iterable-unicode', True)
+        self._test_convert_content('ü' * 0x10000, 'iterable-unicode', True)
 
     def test_convert_content_bytes_iterable(self):
-        self._test_convert_content('a' * 0x10000, 'bytes', True)
+        self._test_convert_content(b'a' * 0x10000, 'bytes', True)
 
     def test_convert_content_unicode_iterable(self):
-        self._test_convert_content(u'Ü' * 0x10000, 'unicode', True)
+        self._test_convert_content('Ü' * 0x10000, 'unicode', True)
 
     def _test_send_converted(self, expected, content, use_chunked_encoding):
         self.env.config.set('trac', 'use_chunked_encoding',
@@ -292,49 +187,72 @@ class MimeviewConverterTestCase(unittest.TestCase):
         req = MockRequest(self.env)
         self.assertRaises(RequestDone, mimeview.send_converted, req,
                           self.in_mimetype, content, 'text')
-        result = req.response_sent.getvalue()
+        sent_bytes = req.response_sent.getvalue()  # always a bytes instance
+        expected_bytes = expected.encode('utf-8') \
+                         if isinstance(expected, str) else expected
         if use_chunked_encoding:
             self.assertNotIn('Content-Length', req.headers_sent)
         else:
             self.assertIn('Content-Length', req.headers_sent)
-            self.assertEqual(str(len(expected)),
+            self.assertEqual(str(len(expected_bytes)),
                              req.headers_sent['Content-Length'])
         self.assertEqual('text/plain', req.headers_sent['Content-Type'])
-        self.assertEqual(set(expected), set(result))
-        self.assertEqual(expected, result)
+        self.assertEqual(expected_bytes, sent_bytes)
 
     def test_send_converted_iterable_bytes(self):
-        self._test_send_converted('b' * 0x10000, 'iterable-bytes', False)
+        self._test_send_converted(b'c' * 0x10000, 'iterable-bytes', False)
 
     def test_send_converted_iterable_unicode(self):
         self._test_send_converted('ü' * 0x10000, 'iterable-unicode', False)
 
     def test_send_converted_bytes(self):
-        self._test_send_converted('a' * 0x10000, 'bytes', False)
+        self._test_send_converted(b'a' * 0x10000, 'bytes', False)
 
     def test_send_converted_unicode(self):
         self._test_send_converted('Ü' * 0x10000, 'unicode', False)
 
     def test_send_converted_iterable_bytes_chunked(self):
-        self._test_send_converted('b' * 0x10000, 'iterable-bytes', True)
+        self._test_send_converted(b'c' * 0x10000, 'iterable-bytes', True)
 
     def test_send_converted_iterable_unicode_chunked(self):
         self._test_send_converted('ü' * 0x10000, 'iterable-unicode', True)
 
     def test_send_converted_bytes_chunked(self):
-        self._test_send_converted('a' * 0x10000, 'bytes', True)
+        self._test_send_converted(b'a' * 0x10000, 'bytes', True)
 
     def test_send_converted_unicode_chunked(self):
         self._test_send_converted('Ü' * 0x10000, 'unicode', True)
 
 
+class MimeviewRenderTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+
+    def test_plain_text_content(self):
+        """Render simple plain text content."""
+        mimeview = Mimeview(self.env)
+        req = MockRequest(self.env)
+        context = RenderingContext(Resource('wiki', 'readme.txt'))
+        context.req = req
+        content = io.BytesIO(b"""\
+Some text.
+""")
+
+        rendered = mimeview.render(context, 'text/plain', content)
+
+        self.assertEqual('<div class="code"><pre>Some text.\n</pre></div>',
+                         str(rendered))
+
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(doctest.DocTestSuite(api))
     suite.addTest(unittest.makeSuite(GetMimeTypeTestCase))
     suite.addTest(unittest.makeSuite(MimeviewTestCase))
-    suite.addTest(unittest.makeSuite(GroupLinesTestCase))
     suite.addTest(unittest.makeSuite(MimeviewConverterTestCase))
+    suite.addTest(unittest.makeSuite(MimeviewRenderTestCase))
     return suite
 
 if __name__ == '__main__':
