@@ -1,18 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015 Edgewall Software
+# Copyright (C) 2015-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
-from __future__ import absolute_import
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 try:
@@ -21,17 +19,18 @@ try:
 except ImportError:
     have_pygments = False
 
-from trac.tests.functional import FunctionalTwillTestCaseSetup, tc
+from trac.tests.functional import FunctionalTestCaseSetup, tc
 
 
-class TestSyntaxHighlightingPreferences(FunctionalTwillTestCaseSetup):
+class TestSyntaxHighlightingPreferences(FunctionalTestCaseSetup):
     def runTest(self):
         """Set preferences for syntax highlighting."""
         self._tester.go_to_preferences("Syntax Highlighting")
-        tc.find('<option value="trac" selected="selected">')
+        tc.find('<option value="">Default: Trac</option>')
+        tc.notfind('<option selected="selected" value="')
         tc.formvalue('userprefs', 'style', 'Emacs')
         tc.submit()
-        tc.find('<option value="emacs" selected="selected">')
+        tc.find('<option selected="selected" value="emacs">')
 
 
 def functionalSuite(suite=None):
