@@ -1,22 +1,19 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2013 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import unittest
 
-from genshi.core import Stream
-from genshi.input import HTMLParser, XML
-
 from trac.mimeview.api import Mimeview
 from trac.mimeview.patch import PatchRenderer
 from trac.test import EnvironmentStub, MockRequest
@@ -30,17 +27,23 @@ class PatchRendererTestCase(unittest.TestCase):
         req = MockRequest(env)
         self.context = web_context(req)
         self.patch = Mimeview(env).renderers[0]
-        patch_html = open(os.path.join(os.path.split(__file__)[0],
-                                       'patch.html'))
-        self.patch_html = Stream(list(HTMLParser(patch_html, encoding='utf-8')))
+        html_file = os.path.join(os.path.dirname(__file__), 'patch.data')
+        self.patch_html = {}
+        testcase = []
+        with open(html_file, 'r', encoding='utf-8') as f:
+            for line in f.readlines():
+                if line.startswith('#'):
+                    self.patch_html[line[1:].strip()] = testcase = []
+                else:
+                    testcase.append(line.rstrip())
 
     def _expected(self, expected_id):
-        return self.patch_html.select('//div[@id="%s"]/div' % expected_id)
+        return self.patch_html[expected_id]
 
     def _test(self, expected_id, result):
-        expected = self._expected(expected_id).render(encoding='utf-8')
-        result = XML(result.render(encoding='utf-8')).render(encoding='utf-8')
-        expected, result = expected.splitlines(), result.splitlines()
+        expected = self._expected(expected_id)
+        result = result.splitlines()
+        self.maxDiff = None
         for exp, res in zip(expected, result):
             self.assertEqual(exp, res)
         self.assertEqual(len(expected), len(result))
@@ -94,6 +97,7 @@ class PatchRendererTestCase(unittest.TestCase):
 """)
         self.assertTrue(result)
         self._test('no_newline_in_changed', result)
+
     def test_diff_to_hdf_expandtabs(self):
         """Regression test related to #4557"""
         changes = self.patch._diff_to_hdf(
