@@ -1,21 +1,22 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2017 Edgewall Software
+# Copyright (C) 2017-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
+# history and logs, available at https://trac.edgewall.org/log/.
+import textwrap
 import unittest
 from xml.etree import ElementTree
 
 from trac.test import EnvironmentStub, MockRequest
 from trac.mimeview.txtl import TextileRenderer, has_textile
+from trac.util.html import Markup
 from trac.web.chrome import web_context
 
 
@@ -33,7 +34,7 @@ class TextileRendererTestCase(unittest.TestCase):
 
     def _render(self, text):
         result = self.renderer.render(self.context, 'textile', text)
-        self.assertEqual(unicode, type(result))
+        self.assertEqual(Markup, type(result))
         return result
 
     def _parse_xml(self, source):
@@ -44,13 +45,13 @@ class TextileRendererTestCase(unittest.TestCase):
             self.assertEqual('Textile', name)
 
     def test_image(self):
-        result = self._render(u"""\
-!https://example.org/foo.png! uníćode
-!//example.net/foo.png!       uníćode
-!/path/to/foo.png!            uníćode
-!foo.png!                     uníćode
-!data:image/png,foo!          uníćode
-""")
+        result = self._render(textwrap.dedent("""\
+            !https://example.org/foo.png! uníćode
+            !//example.net/foo.png!       uníćode
+            !/path/to/foo.png!            uníćode
+            !foo.png!                     uníćode
+            !data:image/png,foo!          uníćode
+            """))
         tree = self._parse_xml(result)
         elements = tree.findall('img')
         self.assertEqual(elements[0].get('src'), 'https://example.org/foo.png')
@@ -65,14 +66,14 @@ class TextileRendererTestCase(unittest.TestCase):
         self.assertEqual(elements[4].get('crossorigin'), None)
 
     def test_style(self):
-        result = self._render(u"""\
-*{background:url(https://example.org/foo.png)}uníćode*
-*{background:url(//example.net/foo.png)      }uníćode*
-*{background:url(/path/to/foo.png)           }uníćode*
-*{background:url(./foo.png)                  }uníćode*
-*{background:url(foo.png)                    }uníćode*
-*{background:url(data:image/png,foo)         }uníćode*
-""")
+        result = self._render(textwrap.dedent("""\
+            *{background:url(https://example.org/foo.png)}uníćode*
+            *{background:url(//example.net/foo.png)      }uníćode*
+            *{background:url(/path/to/foo.png)           }uníćode*
+            *{background:url(./foo.png)                  }uníćode*
+            *{background:url(foo.png)                    }uníćode*
+            *{background:url(data:image/png,foo)         }uníćode*
+            """))
         self.assertNotIn('url(https://example.org/foo.png)', result)
         self.assertNotIn('url(//example.net/foo.png)', result)
         self.assertIn('url(/path/to/foo.png)', result)
@@ -81,15 +82,14 @@ class TextileRendererTestCase(unittest.TestCase):
         self.assertIn('url(data:image/png,foo)', result)
 
     def test_html(self):
-        result = self._render(u"""\
-<a href="ftp://example.org/">unsafe</a>
-<img src="//example.org/foo.png" />
-<span style="background-image:url(http://example.org/foo.png)">unsafe</span>
-""")
+        result = self._render(textwrap.dedent("""\
+            <a href="ftp://example.org/">unsafe</a>
+            <img src="//example.org/foo.png" />
+            <span style="background-image:url(http://example.org/foo.png)">unsafe</span>
+            """))
         self.assertNotIn('href="ftp://', result)
-        self.assertNotIn('<img src="//example.org/foo.png" />', result)
-        self.assertIn('<img src="//example.org/foo.png" '
-                      'crossorigin="anonymous" />', result)
+        self.assertIn(
+            '<img crossorigin="anonymous" src="//example.org/foo.png"', result)
         self.assertNotIn('url(http://example.org/foo.png)', result)
         self.assertIn('<span>unsafe</span>', result)
 
