@@ -1,20 +1,19 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015 Edgewall Software
+# Copyright (C) 2015-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
-from email import message_from_string
+from email import message_from_bytes
 
-import trac.tests.compat
 from trac.core import Component, implements
 from trac.notification.api import (
     IEmailAddressResolver, IEmailSender, INotificationFormatter,
@@ -38,7 +37,7 @@ class TestEmailSender(Component):
 
     def send(self, from_addr, recipients, message):
         self.history.append((from_addr, recipients,
-                             message_from_string(message)))
+                             message_from_bytes(message)))
 
 
 class TestFormatter(Component):
@@ -57,11 +56,11 @@ class TestFormatter(Component):
         if style == 'text/plain':
             if 'raise-text-plain' in text:
                 raise ValueError()
-            return unicode(text)
+            return str(text)
         if style == 'text/html':
             if 'raise-text-html' in text:
                 raise ValueError()
-            return u'<p>%s</p>' % escape(text)
+            return '<p>%s</p>' % escape(text)
 
 
 class TestSubscriber(Component):
@@ -126,7 +125,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         with self.env.db_transaction:
             self._add_session('foo', email='foo@example.org')
             self._add_session('bar', email='bar@example.org',
-                              name=u"Bäŕ's name")
+                              name="Bäŕ's name")
             self._add_session('baz', name='Baz')
             self._add_session('qux', tz='UTC')
             self._add_session('corge', email='corge-mail')
@@ -144,7 +143,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         session = DetachedSession(self.env, sid)
         if values is not None:
             attrs.update(values)
-        for name, value in attrs.iteritems():
+        for name, value in attrs.items():
             session[name] = value
         session.save()
 
@@ -169,7 +168,8 @@ class EmailDistributorTestCase(unittest.TestCase):
         payload = list(message.get_payload())
         self.assertEqual([content_type],
                          [p.get_content_type() for p in payload])
-        self.assertEqual([body], [p.get_payload() for p in payload])
+        self.assertEqual([body], [p.get_payload().rstrip('\n')
+                                  for p in payload])
 
     def _assert_alternative_mail(self, message, body_plain, body_html):
         self.assertNotIn('Bcc', message)
@@ -181,7 +181,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(['text/plain', 'text/html'],
                          [p.get_content_type() for p in alternative])
         self.assertEqual([body_plain, body_html],
-                         [p.get_payload() for p in alternative])
+                         [p.get_payload().rstrip('\n') for p in alternative])
 
     def test_plain(self):
         with self.env.db_transaction:
@@ -196,9 +196,9 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(1, len(history))
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual(set(('foo@example.org', 'bar@example.org',
-                              'baz@example.net', 'qux@example.net',
-                              'cc@example.org', 'bcc@example.org')),
+        self.assertEqual({'foo@example.org', 'bar@example.org',
+                          'baz@example.net', 'qux@example.net',
+                          'cc@example.org', 'bcc@example.org'},
                          set(recipients))
         self._assert_mail(message, 'text/plain', 'blah')
 
@@ -214,10 +214,11 @@ class EmailDistributorTestCase(unittest.TestCase):
             if 'foo@example.org' in recipients:
                 self.assertEqual('trac@example.org', from_addr)
                 self.assertEqual(['foo@example.org'], recipients)
-                self._assert_alternative_mail(message, 'blah', '<p>blah</p>')
+                self._assert_alternative_mail(message, 'blah',
+                                              '<p>blah</p>')
             if 'cc@example.org' in recipients:
                 self.assertEqual('trac@example.org', from_addr)
-                self.assertEqual(set(('cc@example.org', 'bcc@example.org')),
+                self.assertEqual({'cc@example.org', 'bcc@example.org'},
                                  set(recipients))
                 self._assert_mail(message, 'text/plain', 'blah')
 
@@ -232,13 +233,14 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(2, len(history))
         for from_addr, recipients, message in history:
             if 'foo@example.org' in recipients:
-                self.assertEqual(set(('foo@example.org', 'cc@example.org',
-                                      'bcc@example.org')),
-                                 set(recipients))
+                self.assertEqual(
+                    {'foo@example.org', 'cc@example.org', 'bcc@example.org'},
+                    set(recipients))
                 self._assert_mail(message, 'text/plain', 'blah')
             if 'bar@example.org' in recipients:
                 self.assertEqual(['bar@example.org'], recipients)
-                self._assert_alternative_mail(message, 'blah', '<p>blah</p>')
+                self._assert_alternative_mail(message, 'blah',
+                                              '<p>blah</p>')
 
     def test_formats_in_session_and_tracini(self):
         self.config.set('notification', 'smtp_always_cc', 'bar,quux')
@@ -273,7 +275,8 @@ class EmailDistributorTestCase(unittest.TestCase):
             if 'qux@example.org' in recipients:
                 self.assertEqual(['quux@example.org', 'qux@example.org'],
                                  recipients)
-                self._assert_alternative_mail(message, 'blah', '<p>blah</p>')
+                self._assert_alternative_mail(message, 'blah',
+                                              '<p>blah</p>')
 
     def test_broken_plain_formatter(self):
         with self.env.db_transaction:
@@ -301,8 +304,8 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(1, len(history))
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual(set(('foo@example.org', 'bar@example.org',
-                              'cc@example.org', 'bcc@example.org')),
+        self.assertEqual({'foo@example.org', 'bar@example.org',
+                          'cc@example.org', 'bcc@example.org'},
                          set(recipients))
         self._assert_mail(message, 'text/plain', 'raise-text-html')
 
@@ -327,11 +330,11 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(1, len(history))
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual(set(('foo@example.org', 'bar@example.org',
-                              'cc@example.org', 'bcc@example.org')),
+        self.assertEqual({'foo@example.org', 'bar@example.org',
+                          'cc@example.org', 'bcc@example.org'},
                          set(recipients))
         self.assertEqual('cc@example.org, foo@example.org', message['Cc'])
-        self.assertEqual(None, message['Bcc'])
+        self.assertIsNone(message['Bcc'])
         self._assert_mail(message, 'text/plain', 'blah')
 
     def test_from_author_disabled(self):
@@ -344,7 +347,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertNotEqual([], history)
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual('"My Project" <trac@example.org>', message['From'])
+        self.assertEqual('My Project <trac@example.org>', message['From'])
         self.assertEqual(1, len(history))
 
         self._notify_event('blah', author=None)
@@ -352,7 +355,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertNotEqual([], history)
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual('"My Project" <trac@example.org>', message['From'])
+        self.assertEqual('My Project <trac@example.org>', message['From'])
         self.assertEqual(1, len(history))
 
         self.env.config.set('notification', 'smtp_from_name', 'Trac')
@@ -361,7 +364,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertNotEqual([], history)
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual('"Trac" <trac@example.org>', message['From'])
+        self.assertEqual('Trac <trac@example.org>', message['From'])
         self.assertEqual(1, len(history))
 
     def test_from_author_enabled(self):
@@ -375,7 +378,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertNotEqual([], history)
         from_addr, recipients, message = history[0]
         self.assertEqual('bar@example.org', from_addr)
-        self.assertEqual('"=?utf-8?b?QsOkxZUncyBuYW1l?=" <bar@example.org>',
+        self.assertEqual('=?utf-8?b?QsOkxZUncw==?= name <bar@example.org>',
                          message['From'])
         self.assertEqual(1, len(history))
 
@@ -392,7 +395,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertNotEqual([], history)
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual('"My Project" <trac@example.org>', message['From'])
+        self.assertEqual('My Project <trac@example.org>', message['From'])
         self.assertEqual(1, len(history))
 
         self.env.config.set('notification', 'smtp_from_name', 'Trac')
@@ -401,7 +404,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertNotEqual([], history)
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual('"Trac" <trac@example.org>', message['From'])
+        self.assertEqual('Trac <trac@example.org>', message['From'])
         self.assertEqual(1, len(history))
 
     def test_ignore_domains(self):
@@ -462,7 +465,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         history = self._test_without_domain(use_short_addr='enabled')
         from_addr, recipients, message = history[0]
         self.assertEqual('from-trac', from_addr)
-        self.assertEqual('"My Project" <from-trac>', message['From'])
+        self.assertEqual('My Project <from-trac>', message['From'])
         self._assert_equal_sets(['qux', 'cc@example.org', 'bcc1@example.org',
                                  'bcc2', 'foo@example.org', 'baz',
                                  'corge-mail'], recipients)
@@ -473,7 +476,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         history = self._test_without_domain(smtp_default_domain='example.com')
         from_addr, recipients, message = history[0]
         self.assertEqual('from-trac@example.com', from_addr)
-        self.assertEqual('"My Project" <from-trac@example.com>',
+        self.assertEqual('My Project <from-trac@example.com>',
                          message['From'])
         self._assert_equal_sets(['qux@example.com', 'cc@example.org',
                                  'bcc1@example.org', 'bcc2@example.com',
@@ -499,17 +502,165 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(1, len(history))
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual('"My Project" <trac@example.org>', message['From'])
-        self.assertEqual(set(['foo@example.com', 'foo@bar.example.org',
-                              'baz@example.com', 'cc@example.org',
-                              'bcc@example.org']), set(recipients))
+        self.assertEqual('My Project <trac@example.org>', message['From'])
+        self.assertEqual({'foo@example.com', 'foo@bar.example.org',
+                          'baz@example.com', 'cc@example.org',
+                          'bcc@example.org'}, set(recipients))
         self._assert_equal_sets(['cc@example.org'],
                                 self._cclist(message['Cc']))
 
 
+class RecipientMatcherTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+        self.config = self.env.config
+
+    def tearDown(self):
+        self.env.reset_db()
+
+    def _add_session(self, sid, values=None, **attrs):
+        session = DetachedSession(self.env, sid)
+        session['(dummy)'] = 'x'
+        if values is not None:
+            attrs.update(values)
+        for name, value in attrs.items():
+            session[name] = value
+        session.save()
+
+    def test_match_recipient_empty(self):
+        matcher = RecipientMatcher(self.env)
+        self.assertEqual(None, matcher.match_recipient(None))
+        self.assertEqual(None, matcher.match_recipient(''))
+
+    def test_match_recipient_anonymous(self):
+        matcher = RecipientMatcher(self.env)
+        self.assertEqual(None, matcher.match_recipient('anonymous'))
+
+    def test_match_recipient_address(self):
+        matcher = RecipientMatcher(self.env)
+        expected = (None, 0, 'user@example.org')
+        self.assertEqual(expected, matcher.match_recipient('user@example.org'))
+        self.assertEqual(expected,
+                         matcher.match_recipient('<user@example.org>'))
+        self.assertEqual(expected, matcher.match_recipient(
+            'Name name <user@example.org>'))
+        self.assertEqual(expected, matcher.match_recipient(
+            'Námë ńämé <user@example.org>'))
+
+    def test_match_recipient_admit_domains(self):
+        self.config.set('notification', 'admit_domains', 'LOCALDOMAIN')
+        with self.env.db_transaction:
+            self._add_session('user1', email='user1@localhost')
+            self._add_session('user2', email='user2@localdomain')
+            self._add_session('user3', email='user3@example.org')
+            self._add_session('user4@localhost')
+            self._add_session('user5@localdomain')
+            self._add_session('user6@example.org')
+            self._add_session('user7@localhost', email='user7@example.org')
+            self._add_session('user8@localdomain', email='user8@localhost')
+            self._add_session('user9@example.org', email='user9@localdomain')
+        matcher = RecipientMatcher(self.env)
+
+        # authenticated users
+        self.assertEqual(None, matcher.match_recipient('user1'))
+        self.assertEqual(('user2', 1, 'user2@localdomain'),
+                         matcher.match_recipient('user2'))
+        self.assertEqual(('user3', 1, 'user3@example.org'),
+                         matcher.match_recipient('user3'))
+        self.assertEqual(None, matcher.match_recipient('user4@localhost'))
+        self.assertEqual(('user5@localdomain', 1, 'user5@localdomain'),
+                         matcher.match_recipient('user5@localdomain'))
+        self.assertEqual(('user6@example.org', 1, 'user6@example.org'),
+                         matcher.match_recipient('user6@example.org'))
+        self.assertEqual(('user7@localhost', 1, 'user7@example.org'),
+                         matcher.match_recipient('user7@localhost'))
+        self.assertEqual(None, matcher.match_recipient('user8@localdomain'))
+        self.assertEqual(('user9@example.org', 1, 'user9@localdomain'),
+                         matcher.match_recipient('user9@example.org'))
+        # anonymous users
+        self.assertEqual(None, matcher.match_recipient('foobar'))
+        self.assertEqual(None, matcher.match_recipient('anon@localhost'))
+        self.assertEqual((None, 0, 'anon@localdomain'),
+                         matcher.match_recipient('anon@localdomain'))
+        self.assertEqual((None, 0, 'anon@example.org'),
+                         matcher.match_recipient('anon@example.org'))
+
+    def test_match_recipient_use_short_addr(self):
+        self.config.set('notification', 'use_short_addr', 'enabled')
+        with self.env.db_transaction:
+            self._add_session('user1')
+            self._add_session('user2', email='user2-email')
+            self._add_session('user3', email='user3@example.org')
+            self._add_session('user4@example.org', email='user4')
+        matcher = RecipientMatcher(self.env)
+
+        self.assertEqual(('user1', 1, 'user1'),
+                         matcher.match_recipient('user1'))
+        self.assertEqual(('user2', 1, 'user2-email'),
+                         matcher.match_recipient('user2'))
+        self.assertEqual(('user3', 1, 'user3@example.org'),
+                         matcher.match_recipient('user3'))
+        self.assertEqual(('user4@example.org', 1, 'user4'),
+                         matcher.match_recipient('user4@example.org'))
+        self.assertEqual((None, 0, 'user9'), matcher.match_recipient('user9'))
+
+    def test_match_recipient_smtp_default_domain(self):
+        self.config.set('notification', 'smtp_default_domain',
+                        'default.example.net')
+        with self.env.db_transaction:
+            self._add_session('user1')
+            self._add_session('user2', email='user2-email')
+            self._add_session('user3', email='user3@example.org')
+            self._add_session('user4@example.org', email='user4')
+            self._add_session('user5@example.org')
+        matcher = RecipientMatcher(self.env)
+
+        self.assertEqual(('user1', 1, 'user1@default.example.net'),
+                         matcher.match_recipient('user1'))
+        self.assertEqual(('user2', 1, 'user2-email@default.example.net'),
+                         matcher.match_recipient('user2'))
+        self.assertEqual(('user3', 1, 'user3@example.org'),
+                         matcher.match_recipient('user3'))
+        self.assertEqual(('user4@example.org', 1, 'user4@default.example.net'),
+                         matcher.match_recipient('user4@example.org'))
+        self.assertEqual(('user5@example.org', 1, 'user5@example.org'),
+                         matcher.match_recipient('user5@example.org'))
+        self.assertEqual((None, 0, 'user9@default.example.net'),
+                         matcher.match_recipient('user9'))
+
+    def test_match_recipient_ignore_domains(self):
+        self.config.set('notification', 'ignore_domains',
+                        'example.net,example.com')
+        with self.env.db_transaction:
+            self._add_session('user1', email='user1@example.org')
+            self._add_session('user2', email='user2@example.com')
+            self._add_session('user3', email='user3@EXAMPLE.COM')
+            self._add_session('user4@example.org')
+            self._add_session('user5@example.com')
+            self._add_session('user6@EXAMPLE.COM')
+        matcher = RecipientMatcher(self.env)
+
+        # authenticated users
+        self.assertEqual(('user1', 1, 'user1@example.org'),
+                         matcher.match_recipient('user1'))
+        self.assertEqual(None, matcher.match_recipient('user2'))
+        self.assertEqual(None, matcher.match_recipient('user3'))
+        self.assertEqual(('user4@example.org', 1, 'user4@example.org'),
+                         matcher.match_recipient('user4@example.org'))
+        self.assertEqual(None, matcher.match_recipient('user5@example.com'))
+        self.assertEqual(None, matcher.match_recipient('user6@EXAMPLE.COM'))
+        # anonymous users
+        self.assertEqual((None, 0, 'anon@example.org'),
+                         matcher.match_recipient('anon@example.org'))
+        self.assertEqual(None, matcher.match_recipient('anon@example.com'))
+        self.assertEqual(None, matcher.match_recipient('anon@EXAMPLE.COM'))
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(EmailDistributorTestCase))
+    suite.addTest(unittest.makeSuite(RecipientMatcherTestCase))
     return suite
 
 
