@@ -1,26 +1,26 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2018 Edgewall Software
+# Copyright (C) 2008-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import re
 import unittest
 
-from trac.tests.functional import FunctionalTwillTestCaseSetup, \
+from trac.tests.functional import FunctionalTestCaseSetup, \
                                   internal_error, tc
 
 
 #TODO: split this into multiple smaller testcases
-class TestPreferences(FunctionalTwillTestCaseSetup):
+class TestPreferences(FunctionalTestCaseSetup):
     def runTest(self):
         """Set preferences for admin user"""
         self._tester.go_to_preferences()
@@ -48,14 +48,14 @@ class TestPreferences(FunctionalTwillTestCaseSetup):
             tc.find('Your preferences have been saved.')
 
 
-class TestDefaultHandler(FunctionalTwillTestCaseSetup):
+class TestDefaultHandler(FunctionalTestCaseSetup):
     def runTest(self):
         """Set default handler."""
 
         # Project default_handler is selected.
         self._tester.go_to_preferences("User Interface")
         tc.notfind(r'<option[^>]+selected="selected"')
-        tc.find("Default \(WikiModule\)")
+        tc.find("Default: WikiModule")
 
         try:
             # Project default handler still selected after value is changed.
@@ -65,7 +65,7 @@ class TestDefaultHandler(FunctionalTwillTestCaseSetup):
                                      'SearchModule')
             self._tester.go_to_preferences("User Interface")
             tc.notfind('<option[^>]+selected="selected"')
-            tc.find("Default \(SearchModule\)")
+            tc.find("Default: SearchModule")
             tc.notfind(hint)
 
             # Project default handler still selected after module is disabled.
@@ -74,7 +74,7 @@ class TestDefaultHandler(FunctionalTwillTestCaseSetup):
             self._tester.go_to_preferences("User Interface")
             try:
                 tc.notfind('<option[^>]+selected="selected"')
-                tc.find(r"Default \(SearchModule\)")
+                tc.find(r"Default: SearchModule")
                 tc.find(hint)
             finally:
                 self._testenv.remove_config('components', component)
@@ -95,10 +95,10 @@ class TestDefaultHandler(FunctionalTwillTestCaseSetup):
         tc.submit()
         tc.find("Your preferences have been saved\.")
         tc.notfind(r'<option[^>]+selected="selected"')
-        tc.find("Default \(WikiModule\)")
+        tc.find("Default: WikiModule")
 
 
-class RegressionTestRev5785(FunctionalTwillTestCaseSetup):
+class RegressionTestRev5785(FunctionalTestCaseSetup):
     def runTest(self):
         """Test for regression of the fix in r5785"""
         self._tester.go_to_preferences()
@@ -107,23 +107,23 @@ class RegressionTestRev5785(FunctionalTwillTestCaseSetup):
         tc.follow('Login')
 
 
-class RegressionTestTicket5765(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket5765(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/5765
+        """Test for regression of https://trac.edgewall.org/ticket/5765
         Unable to turn off 'Enable access keys' in Preferences
         """
-        self._tester.go_to_preferences("Keyboard Shortcuts")
+        self._tester.go_to_preferences("User Interface")
         tc.formvalue('userprefs', 'accesskeys', True)
-        tc.submit()
+        tc.submit(formname='userprefs')
         tc.find('name="accesskeys".*checked="checked"')
         tc.formvalue('userprefs', 'accesskeys', False)
-        tc.submit()
+        tc.submit(formname='userprefs')
         tc.notfind('name="accesskeys".*checked="checked"')
 
 
-class RegressionTestTicket11319(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket11319(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11319
+        """Test for regression of https://trac.edgewall.org/ticket/11319
         Only alphanumeric characters can be used for session key in advanced
         panel.
         """
@@ -143,9 +143,9 @@ class RegressionTestTicket11319(FunctionalTwillTestCaseSetup):
             self._tester.login('admin')
 
 
-class RegressionTestTicket11337(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket11337(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11337
+        """Test for regression of https://trac.edgewall.org/ticket/11337
         The language select will be disabled if Babel is not installed and a
         hint will be shown. The text of the hint is dependent on whether the
         user has TRAC_ADMIN and the message catalogs have been compiled.
@@ -154,8 +154,8 @@ class RegressionTestTicket11337(FunctionalTwillTestCaseSetup):
 
         babel_hint = "Install Babel for extended language support."
         catalog_hint = "Message catalogs have not been compiled."
-        nonadmin_hint = r"Please contact your\s+Trac administrator\s+" \
-                        r"to enable existing translations."
+        nonadmin_hint = r'\s+'.join("Please contact your Trac administrator "
+                                    "to enable existing translations".split())
         language_select = '<select id="language" name="language">'
         disabled_language_select = \
             '<select id="language" name="language" disabled="disabled" ' \
@@ -198,13 +198,12 @@ class RegressionTestTicket11337(FunctionalTwillTestCaseSetup):
             self._tester.login('admin')
 
 
-class RegressionTestTicket11515(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket11515(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11515
+        """Test for regression of https://trac.edgewall.org/ticket/11515
         Show a notice message with new language setting after it is changed.
         """
-        from trac.util.translation import Locale, has_babel, \
-                                          get_available_locales
+        from trac.util.translation import has_babel, get_available_locales
         from pkg_resources import resource_exists, resource_filename
 
         if not has_babel:
@@ -239,8 +238,8 @@ class RegressionTestTicket11515(FunctionalTwillTestCaseSetup):
             tc.find('Your preferences have been saved')
 
 
-class RegressionTestTicket11531(FunctionalTwillTestCaseSetup):
-    """Test for regression of http://trac.edgewall.org/ticket/11531
+class RegressionTestTicket11531(FunctionalTestCaseSetup):
+    """Test for regression of https://trac.edgewall.org/ticket/11531
     PreferencesModule can be set as the default_handler."""
     def runTest(self):
         default_handler = self._testenv.get_config('trac', 'default_handler')
