@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2018 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 
@@ -17,6 +17,7 @@ import trac.tests.wikisyntax
 import trac.ticket.tests.wikisyntax
 import trac.versioncontrol.web_ui.tests.wikisyntax
 import trac.web.tests.wikisyntax
+import trac.wiki.tests.macros
 import trac.wiki.tests.wikisyntax
 import trac.wiki.tests.formatter
 
