@@ -1,35 +1,28 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2018 Edgewall Software
+# Copyright (C) 2008-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import random
+import re
 import uuid
 
 try:
-    all_words = [x.strip() for x in open('/usr/share/dict/words').readlines()
-                           if x.strip().isalpha()]
+    isalpha = re.compile(r'[A-Za-z]+$').match
+    with open('/usr/share/dict/words', 'r', encoding='utf-8') as f:
+        all_words = list(filter(isalpha, map(lambda v: v.strip(), f)))
+        del f
 except IOError:
-    all_words = [
-        'one',
-        'two',
-        'three',
-        'four',
-        'five',
-        'six',
-        'seven',
-        'eight',
-        'nine',
-        'ten',
-    ]
+    with open('trac/tests/random_words.txt', 'r', encoding='utf-8') as f:
+        all_words = f.read().split()
 
 
 def random_word(min_length=1):
@@ -47,7 +40,7 @@ def random_unique_camel():
     """Returns a unique camelcase word pair"""
     while True:
         camel = random_word(2).title() + random_word(2).title()
-        if not camel in _random_unique_camels:
+        if camel not in _random_unique_camels:
             break
     _random_unique_camels.append(camel)
     return camel
@@ -69,12 +62,12 @@ def random_sentence(word_count=None):
 def random_paragraph(sentence_count=None):
     if sentence_count is None:
         sentence_count = random.randint(1, 10)
-    sentences = [random_sentence(random.randint(2, 15)) for x in range(sentence_count)]
-    return '  '.join(sentences)
+    return '  '.join(random_sentence(random.randint(2, 15))
+                     for x in range(sentence_count))
 
 
 def random_page(paragraph_count=None):
     if paragraph_count is None:
         paragraph_count = random.randint(1, 10)
-    paragraphs = [random_paragraph(random.randint(1, 5)) for x in range(paragraph_count)]
-    return '\r\n\r\n'.join(paragraphs)
+    return '\r\n\r\n'.join(random_paragraph(random.randint(1, 5))
+                           for x in range(paragraph_count))
