@@ -1,42 +1,42 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2018 Edgewall Software
+# Copyright (C) 2008-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import re
-import time
+import socket
 import unittest
 
-from trac.tests.functional import FunctionalTwillTestCaseSetup, \
+from trac.tests.functional import FunctionalTestCaseSetup, \
                                   internal_error, tc
 from trac.util import create_file
 
 
-class TestAttachmentNonexistentParent(FunctionalTwillTestCaseSetup):
+class TestAttachmentNonexistentParent(FunctionalTestCaseSetup):
     def runTest(self):
         """TracError should be raised when navigating to the attachment
         page for a nonexistent resource."""
         self._tester.go_to_wiki('NonexistentPage')
-        tc.find("The page <strong>NonexistentPage</strong> does not exist. "
-                "You can create it here.")
+        tc.find(r"The page[ \n]+<strong>NonexistentPage</strong>[ \n]+"
+                r"does not exist. You can create it here.")
         tc.find(r"\bCreate this page\b")
 
         tc.go(self._tester.url + '/attachment/wiki/NonexistentPage')
-        tc.find('<h1>Trac Error</h1>\s+<p class="message">'
-                'Parent resource NonexistentPage doesn\'t exist</p>')
+        tc.find(r'<h1>Trac Error</h1>\s+<p class="message">'
+                r"Parent resource NonexistentPage doesn&#39;t exist</p>")
 
 
-class TestAboutPage(FunctionalTwillTestCaseSetup):
+class TestAboutPage(FunctionalTestCaseSetup):
     def runTest(self):
         """Validate the About page."""
         tc.follow(r"\bAbout Trac\b")
@@ -45,15 +45,13 @@ class TestAboutPage(FunctionalTwillTestCaseSetup):
         tc.find(r"<h2>Configuration</h2>")
 
 
-class TestErrorPage(FunctionalTwillTestCaseSetup):
+class TestErrorPage(FunctionalTestCaseSetup):
     """Validate the error page.
     Defects reported to trac-hacks should use the Component defined in the
     plugin's URL (#11434).
     """
     def runTest(self):
         env = self._testenv.get_trac_environment()
-        env.config.set('components', 'RaiseExceptionPlugin.*', 'enabled')
-        env.config.save()
         create_file(os.path.join(env.plugins_dir, 'RaiseExceptionPlugin.py'),
 """\
 from trac.core import Component, TracError, implements
@@ -128,122 +126,65 @@ class RaiseExceptionPlugin(Component):
             env.config.set('components', 'RaiseExceptionPlugin.*', 'disabled')
 
 
-class RegressionTestRev6017(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket3833(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of the plugin reload fix in r6017"""
-        # Setup the DeleteTicket plugin
+        """Test for regression of https://trac.edgewall.org/ticket/3833"""
         env = self._testenv.get_trac_environment()
-        plugin = open(os.path.join(self._testenv.trac_src,
-                                   'sample-plugins', 'workflow',
-                                   'DeleteTicket.py')).read()
-        plugin_path = os.path.join(env.plugins_dir, 'DeleteTicket.py')
-        open(plugin_path, 'w').write(plugin)
-        prevconfig = env.config.get('ticket', 'workflow')
-        env.config.set('ticket', 'workflow',
-                       prevconfig + ',DeleteTicketActionController')
+        trac_log = os.path.join(env.log_dir, 'trac.log')
+
+        def read_log_file(offset):
+            with open(trac_log, 'rb') as fd:
+                fd.seek(offset)
+                return str(fd.read(), 'utf-8')
+
+        # Verify that file logging is enabled as info level.
+        log_size = os.path.getsize(trac_log)
+        env.log.info("RegressionTestTicket3833 info1")
+        info1 = "RegressionTestTicket3833 info1"
+        log_content = read_log_file(log_size)
+        self.assertEqual('INFO', env.config.get('logging', 'log_level'))
+        self.assertIn(info1, log_content),
+
+        # Verify that info level is not logged at warning level.
+        env.config.set('logging', 'log_level', 'WARNING')
         env.config.save()
-        env = self._testenv.get_trac_environment() # reloads the environment
-
-        loaded_components = env.compmgr.__metaclass__._components
-        delete_plugins = [c for c in loaded_components
-                          if 'DeleteTicketActionController' in c.__name__]
-        try:
-            self.assertEqual(len(delete_plugins), 1,
-                             "Plugin loaded more than once.")
-
-        finally:
-            # Remove the DeleteTicket plugin
-            env.config.set('ticket', 'workflow', prevconfig)
-            env.config.save()
-            for ext in ('py', 'pyc', 'pyo'):
-                filename = os.path.join(env.plugins_dir,
-                                        'DeleteTicket.%s' % ext)
-                if os.path.exists(filename):
-                    os.unlink(filename)
-
-
-class RegressionTestTicket3833a(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/3833 a"""
         env = self._testenv.get_trac_environment()
-        # Assume the logging is already set to debug.
-        traclogfile = open(os.path.join(env.log_dir, 'trac.log'))
-        # Seek to the end of file so we only look at new log output
-        traclogfile.seek(0, 2)
-
-        # Verify that logging is on initially
-        env.log.debug("RegressionTestTicket3833 debug1")
-        debug1 = traclogfile.read()
-        self.assertNotEqual(debug1.find("RegressionTestTicket3833 debug1"), -1,
-                            'Logging off when it should have been on.\n%r'
-                            % debug1)
-
-
-class RegressionTestTicket3833b(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/3833 b"""
-        # Turn logging off, try to log something, and verify that it does
-        # not show up.
-        env = self._testenv.get_trac_environment()
-        traclogfile = open(os.path.join(env.log_dir, 'trac.log'))
-        # Seek to the end of file so we only look at new log output
-        traclogfile.seek(0, 2)
+        log_size = os.path.getsize(trac_log)
+        info2 = "RegressionTestTicket3833 info2"
+        warn2 = "RegressionTestTicket3833 warn2"
+        env.log.info(info2)
+        env.log.warning(warn2)
+        log_content = read_log_file(log_size)
+        self.assertEqual('WARNING', env.config.get('logging', 'log_level'))
+        self.assertNotIn(info2, log_content)
+        self.assertIn(warn2, log_content)
 
+        # Revert to info level logging.
         env.config.set('logging', 'log_level', 'INFO')
         env.config.save()
         env = self._testenv.get_trac_environment()
-        env.log.debug("RegressionTestTicket3833 debug2")
-        env.log.info("RegressionTestTicket3833 info2")
-        debug2 = traclogfile.read()
-        self.assertNotEqual(debug2.find("RegressionTestTicket3833 info2"), -1,
-                            'Logging at info failed.\n%r' % debug2)
-        self.assertEqual(debug2.find("RegressionTestTicket3833 debug2"), -1,
-                         'Logging still on when it should have been off.\n%r'
-                         % debug2)
+        log_size = os.path.getsize(trac_log)
+        info3 = "RegressionTestTicket3833 info3"
+        warn3 = "RegressionTestTicket3833 warn3"
+        env.log.info(info3)
+        env.log.warning(warn3)
+        log_content = read_log_file(log_size)
+        self.assertEqual('INFO', env.config.get('logging', 'log_level'))
+        self.assertIn(info3, log_content)
+        self.assertIn(warn3, log_content)
 
 
-class RegressionTestTicket3833c(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket5572(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/3833 c"""
-        # Turn logging back on, try to log something, and verify that it
-        # does show up.
-        env = self._testenv.get_trac_environment()
-        traclogfile = open(os.path.join(env.log_dir, 'trac.log'))
-        # Seek to the end of file so we only look at new log output
-        traclogfile.seek(0, 2)
-
-        env.config.set('logging', 'log_level', 'DEBUG')
-        time.sleep(2)
-        env.config.save()
-        #time.sleep(2)
-        env = self._testenv.get_trac_environment()
-        #time.sleep(2)
-        env.log.debug("RegressionTestTicket3833 debug3")
-        env.log.info("RegressionTestTicket3833 info3")
-        #time.sleep(2)
-        debug3 = traclogfile.read()
-        message = ''
-        success = debug3.find("RegressionTestTicket3833 debug3") != -1
-        if not success:
-            # Ok, the testcase failed, but we really need logging enabled.
-            env.log.debug("RegressionTestTicket3833 fixup3")
-            fixup3 = traclogfile.read()
-            message = 'Logging still off when it should have been on.\n' \
-                      '%r\n%r' % (debug3, fixup3)
-        self.assertTrue(success, message)
-
-
-class RegressionTestTicket5572(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/5572"""
+        """Test for regression of https://trac.edgewall.org/ticket/5572"""
         # TODO: this ticket (implemented in r6011) adds a new feature to
         # make the progress bar more configurable.  We need to test this
         # new configurability.
 
 
-class RegressionTestTicket7209(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket7209(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/7209"""
+        """Test for regression of https://trac.edgewall.org/ticket/7209"""
         ticketid = self._tester.create_ticket()
         self._tester.create_ticket()
         self._tester.add_comment(ticketid)
@@ -273,9 +214,9 @@ class RegressionTestTicket7209(FunctionalTwillTestCaseSetup):
         tc.notfind('Second Attachment')
 
 
-class RegressionTestTicket9880(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket9880(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/9880
+        """Test for regression of https://trac.edgewall.org/ticket/9880
 
         Upload of a file which the browsers associates a Content-Type
         of multipart/related (e.g. an .mht file) should succeed.
@@ -291,23 +232,55 @@ See also http://bugs.python.org/issue15564.
 """)
 
 
-class RegressionTestTicket3663(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket3663(FunctionalTestCaseSetup):
     def runTest(self):
         """Regression test for non-UTF-8 PATH_INFO (#3663)
 
         Verify that URLs not encoded with UTF-8 are reported as invalid.
         """
+        def fetch(uri):
+            # http.client module disallows non-ascii characters in request
+            # line. Instead, use directly socket module.
+            cookie = '; '.join('%s=%s' % (c['name'], c['value'])
+                               for c in tc.get_cookies()).encode('utf-8')
+            req = (b'GET %s HTTP/1.0\r\n'
+                   b'Host: %s:%d\r\n'
+                   b'Cookie: %s\r\n'
+                   b'Connection: close\r\n'
+                   b'\r\n' % (uri, b'127.0.0.1', self._testenv.port, cookie))
+            resp = []
+            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
+                s.connect(('127.0.0.1', self._testenv.port))
+                s.send(req)
+                while True:
+                    chunk = s.recv(4096)
+                    if not chunk:
+                        break
+                    resp.append(chunk)
+            resp = b''.join(resp)
+            if not resp:
+                raise RuntimeError('No response')
+            match = re.match(b'HTTP\/[0-9]\.[0-9] +([0-9]{3}) +.*\r\n', resp)
+            if not match:
+                raise RuntimeError('No status line: %r' % resp)
+            status = int(match.group(1))
+            pos = resp.find(b'\r\n\r\n', len(match.group(0)))
+            if pos == -1:
+                raise RuntimeError('Missing CRLFCRLF in response: %r' % resp)
+            body = resp[pos:]
+            return status, body
+
         # invalid PATH_INFO
-        self._tester.go_to_wiki(u'été'.encode('latin1'))
-        tc.code(404)
-        tc.find('Invalid URL encoding')
+        status, body = fetch('/wiki/été'.encode('latin1'))
+        self.assertEqual(404, status)
+        self.assertIn(b'Invalid URL encoding', body)
         # invalid SCRIPT_NAME
-        tc.go(u'été'.encode('latin1'))
-        tc.code(404)
-        tc.find('Invalid URL encoding')
+        status, body = fetch('/été'.encode('latin1'))
+        self.assertEqual(404, status)
+        self.assertIn(b'Invalid URL encoding', body)
 
 
-class RegressionTestTicket6318(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket6318(FunctionalTestCaseSetup):
     def runTest(self):
         """Regression test for non-ascii usernames (#6318)
         """
@@ -320,13 +293,12 @@ class RegressionTestTicket6318(FunctionalTwillTestCaseSetup):
         self._tester.logout()
         try:
             # also test a regular ascii user name
-            self._testenv.adduser(u'user')
-            self._tester.login(u'user')
+            self._tester.login('user')
             self._tester.go_to_front()
             self._tester.logout()
             # now test utf-8 user name
-            self._testenv.adduser(u'joé')
-            self._tester.login(u'joé')
+            self._testenv.adduser('joé')
+            self._tester.login('joé')
             self._tester.go_to_front()
             # when failed to retrieve session, FakeSession() and FakePerm()
             # are used and the req.perm has no permissions.
@@ -336,11 +308,11 @@ class RegressionTestTicket6318(FunctionalTwillTestCaseSetup):
             # finally restore expected 'admin' login
             self._tester.login('admin')
         finally:
-            self._testenv.deluser(u'joé')
+            self._testenv.deluser('joé')
 
 
-class RegressionTestTicket11434(FunctionalTwillTestCaseSetup):
-    """Test for regression of http://trac.edgewall.org/ticket/11434
+class RegressionTestTicket11434(FunctionalTestCaseSetup):
+    """Test for regression of https://trac.edgewall.org/ticket/11434
     Defects reported to trac-hacks should use the Component defined in the
     plugin's URL.
     """
@@ -378,31 +350,31 @@ class RaiseExceptionPlugin(Component):
             env.config.set('components', 'RaiseExceptionPlugin.*', 'disabled')
 
 
-class RegressionTestTicket11503a(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket11503a(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11503 a"""
+        """Test for regression of https://trac.edgewall.org/ticket/11503 a"""
         base = self._tester.url
 
         tc.go(base + '/notf%C5%91und/')
         tc.notfind(internal_error)
-        tc.url(re.escape(base + '/notf%C5%91und') + r'\Z')
+        tc.url(base + '/notf%C5%91und', regexp=False)
 
         tc.go(base + '/notf%C5%91und/?type=def%C3%A9ct')
         tc.notfind(internal_error)
-        tc.url(re.escape(base + '/notf%C5%91und?type=def%C3%A9ct') + r'\Z')
+        tc.url(base + '/notf%C5%91und?type=def%C3%A9ct', regexp=False)
 
         tc.go(base + '/notf%C5%91und/%252F/?type=%252F')
         tc.notfind(internal_error)
-        tc.url(re.escape(base + '/notf%C5%91und/%252F?type=%252F') + r'\Z')
+        tc.url(base + '/notf%C5%91und/%252F?type=%252F', regexp=False)
 
 
-class RegressionTestTicket11503b(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket11503b(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11503 b"""
+        """Test for regression of https://trac.edgewall.org/ticket/11503 b"""
         env = self._testenv.get_trac_environment()
         try:
             env.config.set('mainnav', 'wiki.href',
-                           u'/wiki/SändBõx?action=history&blah=%252F')
+                           '/wiki/SändBõx?action=history&blah=%252F')
             env.config.save()
             # reloads the environment
             env = self._testenv.get_trac_environment()
@@ -423,10 +395,7 @@ def functionalSuite(suite=None):
     suite.addTest(TestAttachmentNonexistentParent())
     suite.addTest(TestAboutPage())
     suite.addTest(TestErrorPage())
-    suite.addTest(RegressionTestRev6017())
-    suite.addTest(RegressionTestTicket3833a())
-    suite.addTest(RegressionTestTicket3833b())
-    suite.addTest(RegressionTestTicket3833c())
+    suite.addTest(RegressionTestTicket3833())
     suite.addTest(RegressionTestTicket5572())
     suite.addTest(RegressionTestTicket7209())
     suite.addTest(RegressionTestTicket9880())
