@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2020 Edgewall Software
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Include a basic SMTP server, based on L. Smithson
 # (lsmithson@open-networks.co.uk) extensible Python SMTP Server
@@ -27,6 +27,8 @@ import socket
 import string
 import threading
 import unittest
+from contextlib import closing
+from email.header import decode_header as _email_decode_header
 
 from trac.config import ConfigurationError
 from trac.notification import SendmailEmailSender, SmtpEmailSender
@@ -35,7 +37,6 @@ from trac.test import EnvironmentStub
 LF = '\n'
 CR = '\r'
 SMTP_TEST_PORT = 7000 + os.getpid() % 1000
-email_re = re.compile(r'([\w\d_\.\-])+\@(([\w\d\-])+\.)+([\w\d]{2,4})+')
 header_re = re.compile(r'^=\?(?P<charset>[\w\d\-]+)\?(?P<code>[qb])\?(?P<value>.*)\?=$')
 
 
@@ -82,8 +83,8 @@ def strip_address(address):
     Strip the leading & trailing <> from an address.  Handy for
     getting FROM: addresses.
     """
-    start = string.index(address, '<') + 1
-    end = string.index(address, '>')
+    start = address.find(b'<') + 1
+    end = address.find(b'>')
     return address[start:end]
 
 
@@ -92,9 +93,9 @@ def split_to(address):
     Return 'address' as undressed (host, fulladdress) tuple.
     Handy for use with TO: addresses.
     """
-    start = string.index(address, '<') + 1
-    sep = string.index(address, '@') + 1
-    end = string.index(address, '>')
+    start = address.find('<') + 1
+    sep = address.find('@') + 1
+    end = address.find('>')
     return address[sep:end], address[start:end]
 
 
@@ -128,9 +129,9 @@ class SMTPServerEngine(object):
         SMTPServerInterface methods on the object passed at
         construction time.
         """
-        self.socket.send("220 Welcome to Trac notification test server\r\n")
+        self.socket.send(b"220 Welcome to Trac notification test server\r\n")
         while 1:
-            data = ''
+            data = b''
             complete_line = 0
             # Make sure an entire line is received before handing off
             # to the state engine. Thanks to John Hall for pointing
@@ -140,7 +141,7 @@ class SMTPServerEngine(object):
                     lump = self.socket.recv(1024)
                     if lump:
                         data += lump
-                        if len(data) >= 2 and data[-2:] == '\r\n':
+                        if len(data) >= 2 and data[-2:] == b'\r\n':
                             complete_line = 1
                             if self.state != SMTPServerEngine.ST_DATA:
                                 rsp, keep = self.do_command(data)
@@ -148,7 +149,7 @@ class SMTPServerEngine(object):
                                 rsp = self.do_data(data)
                                 if rsp is None:
                                     continue
-                            self.socket.send(rsp + "\r\n")
+                            self.socket.send(rsp + b"\r\n")
                             if keep == 0:
                                 self.socket.close()
                                 return
@@ -160,46 +161,45 @@ class SMTPServerEngine(object):
 
     def do_command(self, data):
         """Process a single SMTP Command"""
-        cmd = data[0:4]
-        cmd = string.upper(cmd)
+        cmd = data[0:4].upper()
         keep = 1
         rv = None
-        if cmd == "HELO":
+        if cmd == b"HELO":
             self.state = SMTPServerEngine.ST_HELO
             rv = self.impl.helo(data[5:])
-        elif cmd == "RSET":
+        elif cmd == b"RSET":
             rv = self.impl.reset(data[5:])
-            self.data_accum = ""
+            self.data_accum = b""
             self.state = SMTPServerEngine.ST_INIT
-        elif cmd == "NOOP":
+        elif cmd == b"NOOP":
             pass
-        elif cmd == "QUIT":
+        elif cmd == b"QUIT":
             rv = self.impl.quit(data[5:])
             keep = 0
-        elif cmd == "MAIL":
+        elif cmd == b"MAIL":
             if self.state != SMTPServerEngine.ST_HELO:
-                return "503 Bad command sequence", 1
+                return b"503 Bad command sequence", 1
             self.state = SMTPServerEngine.ST_MAIL
             rv = self.impl.mail_from(data[5:])
-        elif cmd == "RCPT":
+        elif cmd == b"RCPT":
             if (self.state != SMTPServerEngine.ST_MAIL) and \
                (self.state != SMTPServerEngine.ST_RCPT):
                 return "503 Bad command sequence", 1
             self.state = SMTPServerEngine.ST_RCPT
             rv = self.impl.rcpt_to(data[5:])
-        elif cmd == "DATA":
+        elif cmd == b"DATA":
             if self.state != SMTPServerEngine.ST_RCPT:
-                return "503 Bad command sequence", 1
+                return b"503 Bad command sequence", 1
             self.state = SMTPServerEngine.ST_DATA
-            self.data_accum = ""
-            return "354 OK, Enter data, terminated with a \\r\\n.\\r\\n", 1
+            self.data_accum = b""
+            return b"354 OK, Enter data, terminated with a \\r\\n.\\r\\n", 1
         else:
-            return "505 Eh? WTF was that?", 1
+            return b"505 Eh? WTF was that?", 1
 
         if rv:
             return rv, keep
         else:
-            return "250 OK", keep
+            return b"250 OK", keep
 
     def do_data(self, data):
         """
@@ -207,14 +207,14 @@ class SMTPServerEngine(object):
         terminator is found.
         """
         self.data_accum = self.data_accum + data
-        if len(self.data_accum) > 4 and self.data_accum[-5:] == '\r\n.\r\n':
+        if len(self.data_accum) > 4 and self.data_accum[-5:] == b'\r\n.\r\n':
             self.data_accum = self.data_accum[:-5]
             rv = self.impl.data(self.data_accum)
             self.state = SMTPServerEngine.ST_HELO
             if rv:
                 return rv
             else:
-                return "250 OK - Data and terminator. found"
+                return b"250 OK - Data and terminator. found"
         else:
             return None
 
@@ -229,6 +229,7 @@ class SMTPServer(object):
 
     def __init__(self, host, port):
         self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self._socket.settimeout(0.25)
         self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self._socket.bind((host, port))
         self._socket_service = None
@@ -237,6 +238,8 @@ class SMTPServer(object):
         while self._resume:
             try:
                 nsd = self._socket.accept()
+            except socket.timeout:
+                continue
             except socket.error:
                 return
             self._socket_service = nsd[0]
@@ -279,16 +282,17 @@ class SMTPServerStore(SMTPServerInterface):
         self.reset(None)
 
     def mail_from(self, args):
-        if args.lower().startswith('from:'):
-            self.sender = strip_address(args[5:].replace('\r\n', '').strip())
+        if args.lower().startswith(b'from:'):
+            self.sender = str(strip_address(args[5:].replace(b'\r\n', b'')
+                                            .strip()), 'utf-8')
 
     def rcpt_to(self, args):
-        if args.lower().startswith('to:'):
-            rcpt = args[3:].replace('\r\n', '').strip()
-            self.recipients.append(strip_address(rcpt))
+        if args.lower().startswith(b'to:'):
+            rcpt = args[3:].replace(b'\r\n', b'').strip()
+            self.recipients.append(str(strip_address(rcpt), 'utf-8'))
 
     def data(self, args):
-        self.message = args
+        self.message = str(args, 'utf-8')
 
     def quit(self, args):
         pass
@@ -323,14 +327,6 @@ class SMTPThreadedServer(threading.Thread):
     def stop(self):
         # run from the main thread
         self.server.stop()
-        # send a message to make the SMTP server quit gracefully
-        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        try:
-            s.connect(('127.0.0.1', self.port))
-            r = s.send("QUIT\r\n")
-        except socket.error:
-            pass
-        s.close()
         # wait for the SMTP server to complete (for up to 2 secs)
         self.join(2.0)
         # clean up the SMTP server (and force quit if needed)
@@ -349,45 +345,29 @@ class SMTPThreadedServer(threading.Thread):
         self.store.reset(None)
 
 
-def smtp_address(fulladdr):
-    mo = email_re.search(fulladdr)
-    if mo:
-        return mo.group(0)
-    if start >= 0:
-        return fulladdr[start+1:-1]
-    return fulladdr
-
-
 def decode_header(header):
     """ Decode a MIME-encoded header value """
-    mo = header_re.match(header)
-    # header does not seem to be MIME-encoded
-    if not mo:
-        return header
-    # attempts to decode the header,
-    # following the specified MIME encoding and charset
-    try:
-        encoding = mo.group('code').lower()
-        if encoding == 'q':
-            val = quopri.decodestring(mo.group('value'), header=True)
-        elif encoding == 'b':
-            val = base64.decodestring(mo.group('value'))
-        else:
-            raise AssertionError("unsupported encoding: %s" % encoding)
-        header = unicode(val, mo.group('charset'))
-    except Exception as e:
-        raise AssertionError(e)
-    return header
+    l = []
+    for s, charset in _email_decode_header(header):
+        if charset:
+            s = str(s, charset)
+        elif isinstance(s, bytes):
+            s = str(s, 'utf-8')
+        l.append(s)
+    return ''.join(l)
 
 
-def parse_smtp_message(msg):
+def parse_smtp_message(msg, decode=True):
     """ Split a SMTP message into its headers and body.
         Returns a (headers, body) tuple
         We do not use the email/MIME Python facilities here
         as they may accept invalid RFC822 data, or data we do not
         want to support nor generate """
+    if msg is None:
+        raise AssertionError('msg is None')
     headers = {}
     lh = None
+    lval = None
     body = None
     # last line does not contain the final line ending
     msg += '\r\n'
@@ -418,16 +398,18 @@ def parse_smtp_message(msg):
                         # unexpected multiline
                         raise AssertionError("unexpected folded line: %s"
                                              % line)
-                    val = decode_header(line.strip(' \t'))
+                    val = line.strip(' \t')
+                    if lval:
+                        val = ' ' + val
                     # appends the current line to the previous one
                     if not isinstance(headers[lh], tuple):
                         headers[lh] += val
                     else:
-                        headers[lh][-1] = headers[lh][-1] + val
+                        headers[lh][-1] += val
                 else:
                     # splits header name from value
                     (h, v) = line.split(':', 1)
-                    val = decode_header(v.strip())
+                    val = v.strip()
                     if h in headers:
                         if isinstance(headers[h], tuple):
                             headers[h] += val
@@ -437,6 +419,16 @@ def parse_smtp_message(msg):
                         headers[h] = val
                     # stores the last header (for multi-line headers)
                     lh = h
+                lval = val
+    # decode headers
+    if decode:
+        for h in headers:
+            v = headers[h]
+            if isinstance(v, tuple):
+                v = tuple(map(decode_header, v))
+            else:
+                v = decode_header(v)
+            headers[h] = v
     # returns the headers and the message body
     return headers, body
 
