@@ -1,23 +1,23 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2018 Edgewall Software
+# Copyright (C) 2007-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import doctest
 import unittest
 
-from genshi.builder import tag
 from trac import resource
 from trac.core import Component, implements
 from trac.test import EnvironmentStub, MockRequest
+from trac.util.html import tag
 from trac.web.chrome import web_context
 
 
@@ -72,7 +72,7 @@ class RenderResourceLinkTestCase(unittest.TestCase):
         link = resource.render_resource_link(self.env, self.context, res)
         html = tag.a('fake:exists',  class_='fake',
                      href='/trac.cgi/fake/exists?version=1')
-        self.assertEqual(unicode(html), unicode(link))
+        self.assertEqual(str(html), str(link))
 
     def test_resource_exists_summary_format(self):
         res = resource.Resource('fake', 'exists', version=1)
@@ -80,14 +80,14 @@ class RenderResourceLinkTestCase(unittest.TestCase):
                                              res, 'summary')
         html = tag.a('fake:exists at version 1', class_='fake',
                      href='/trac.cgi/fake/exists?version=1')
-        self.assertEqual(unicode(html), unicode(link))
+        self.assertEqual(str(html), str(link))
 
     def test_resource_missing_default_format(self):
         res = resource.Resource('fake', 'missing', version=1)
         link = resource.render_resource_link(self.env, self.context, res)
         html = tag.a('fake:missing', class_='fake missing',
                      rel='nofollow', href='/trac.cgi/fake/missing?version=1')
-        self.assertEqual(unicode(html), unicode(link))
+        self.assertEqual(str(html), str(link))
 
     def test_resource_missing_summary_format(self):
         res = resource.Resource('fake', 'missing', version=1)
@@ -95,14 +95,14 @@ class RenderResourceLinkTestCase(unittest.TestCase):
                                              res, 'summary')
         html = tag.a('fake:missing at version 1', class_='fake missing',
                      rel='nofollow', href='/trac.cgi/fake/missing?version=1')
-        self.assertEqual(unicode(html), unicode(link))
+        self.assertEqual(str(html), str(link))
 
     def test_resource_has_no_manager_default_format(self):
         res = resource.Resource('unmanaged', 'exists', version=1)
         link = resource.render_resource_link(self.env, self.context, res)
         html = tag.a('unmanaged:exists', class_='unmanaged',
                      href='/trac.cgi/unmanaged/exists?version=1')
-        self.assertEqual(unicode(html), unicode(link))
+        self.assertEqual(str(html), str(link))
 
     def test_resource_has_no_manager_summary_format(self):
         res = resource.Resource('unmanaged', 'exists', version=1)
@@ -110,7 +110,7 @@ class RenderResourceLinkTestCase(unittest.TestCase):
                                              res, 'summary')
         html = tag.a('unmanaged:exists at version 1', class_='unmanaged',
                      href='/trac.cgi/unmanaged/exists?version=1')
-        self.assertEqual(unicode(html), unicode(link))
+        self.assertEqual(str(html), str(link))
 
 def test_suite():
     suite = unittest.TestSuite()
