@@ -1,29 +1,29 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2018 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
-import tempfile
+import io
 import unittest
 
 from trac.attachment import Attachment
 from trac.mimeview.api import RenderingContext
 from trac.resource import Resource
 from trac.search.web_ui import SearchModule
-from trac.test import MockPerm
+from trac.test import MockPerm, mkdtemp
 from trac.web.href import Href
 from trac.wiki.tests import formatter
 
 
-SEARCH_TEST_CASES = u"""
+SEARCH_TEST_CASES = """
 ============================== search: link resolver
 search:foo
 search:"foo bar"
@@ -58,7 +58,7 @@ search:"?q=foo bar&wiki=on"
 ------------------------------
 """
 
-ATTACHMENT_TEST_CASES = u"""
+ATTACHMENT_TEST_CASES = """
 ============================== attachment: link resolver (deprecated)
 attachment:wiki:WikiStart:file.txt (deprecated)
 attachment:ticket:123:file.txt (deprecated)
@@ -128,22 +128,22 @@ attachment:file.txt?format=raw
 def attachment_setup(tc):
     import trac.ticket.api
     import trac.wiki.api
-    tc.env.path = tempfile.mkdtemp(prefix='trac-tempenv-')
+    tc.env.path = mkdtemp()
     with tc.env.db_transaction as db:
         db("INSERT INTO wiki (name,version) VALUES ('SomePage/SubPage',1)")
         db("INSERT INTO ticket (id) VALUES (123)")
     attachment = Attachment(tc.env, 'ticket', 123)
-    attachment.insert('file.txt', tempfile.TemporaryFile(), 0)
+    attachment.insert('file.txt', io.BytesIO(b''), 0)
     attachment = Attachment(tc.env, 'wiki', 'WikiStart')
-    attachment.insert('file.txt', tempfile.TemporaryFile(), 0)
+    attachment.insert('file.txt', io.BytesIO(b''), 0)
     attachment = Attachment(tc.env, 'wiki', 'SomePage/SubPage')
-    attachment.insert('foo.txt', tempfile.TemporaryFile(), 0)
+    attachment.insert('foo.txt', io.BytesIO(b''), 0)
 
 def attachment_teardown(tc):
     tc.env.reset_db_and_disk()
 
 
-EMAIL_TEST_CASE_DEFAULT = u"""
+EMAIL_TEST_CASE_DEFAULT = """
 ============================== mailto: obfuscated by default, like plain email
 user@example.org vs. mailto:user@example.org
 and [mailto:user@example.org Joe User]
@@ -168,7 +168,7 @@ def email_default_context():
     return context
 
 
-EMAIL_TEST_CASE_NEVER_OBFUSCATE = u"""
+EMAIL_TEST_CASE_NEVER_OBFUSCATE = """
 ============================== mailto: not obfuscated, unlike plain email
 user@example.org vs. mailto:user@example.org
 and [mailto:user@example.org Joe User]
