@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 from datetime import timedelta
 
@@ -19,6 +19,7 @@ from trac.test import EnvironmentStub, MockRequest
 from trac.ticket import model
 from trac.ticket.api import TicketSystem
 from trac.ticket.model import Milestone, Ticket, Version
+from trac.ticket.test import insert_ticket
 from trac.util.datefmt import datetime_now, utc
 
 import unittest
@@ -37,25 +38,44 @@ class TicketSystemTestCase(unittest.TestCase):
 
     def _get_actions(self, ticket_dict):
         ts = TicketSystem(self.env)
-        ticket = Ticket(self.env)
-        ticket.populate(ticket_dict)
-        id = ticket.insert()
-        return ts.get_available_actions(self.req, Ticket(self.env, id))
+        ticket = insert_ticket(self.env, **ticket_dict)
+        return ts.get_available_actions(self.req, Ticket(self.env, ticket.id))
 
     def _get_ticket_field(self, field_name):
         fields = TicketSystem(self.env).get_ticket_fields()
-        return (i for i in fields if i['name'] == field_name).next()
+        return next((i for i in fields if i['name'] == field_name))
 
     def test_custom_field_text(self):
-        self.env.config.set('ticket-custom', 'test', 'text')
-        self.env.config.set('ticket-custom', 'test.label', 'Test')
-        self.env.config.set('ticket-custom', 'test.value', 'Foo bar')
-        self.env.config.set('ticket-custom', 'test.format', 'wiki')
+        def add_text_field(name, format):
+            self.env.config.set('ticket-custom', name, 'text')
+            self.env.config.set('ticket-custom', '%s.label' % name, 'Test')
+            self.env.config.set('ticket-custom', '%s.value' % name, 'Foo bar')
+            self.env.config.set('ticket-custom', '%s.format' % name, format)
+
+        add_text_field('test1', 'plain')
+        add_text_field('test2', 'wiki')
+        add_text_field('test3', 'reference')
+        add_text_field('test4', 'list')
+
         fields = TicketSystem(self.env).get_custom_fields()
-        self.assertEqual({'name': 'test', 'type': 'text', 'label': 'Test',
-                          'value': 'Foo bar', 'order': 0, 'format': 'wiki',
-                          'custom': True},
+        self.assertEqual({'name': 'test1', 'type': 'text', 'label': 'Test',
+                          'value': 'Foo bar', 'max_size': 0, 'order': 0,
+                          'format': 'plain', 'custom': True},
                          fields[0])
+        self.assertEqual({'name': 'test2', 'type': 'text', 'label': 'Test',
+                          'value': 'Foo bar', 'max_size': 0, 'order': 0,
+                          'format': 'wiki', 'custom': True},
+                         fields[1])
+        self.assertEqual({'name': 'test3', 'type': 'text', 'label': 'Test',
+                          'value': 'Foo bar', 'max_size': 0, 'order': 0,
+                          'ticketlink_query': None,
+                          'format': 'reference', 'custom': True},
+                         fields[2])
+        self.assertEqual({'name': 'test4', 'type': 'text', 'label': 'Test',
+                          'value': 'Foo bar', 'max_size': 0, 'order': 0,
+                          'ticketlink_query': None,
+                          'format': 'list', 'custom': True},
+                         fields[3])
 
     def test_custom_field_select(self):
         self.env.config.set('ticket-custom', 'test', 'select')
@@ -65,7 +85,8 @@ class TicketSystemTestCase(unittest.TestCase):
         fields = TicketSystem(self.env).get_custom_fields()
         self.assertEqual({'name': 'test', 'type': 'select', 'label': 'Test',
                           'value': '1', 'options': ['option1', 'option2'],
-                          'order': 0, 'custom': True},
+                          'order': 0, 'ticketlink_query': None,
+                          'custom': True},
                          fields[0])
 
     def test_custom_field_optional_select(self):
@@ -76,7 +97,29 @@ class TicketSystemTestCase(unittest.TestCase):
         fields = TicketSystem(self.env).get_custom_fields()
         self.assertEqual({'name': 'test', 'type': 'select', 'label': 'Test',
                           'value': '1', 'options': ['option1', 'option2'],
-                          'order': 0, 'optional': True, 'custom': True},
+                          'order': 0, 'optional': True,
+                          'ticketlink_query': None, 'custom': True},
+                         fields[0])
+
+    def test_custom_field_select_without_options(self):
+        self.env.config.set('ticket-custom', 'test', 'select')
+        self.env.config.set('ticket-custom', 'test.label', 'Test')
+        self.env.config.set('ticket-custom', 'test.value', '1')
+        fields = TicketSystem(self.env).get_custom_fields()
+        self.assertEqual(0, len(fields))
+
+    def test_custom_field_select_with_ticketlink_query(self):
+        self.env.config.set('ticket-custom', 'test', 'select')
+        self.env.config.set('ticket-custom', 'test.label', 'Test')
+        self.env.config.set('ticket-custom', 'test.value', '1')
+        self.env.config.set('ticket-custom', 'test.options', 'option1|option2')
+        self.env.config.set('ticket-custom', 'test.ticketlink_query',
+                            '?status=new')
+        fields = TicketSystem(self.env).get_custom_fields()
+        self.assertEqual({'name': 'test', 'type': 'select', 'label': 'Test',
+                          'value': '1', 'options': ['option1', 'option2'],
+                          'order': 0, 'ticketlink_query': '?status=new',
+                          'custom': True},
                          fields[0])
 
     def test_custom_field_textarea(self):
@@ -88,7 +131,7 @@ class TicketSystemTestCase(unittest.TestCase):
         fields = TicketSystem(self.env).get_custom_fields()
         self.assertEqual({'name': 'test', 'type': 'textarea', 'label': 'Test',
                           'value': 'Foo bar', 'height': 4, 'order': 0,
-                          'format': 'wiki', 'custom': True},
+                          'max_size': 0, 'format': 'wiki', 'custom': True},
                          fields[0])
 
     def test_description_field(self):
@@ -98,21 +141,22 @@ class TicketSystemTestCase(unittest.TestCase):
                           field)
 
     def test_custom_field_checkbox(self):
-        def add_checkbox(name, value):
+        def add_checkbox_field(name, value):
             self.env.config.set('ticket-custom', name, 'checkbox')
             self.env.config.set('ticket-custom', '%s.value' % name, value)
 
-        add_checkbox('checkbox0', 'true')
-        add_checkbox('checkbox1', 1)
-        add_checkbox('checkbox2', 'enabled')
-        add_checkbox('checkbox3', 0)
-        add_checkbox('checkbox4', 'tru')
-        add_checkbox('checkbox5', 'off')
+        add_checkbox_field('checkbox0', 'true')
+        add_checkbox_field('checkbox1', 1)
+        add_checkbox_field('checkbox2', 'enabled')
+        add_checkbox_field('checkbox3', 0)
+        add_checkbox_field('checkbox4', 'tru')
+        add_checkbox_field('checkbox5', 'off')
 
         fields = TicketSystem(self.env).get_custom_fields()
         self.assertEqual({'name': 'checkbox0', 'type': 'checkbox',
                           'label': 'Checkbox0', 'value': '1',
-                          'order': 0, 'custom': True},
+                          'order': 0, 'ticketlink_query': None,
+                          'custom': True},
                          fields[0])
         self.assertEqual('1', fields[1]['value'])
         self.assertEqual('1', fields[2]['value'])
@@ -120,6 +164,52 @@ class TicketSystemTestCase(unittest.TestCase):
         self.assertEqual('0', fields[4]['value'])
         self.assertEqual('0', fields[5]['value'])
 
+    def test_custom_field_checkbox_with_ticketlink_query(self):
+        self.env.config.set('ticket-custom', 'checkbox0', 'checkbox')
+        ticketlink_query = '?status=assigned'
+        self.env.config.set('ticket-custom', 'checkbox0.ticketlink_query',
+                            ticketlink_query)
+
+        fields = TicketSystem(self.env).get_custom_fields()
+        self.assertEqual({'name': 'checkbox0', 'type': 'checkbox',
+                          'label': 'Checkbox0', 'value': '0',
+                          'order': 0, 'ticketlink_query': ticketlink_query,
+                          'custom': True},
+                         fields[0])
+
+    def test_custom_field_radio(self):
+        def add_radio(name, value=None):
+            self.env.config.set('ticket-custom', name, 'radio')
+            if value:
+                self.env.config.set('ticket-custom', '%s.value' % name, value)
+            self.env.config.set('ticket-custom', '%s.options' % name, '1|2|3')
+
+        add_radio('radio0')
+        add_radio('radio1', '2')
+
+        fields = TicketSystem(self.env).get_custom_fields()
+        self.assertEqual({'name': 'radio0', 'type': 'radio',
+                          'label': 'Radio0', 'value': '',
+                          'options': ['1', '2', '3'], 'order': 0,
+                          'ticketlink_query': None, 'custom': True},
+                         fields[0])
+        self.assertEqual('2', fields[1]['value'])
+
+    def test_custom_field_radio_with_ticketlink_query(self):
+        self.env.config.set('ticket-custom', 'radio0', 'radio')
+        self.env.config.set('ticket-custom', 'radio0.options', '1|2|3')
+        ticketlink_query = '?status=assigned'
+        self.env.config.set('ticket-custom', 'radio0.ticketlink_query',
+                            ticketlink_query)
+
+        fields = TicketSystem(self.env).get_custom_fields()
+        self.assertEqual({'name': 'radio0', 'type': 'radio',
+                          'label': 'Radio0', 'value': '',
+                          'options': ['1', '2', '3'], 'order': 0,
+                          'ticketlink_query': ticketlink_query,
+                          'custom': True},
+                         fields[0])
+
     def test_custom_field_time(self):
         self.env.config.set('ticket-custom', 'test', 'time')
         self.env.config.set('ticket-custom', 'test.label', 'Test')
@@ -139,27 +229,48 @@ class TicketSystemTestCase(unittest.TestCase):
         ticket_custom.set('FiEld5', 'text')
 
         ts = TicketSystem(self.env)
+        custom_fields = ts.custom_fields
+        fields = ts.fields
 
-        self.assertEqual(2, len(ts.custom_fields))
-        self.assertIsNotNone(ts.custom_fields.by_name('field4'))
-        self.assertIsNotNone(ts.custom_fields.by_name('field5'))
-        self.assertIsNotNone(ts.fields.by_name('field4'))
-        self.assertIsNotNone(ts.fields.by_name('field5'))
+        self.assertEqual(2, len(custom_fields))
+        self.assertIsNotNone(custom_fields.by_name('field4'))
+        self.assertIsNotNone(custom_fields.by_name('field5'))
+        self.assertIsNotNone(fields.by_name('field4'))
+        self.assertIsNotNone(fields.by_name('field5'))
+        self.assertIn(
+            ('WARNING', 'Invalid name for custom field: "_field1" (ignoring)'),
+            self.env.log_messages)
+        self.assertIn(
+            ('WARNING', 'Invalid name for custom field: "2field" (ignoring)'),
+            self.env.log_messages)
+        self.assertIn(
+            ('WARNING', 'Invalid name for custom field: "f3%^&*" (ignoring)'),
+            self.env.log_messages)
 
     def test_custom_field_with_reserved_name(self):
         ticket_custom = self.env.config['ticket-custom']
         ticket_custom.set('owner', 'select')
+        ticket_custom.set('owner.options', 'u1|u2|u3')
         ticket_custom.set('description', 'text')
 
         ts = TicketSystem(self.env)
+        custom_fields = ts.custom_fields
 
+        self.assertIn(
+            ('WARNING',
+             'Field name "owner" is a reserved name (ignoring)'),
+            self.env.log_messages)
+        self.assertIn(
+            ('WARNING',
+             'Field name "description" is a reserved name (ignoring)'),
+            self.env.log_messages)
         self.assertEqual({'name': 'owner', 'label': 'Owner', 'type': 'text'},
                          ts.fields.by_name('owner'))
         self.assertEqual({'name': 'description', 'label': 'Description',
                           'type': 'textarea', 'format': 'wiki'},
                          ts.fields.by_name('description'))
-        self.assertIsNone(ts.custom_fields.by_name('owner'))
-        self.assertIsNone(ts.custom_fields.by_name('description'))
+        self.assertIsNone(custom_fields.by_name('owner'))
+        self.assertIsNone(custom_fields.by_name('description'))
 
     def test_custom_field_order(self):
         self.env.config.set('ticket-custom', 'test1', 'text')
@@ -372,7 +483,7 @@ class TicketSystemTestCase(unittest.TestCase):
                          updated_milestone_field['options'])
 
     def test_resource_exists_valid_resource_id(self):
-        Ticket(self.env).insert()
+        insert_ticket(self.env)
         r1 = Resource('ticket', 1)
         r2 = Resource('ticket', 2)
 
