@@ -1,23 +1,21 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014 Edgewall Software
+# Copyright (C) 2014-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
-from __future__ import print_function
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import tempfile
+import textwrap
 import unittest
 
-import trac.tests.compat
 from trac.config import ConfigurationError
 from trac.perm import PermissionSystem
 from trac.test import EnvironmentStub, MockRequest
@@ -25,6 +23,7 @@ from trac.ticket.api import TicketSystem
 from trac.ticket.batch import BatchModifyModule
 from trac.ticket.default_workflow import ConfigurableTicketWorkflow
 from trac.ticket.model import Component, Ticket
+from trac.ticket.test import insert_ticket
 from trac.ticket.web_ui import TicketModule
 from trac.util import create_file
 from trac.util.datefmt import to_utimestamp
@@ -35,7 +34,7 @@ from tracopt.perm.authz_policy import AuthzPolicy
 class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.env = EnvironmentStub(default_data=True)
+        self.env = EnvironmentStub()
         config = self.env.config
         config.set('ticket-workflow', 'change_owner', 'new -> new')
         config.set('ticket-workflow', 'change_owner.operations', 'set_owner')
@@ -66,7 +65,7 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
                 resolve_action = attrs
 
         self.assertIsNotNone(resolve_action)
-        self.assertIn('set_milestone', resolve_action.keys())
+        self.assertIn('set_milestone', list(resolve_action))
         self.assertEqual('reject', resolve_action['set_milestone'])
 
     def test_owner_from_component(self):
@@ -92,25 +91,19 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
         self._add_component('component3', 'cowner3')
         self._add_component('component4', 'cowner4')
 
-        ticket = Ticket(self.env)
-        ticket.populate({
-            'reporter': 'reporter1',
-            'summary': 'the summary',
-            'component': 'component3',
-            'owner': 'cowner3',
-            'status': 'new',
-        })
-        tkt_id = ticket.insert()
+        ticket = insert_ticket(self.env, reporter='reporter1',
+                               summary='the summary', component='component3',
+                               owner='cowner3', status='new')
 
         req = MockRequest(self.env, method='POST', args={
-            'id': tkt_id,
+            'id': ticket.id,
             'field_component': 'component4',
             'submit': True,
             'action': 'leave',
             'view_time': str(to_utimestamp(ticket['changetime'])),
         })
         self.assertRaises(RequestDone, self.ticket_module.process_request, req)
-        ticket = Ticket(self.env, tkt_id)
+        ticket = Ticket(self.env, ticket.id)
 
         self.assertEqual('component4', ticket['component'])
         self.assertEqual('cowner4', ticket['owner'])
@@ -122,17 +115,12 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
         self._add_component('component3', 'cowner3')
         self._add_component('component4', 'cowner4')
 
-        ticket = Ticket(self.env)
-        ticket.populate({
-            'reporter': 'reporter1',
-            'summary': 'the summary',
-            'component': 'component3',
-            'status': 'new',
-        })
-        tkt_id = ticket.insert()
+        ticket = insert_ticket(self.env, reporter='reporter1',
+                               summary='the summary', component='component3',
+                               status='new')
 
         req = MockRequest(self.env, method='POST', args={
-            'id': tkt_id,
+            'id': ticket.id,
             'field_component': 'component4',
             'submit': True,
             'action': 'change_owner',
@@ -140,7 +128,7 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
             'view_time': str(to_utimestamp(ticket['changetime'])),
         })
         self.assertRaises(RequestDone, self.ticket_module.process_request, req)
-        ticket = Ticket(self.env, tkt_id)
+        ticket = Ticket(self.env, ticket.id)
 
         self.assertEqual('component4', ticket['component'])
         self.assertEqual('owner1', ticket['owner'])
@@ -152,25 +140,19 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
         self._add_component('component3', 'cowner3')
         self._add_component('component4', 'cowner4')
 
-        ticket = Ticket(self.env)
-        ticket.populate({
-            'reporter': 'reporter1',
-            'summary': 'the summary',
-            'component': 'component3',
-            'owner': 'owner1',
-            'status': 'new',
-        })
-        tkt_id = ticket.insert()
+        ticket = insert_ticket(self.env, reporter='reporter1',
+                               summary='the summary', component='component3',
+                               owner='owner1', status='new')
 
         req = MockRequest(self.env, method='POST', args={
-            'id': tkt_id,
+            'id': ticket.id,
             'field_component': 'component4',
             'submit': True,
             'action': 'leave',
             'view_time': str(to_utimestamp(ticket['changetime'])),
         })
         self.assertRaises(RequestDone, self.ticket_module.process_request, req)
-        ticket = Ticket(self.env, tkt_id)
+        ticket = Ticket(self.env, ticket.id)
 
         self.assertEqual('component4', ticket['component'])
         self.assertEqual('owner1', ticket['owner'])
@@ -182,32 +164,133 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
         self._add_component('component3', 'cowner3')
         self._add_component('component4', '')
 
-        ticket = Ticket(self.env)
-        ticket.populate({
-            'reporter': 'reporter1',
-            'summary': 'the summary',
-            'component': 'component3',
-            'owner': 'cowner3',
-            'status': 'new',
-        })
-        tkt_id = ticket.insert()
+        ticket = insert_ticket(self.env, reporter='reporter1',
+                               summary='the summary', component='component3',
+                               owner='cowner3', status='new')
 
         req = MockRequest(self.env, method='POST', args={
-            'id': tkt_id,
+            'id': ticket.id,
             'field_component': 'component4',
             'submit': True,
             'action': 'leave',
             'view_time': str(to_utimestamp(ticket['changetime'])),
         })
         self.assertRaises(RequestDone, self.ticket_module.process_request, req)
-        ticket = Ticket(self.env, tkt_id)
+        ticket = Ticket(self.env, ticket.id)
 
         self.assertEqual('component4', ticket['component'])
         self.assertEqual('cowner3', ticket['owner'])
 
+    def _test_get_allowed_owners(self):
+        ticket = insert_ticket(self.env, summary='Ticket 1')
+        self.env.insert_users([('user1', None, None, 1),
+                               ('user2', None, None, 1),
+                               ('user3', None, None, 1)])
+        ps = PermissionSystem(self.env)
+        for user in ('user1', 'user3'):
+            ps.grant_permission(user, 'TICKET_MODIFY')
+        self.env.config.set('ticket', 'restrict_owner', True)
+        return ticket
+
+    def test_get_allowed_owners_returns_set_owner_list(self):
+        """Users specified in `set_owner` for the action are returned."""
+        req = None
+        action = {'set_owner': ['user4', 'user5']}
+        ticket = self._test_get_allowed_owners()
+        self.assertEqual(['user4', 'user5'],
+                         self.ctlr.get_allowed_owners(req, ticket, action))
+
+    def test_get_allowed_owners_returns_user_with_ticket_modify(self):
+        """Users with TICKET_MODIFY are are returned if `set_owner` is
+        not specified for the action.
+        """
+        req = None
+        action = {}
+        ticket = self._test_get_allowed_owners()
+        self.assertEqual(['user1', 'user3'],
+                         self.ctlr.get_allowed_owners(req, ticket, action))
+
+    def test_status_change_with_operation(self):
+        """Status change with operation."""
+        ticket = Ticket(self.env)
+        ticket['new'] = 'status1'
+        ticket['owner'] = 'user1'
+        ticket.insert()
+        req = MockRequest(self.env, path_info='/ticket', authname='user2',
+                          method='POST')
+
+        label, control, hints = \
+            self.ctlr.render_ticket_action_control(req, ticket, 'accept')
+
+        self.assertEqual('accept', label)
+        self.assertEqual('', str(control))
+        self.assertEqual("The owner will be <span class=\"trac-author-user\">"
+                         "user2</span>. The status will be 'accepted'.",
+                         str(hints))
+
+    def test_status_change_with_no_operation(self):
+        """Existing ticket status change with no operation."""
+        config = self.env.config
+        config.set('ticket-workflow', 'change_status', 'status1 -> status2')
+        self._reload_workflow()
+        ticket = Ticket(self.env)
+        ticket['status'] = 'status1'
+        ticket.insert()
+        req = MockRequest(self.env, path_info='/ticket', method='POST')
+
+        label, control, hints = \
+            self.ctlr.render_ticket_action_control(req, ticket,
+                                                   'change_status')
+
+        self.assertEqual('change status', label)
+        self.assertEqual('', str(control))
+        self.assertEqual("Next status will be 'status2'.", str(hints))
+
+    def test_new_ticket_status_change_with_no_operation(self):
+        """New ticket status change with no operation."""
+        config = self.env.config
+        config.set('ticket-workflow', 'change_status', '<none> -> status1')
+        self._reload_workflow()
+        ticket = Ticket(self.env)
+        req = MockRequest(self.env, path_info='/newticket', method='POST')
+
+        label, control, hints = \
+            self.ctlr.render_ticket_action_control(req, ticket,
+                                                   'change_status')
+
+        self.assertEqual('change status', label)
+        self.assertEqual('', str(control))
+        self.assertEqual("The status will be 'status1'.", str(hints))
+
+    def test_operation_with_no_status_change(self):
+        """Operation with no status change."""
+        config = self.env.config
+        config.set('ticket-workflow', 'change_owner', 'closed -> closed')
+        config.set('ticket-workflow', 'change_owner.operations', 'set_owner')
+
+        self._reload_workflow()
+        ticket = Ticket(self.env)
+        ticket['status'] = 'closed'
+        ticket['owner'] = 'user2'
+        ticket.insert()
+        req = MockRequest(self.env, path_info='/ticket', method='POST',
+                          authname='user1')
+
+        label, control, hints = \
+            self.ctlr.render_ticket_action_control(req, ticket,
+                                                   'change_owner')
+
+        self.assertEqual('change owner', label)
+        self.assertEqual(
+            'to <input id="action_change_owner_reassign_owner" '
+            'name="action_change_owner_reassign_owner" type="text" '
+            'value="user1" />', str(control))
+        self.assertEqual(
+            'The owner will be changed from <span class="trac-author">'
+            'user2</span> to the specified user.', str(hints))
+
     def test_transition_to_star(self):
-        """Workflow hint is not be added in a workflow transition to *,
-        for example: <none> -> *
+        """Action not rendered by CTW for transition to *
 
         AdvancedTicketWorkflow uses the behavior for the triage operation
         (see #12823)
@@ -219,27 +302,83 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
         ticket = Ticket(self.env)
         req = MockRequest(self.env, path_info='/newticket', method='POST')
 
-        label, control, hints = \
-            self.ctlr.render_ticket_action_control(req, ticket,
-                                                   'create_and_triage')
+        actions = self.ctlr.get_ticket_actions(req, ticket)
 
-        self.assertEqual('create and triage', label)
-        self.assertEqual('', unicode(control))
-        self.assertEqual('', unicode(hints))
+        # create_and_triage not in actions
+        self.assertEqual({(1, 'create'), (0, 'create_and_assign')},
+                         set(actions))
 
-    def test_ignores_other_operations(self):
-        """Ignores operations not defined by ConfigurableTicketWorkflow.
+    def test_transition_to_star_with_leave_operation(self):
+        """Action is rendered by CTW for transition to * with leave_status
         """
-        self.env.config.set('ticket-workflow', 'review', 'assigned -> review')
-        self.env.config.set('ticket-workflow', 'review.operations',
-                            'CodeReview')
-        ctw = ConfigurableTicketWorkflow(self.env)
-        ticket = Ticket(self.env)
-        ticket.populate({'summary': '#13013', 'status': 'assigned'})
-        ticket.insert()
-        req = MockRequest(self.env)
+        config = self.env.config
+        config.set('ticket-workflow', 'change_owner', 'assigned,closed -> *')
+        config.set('ticket-workflow', 'change_owner.operations',
+                   'leave_status,set_owner')
+        self._reload_workflow()
+        status = ['assigned', 'closed']
+        for s in status:
+            ticket = Ticket(self.env)
+            ticket['status'] = s
+            ticket['owner'] = 'user2'
+            ticket.insert()
+            req = MockRequest(self.env, path_info='/ticket', method='POST',
+                              authname='user1')
 
-        self.assertNotIn((0, 'review'), ctw.get_ticket_actions(req, ticket))
+            label, control, hints = \
+                self.ctlr.render_ticket_action_control(req, ticket,
+                                                       'change_owner')
+            self.assertEqual('change owner', label)
+            self.assertEqual(
+                'to <input id="action_change_owner_reassign_owner" '
+                'name="action_change_owner_reassign_owner" type="text" '
+                'value="user1" />', str(control))
+            self.assertEqual(
+                'The owner will be changed from <span class="trac-author">'
+                'user2</span> to the specified user.', str(hints))
+
+    def test_leave_operation(self):
+        ticket = Ticket(self.env)
+        ticket['status'] = 'assigned'
+        ticket['owner'] = 'user2'
+        ticket.insert()
+        req = MockRequest(self.env, path_info='/ticket', method='POST',
+                          authname='user1')
+
+        label, control, hints = \
+            self.ctlr.render_ticket_action_control(req, ticket, 'leave')
+
+        self.assertEqual('leave', label)
+        self.assertEqual('as assigned', str(control))
+        self.assertEqual('The owner will remain <span class="trac-author">'
+                         'user2</span>.', str(hints))
+
+    def test_get_actions_by_operation_for_req(self):
+        """Request with no permission checking."""
+        req = MockRequest(self.env, path_info='/ticket/1')
+        ticket = insert_ticket(self.env, status='new')
+        actions = self.ctlr.get_actions_by_operation_for_req(req, ticket,
+                                                             'set_owner')
+        self.assertEqual([(0, 'change_owner'), (0, 'reassign')],
+                         sorted(actions))
+
+    def test_get_actions_by_operation_for_req_with_ticket_modify(self):
+        """User without TICKET_MODIFY won't have reassign action."""
+        req = MockRequest(self.env, authname='user1', path_info='/ticket/1')
+        ticket = insert_ticket(self.env, status='new')
+        actions = self.ctlr.get_actions_by_operation_for_req(req, ticket,
+                                                             'set_owner')
+        self.assertEqual([(0, 'change_owner')], sorted(actions))
+
+    def test_get_actions_by_operation_for_req_without_ticket_modify(self):
+        """User with TICKET_MODIFY will have reassign action."""
+        PermissionSystem(self.env).grant_permission('user1', 'TICKET_MODIFY')
+        req = MockRequest(self.env, authname='user1', path_info='/ticket/1')
+        ticket = insert_ticket(self.env, status='new')
+        actions = self.ctlr.get_actions_by_operation_for_req(req, ticket,
+                                                             'set_owner')
+        self.assertEqual([(0, 'change_owner'), (0, 'reassign')],
+                         sorted(actions))
 
 
 class ResetActionTestCase(unittest.TestCase):
@@ -250,9 +389,7 @@ class ResetActionTestCase(unittest.TestCase):
         self.ctlr = TicketSystem(self.env).action_controllers[0]
         self.req1 = MockRequest(self.env, authname='user1')
         self.req2 = MockRequest(self.env, authname='user2')
-        self.ticket = Ticket(self.env)
-        self.ticket['status'] = 'invalid'
-        self.ticket.insert()
+        self.ticket = insert_ticket(self.env, status='invalid')
 
     def tearDown(self):
         self.env.reset_db()
@@ -275,6 +412,26 @@ class ResetActionTestCase(unittest.TestCase):
         self.assertIn((0, '_reset'), actions2)
         self.assertEqual('new', chgs2['status'])
 
+    def test_default_reset_action_without_new_state(self):
+        """Default reset action not available when no new state."""
+        self.perm_sys.grant_permission('user2', 'TICKET_ADMIN')
+        config = self.env.config
+        # Replace 'new' state with 'untriaged'
+        config.set('ticket-workflow', 'create',
+                   '<none> -> untriaged')
+        config.set('ticket-workflow', 'accept',
+                   'untriaged,assigned,accepted,reopened -> accepted')
+        config.set('ticket-workflow', 'resolve',
+                   'untriaged,assigned,accepted,reopened -> closed')
+        config.set('ticket-workflow', 'reassign',
+                   'untriaged,assigned,accepted,reopened -> assigned')
+        self._reload_workflow()
+
+        actions = self.ctlr.get_ticket_actions(self.req2, self.ticket)
+
+        self.assertEqual(1, len(actions))
+        self.assertNotIn((0, '_reset'), actions)
+
     def test_custom_reset_action(self):
         """Custom reset action in [ticket-workflow] section."""
         config = self.env.config['ticket-workflow']
@@ -302,9 +459,7 @@ class SetOwnerAttributeTestCase(unittest.TestCase):
         self.env = EnvironmentStub(default_data=True)
         self.perm_sys = PermissionSystem(self.env)
         self.ctlr = TicketSystem(self.env).action_controllers[0]
-        self.ticket = Ticket(self.env)
-        self.ticket['status'] = 'new'
-        self.ticket.insert()
+        self.ticket = insert_ticket(self.env, status='new')
         self.env.insert_users([
             (user, None, None) for user in ('user1', 'user2', 'user3', 'user4')
         ])
@@ -324,8 +479,8 @@ class SetOwnerAttributeTestCase(unittest.TestCase):
             self.perm_sys.grant_permission(*perm)
         self.req = MockRequest(self.env, authname='user1')
         self.expected = """\
-to <select name="action_reassign_reassign_owner" \
-id="action_reassign_reassign_owner"><option selected="True" \
+to <select id="action_reassign_reassign_owner" \
+name="action_reassign_reassign_owner"><option selected="selected" \
 value="user1">user1</option><option value="user2">user2</option>\
 <option value="user3">user3</option></select>"""
 
@@ -408,11 +563,11 @@ class SetOwnerToSelfAttributeTestCase(unittest.TestCase):
 
         self.assertIn('accept', ticket_actions)
         self.assertEqual(label, 'accept')
-        self.assertEqual('', unicode(control))
+        self.assertEqual('', str(control))
         self.assertEqual('The owner will be changed from '
                          '<span class="trac-author">User 2</span> to '
                          '<span class="trac-author-user">User 1</span>.',
-                         unicode(hints))
+                         str(hints))
 
     def test_owner_is_self_and_state_change(self):
         """Ticket owner is auth'ed user with state change.
@@ -428,9 +583,10 @@ class SetOwnerToSelfAttributeTestCase(unittest.TestCase):
 
         self.assertIn('accept', ticket_actions)
         self.assertEqual(label, 'accept')
-        self.assertEqual('', unicode(control))
+        self.assertEqual('', str(control))
         self.assertEqual('The owner will remain <span class="trac-author-user">'
-                         'User 1</span>.', unicode(hints))
+                         'User 1</span>. Next status will be \'accepted\'.',
+                         str(hints))
 
     def test_owner_is_self_and_no_state_change(self):
         """Ticket owner is the auth'ed user and no state change.
@@ -468,13 +624,13 @@ class SetOwnerToSelfAttributeTestCase(unittest.TestCase):
         self.assertIn('resolve_as_owner', ticket_actions)
         self.assertEqual(label, 'resolve as owner')
         self.assertEqual(
-            'as fixed<input type="hidden" '
-            'name="action_resolve_as_owner_resolve_resolution" value="fixed" '
-            'id="action_resolve_as_owner_resolve_resolution"/>',
-            unicode(control))
+            'as fixed<input id="action_resolve_as_owner_resolve_resolution" '
+            'name="action_resolve_as_owner_resolve_resolution" type="hidden" '
+            'value="fixed" />', str(control))
         self.assertEqual(
-            'The owner will remain <span class="trac-author-user">User 1'
-            '</span>. The resolution will be set to fixed.', unicode(hints))
+            "The owner will remain <span class=\"trac-author-user\">User 1"
+            "</span>. The resolution will be set to fixed. Next status will "
+            "be 'closed'.", str(hints))
 
     def test_owner_is_self_no_state_change_and_multiple_operations(self):
         """Ticket owner is auth'ed user, no state change and multiple ops.
@@ -498,12 +654,11 @@ class SetOwnerToSelfAttributeTestCase(unittest.TestCase):
         self.assertIn('fix_resolution', ticket_actions)
         self.assertEqual(label, 'fix resolution')
         self.assertEqual(
-            'as invalid<input type="hidden" '
-            'name="action_fix_resolution_resolve_resolution" value="invalid" '
-            'id="action_fix_resolution_resolve_resolution"/>',
-            unicode(control))
-        self.assertEqual('The resolution will be set to invalid.',
-                         unicode(hints))
+            'as invalid<input id="action_fix_resolution_resolve_resolution" '
+            'name="action_fix_resolution_resolve_resolution" type="hidden" '
+            'value="invalid" />', str(control))
+        self.assertEqual("The resolution will be set to invalid.",
+                         str(hints))
 
 
 class RestrictOwnerTestCase(unittest.TestCase):
@@ -529,9 +684,7 @@ class RestrictOwnerTestCase(unittest.TestCase):
         self.env.config.set('authz_policy', 'authz_file', self.authz_file)
         self.ctlr = TicketSystem(self.env).action_controllers[0]
         self.req1 = MockRequest(self.env, authname='user1')
-        self.ticket = Ticket(self.env)
-        self.ticket['status'] = 'new'
-        self.ticket.insert()
+        self.ticket = insert_ticket(self.env, status='new')
 
     def tearDown(self):
         self.env.reset_db()
@@ -559,10 +712,10 @@ class RestrictOwnerTestCase(unittest.TestCase):
         owners list (#10833).
         """
         self.env.config.set('trac', 'show_full_names', False)
-        create_file(self.authz_file, """\
-[ticket:1]
-user4 = !TICKET_MODIFY
-""")
+        create_file(self.authz_file, textwrap.dedent("""\
+            [ticket:1]
+            user4 = !TICKET_MODIFY
+            """))
 
         ctrl = self.ctlr.render_ticket_action_control(self.req1, self.ticket,
                                                       'reassign')
@@ -580,10 +733,10 @@ user4 = !TICKET_MODIFY
 
         self.assertEqual('reassign', ctrl[0])
         self.assertEqual("""\
-to <select name="action_reassign_reassign_owner" \
-id="action_reassign_reassign_owner">\
+to <select id="action_reassign_reassign_owner"\
+ name="action_reassign_reassign_owner">\
 <option value="user4">User B</option>\
-<option selected="True" value="user1">User C</option>\
+<option selected="selected" value="user1">User C</option>\
 <option value="user3">User D</option></select>\
 """, str(ctrl[1]))
 
@@ -611,7 +764,7 @@ class SetResolutionAttributeTestCase(unittest.TestCase):
             self.ctlr.render_ticket_action_control(req, ticket, 'resolve')
             self.fail('ConfigurationError not raised')
         except ConfigurationError as e:
-            self.assertIn('but none is defined', unicode(e))
+            self.assertIn('but none is defined', str(e))
 
     def test_undefined_resolutions(self):
         config = self.env.config['ticket-workflow']
@@ -631,7 +784,7 @@ class SetResolutionAttributeTestCase(unittest.TestCase):
             self.ctlr.render_ticket_action_control(req, ticket, 'resolve')
             self.fail('ConfigurationError not raised')
         except ConfigurationError as e:
-            self.assertIn('but uses undefined resolutions', unicode(e))
+            self.assertIn('but uses undefined resolutions', str(e))
 
 
 def test_suite():
