@@ -1,24 +1,23 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2018 Edgewall Software
+# Copyright (C) 2008-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 
-import trac.tests.compat
-from trac.tests.functional import FunctionalTwillTestCaseSetup, tc
+from trac.tests.functional import FunctionalTestCaseSetup, tc
 
 
-class SetOwnerOperation(FunctionalTwillTestCaseSetup):
+class SetOwnerOperation(FunctionalTestCaseSetup):
 
     def setUp(self):
         super(SetOwnerOperation, self).setUp()
@@ -47,8 +46,9 @@ class SetOwnerOperation(FunctionalTwillTestCaseSetup):
         self._tester.go_to_ticket(ticket_id)
         tc.find('The owner will be changed from '
                 '<span class="trac-author">lammy</span>')
-        tc.find('<input type="text" name="action_reassign_reassign_owner" '
-                'value="admin" id="action_reassign_reassign_owner" />')
+        tc.find('<input id="action_reassign_reassign_owner" '
+                'name="action_reassign_reassign_owner" type="text" '
+                'value="admin" />')
 
     def test_restrict_owner_not_known_user(self):
         """When using the workflow operation `set_owner` with
@@ -66,7 +66,7 @@ class SetOwnerOperation(FunctionalTwillTestCaseSetup):
             self._tester.go_to_ticket(ticket_id)
             tc.find('The owner will be changed from '
                     '<span class="trac-author">lammy</span>')
-            tc.notfind('<option value="anonymous" selected="selected">'
+            tc.notfind('<option selected="selected" value="anonymous">'
                        'anonymous</option>')
 
         finally:
@@ -119,20 +119,22 @@ class SetOwnerOperation(FunctionalTwillTestCaseSetup):
             self._tester.go_to_ticket(ticket_id)
 
             tc.notfind('<select name="action_reassign_reassign_owner"')
-            tc.find('<input type="hidden" '
-                    'name="action_reassign_reassign_owner" '
-                    'value="alice" id="action_reassign_reassign_owner" />')
+            tc.find('<input id="action_reassign_reassign_owner" '
+                    'name="action_reassign_reassign_owner" type="hidden" '
+                    'value="alice" />')
             tc.find('The owner will be changed from '
                     '<span class="trac-author">lammy</span> to '
                     '<span class="trac-author">alice</span>')
 
-            tc.notfind('<input type="text" name="action_reassign_reassign_owner" '
-                       'value="admin" id="action_reassign_reassign_owner" />')
+            tc.notfind('<input id="action_reassign_reassign_owner" '
+                       'name="action_reassign_reassign_owner" type="text" '
+                       'value="admin" />')
             tc.notfind('<option selected="selected" value="admin">admin</option>')
             tc.notfind('<option value="admin">admin</option>')
 
-            tc.notfind('<input type="text" name="action_reassign_reassign_owner" '
-                       'value="lammy" id="action_reassign_reassign_owner" />')
+            tc.notfind('<input id="action_reassign_reassign_owner"'
+                       'name="action_reassign_reassign_owner" type="text" '
+                       'value="lammy" />')
             tc.notfind('<option selected="selected" value="lammy">lammy</option>')
             tc.notfind('<option value="lammy">lammy</option>')
 
@@ -140,10 +142,10 @@ class SetOwnerOperation(FunctionalTwillTestCaseSetup):
             self.env.config.remove('ticket-workflow', 'reassign.set_owner')
 
 
-class MaySetOwnerOperationRestrictOwnerFalse(FunctionalTwillTestCaseSetup):
+class MaySetOwnerOperationRestrictOwnerFalse(FunctionalTestCaseSetup):
     """Test cases for may_set_owner operation with
     `[ticket] restrict_owner = False`
-    http://trac.edgewall.org/ticket/10018
+    https://trac.edgewall.org/ticket/10018
     """
     def setUp(self):
         super(MaySetOwnerOperationRestrictOwnerFalse, self).setUp()
@@ -171,8 +173,9 @@ class MaySetOwnerOperationRestrictOwnerFalse(FunctionalTwillTestCaseSetup):
         self._tester.go_to_ticket(ticket_id)
         tc.find('The owner will be changed from '
                 '<span class="trac-author">lammy</span>')
-        tc.find('<input type="text" name="action_reassign_reassign_owner"'
-                ' value="lammy" id="action_reassign_reassign_owner" />')
+        tc.find('<input id="action_reassign_reassign_owner"'
+                ' name="action_reassign_reassign_owner" type="text"'
+                ' value="lammy" />')
 
     def test_default_no_owner(self):
         """The assign-to field will default to a blank field if the ticket
@@ -184,8 +187,8 @@ class MaySetOwnerOperationRestrictOwnerFalse(FunctionalTwillTestCaseSetup):
         tc.find("The ticket will remain with no owner.")
         tc.find('The owner will be changed from '
                 '<span class="trac-author-none">\(none\)</span>')
-        tc.find('<input type="text" name="action_reassign_reassign_owner"'
-                ' id="action_reassign_reassign_owner" />')
+        tc.find('<input id="action_reassign_reassign_owner"'
+                ' name="action_reassign_reassign_owner" type="text" />')
 
     def test_default_restrict_owner(self):
         """The assign-to field will default to the ticket's current owner
@@ -255,7 +258,7 @@ class MaySetOwnerOperationRestrictOwnerFalse(FunctionalTwillTestCaseSetup):
             tc.find('The owner will be changed from '
                     '<span class="trac-author">lammy</span>')
 
-            tc.find('<select name="action_reassign_reassign_owner"')
+            tc.find('<select id="action_reassign_reassign_owner"')
             tc.notfind('<input type="hidden" '
                     'name="action_reassign_reassign_owner" '
                     'value="alice" id="action_reassign_reassign_owner" />')
@@ -272,9 +275,9 @@ class MaySetOwnerOperationRestrictOwnerFalse(FunctionalTwillTestCaseSetup):
             self._tester.go_to_ticket(ticket_id)
 
             tc.notfind('<select name="action_reassign_reassign_owner"')
-            tc.find('<input type="hidden" '
+            tc.find('<input id="action_reassign_reassign_owner" '
                     'name="action_reassign_reassign_owner" '
-                    'value="lammy" id="action_reassign_reassign_owner" />')
+                    'type="hidden" value="lammy" />')
             tc.find('The owner will remain '
                     '<span class="trac-author">lammy</span>')
             tc.notfind('<option selected="selected" value="lammy">lammy</option>')
@@ -285,7 +288,7 @@ class MaySetOwnerOperationRestrictOwnerFalse(FunctionalTwillTestCaseSetup):
 
 
 
-class MaySetOwnerOperationDefaultRestrictOwnerNone(FunctionalTwillTestCaseSetup):
+class MaySetOwnerOperationDefaultRestrictOwnerNone(FunctionalTestCaseSetup):
     def runTest(self):
         """When using the workflow operation `may_set_owner` with
         restrict_owner=true, the assign-to field will default to an empty
@@ -309,7 +312,7 @@ class MaySetOwnerOperationDefaultRestrictOwnerNone(FunctionalTwillTestCaseSetup)
             tc.find("The ticket will remain with no owner.")
             tc.find('The owner will be changed from '
                     '<span class="trac-author-none">\(none\)</span>')
-            tc.find('<option selected="selected" value="">\(none\)</option>')
+            tc.find('<option selected="selected">\(none\)</option>')
         finally:
             env.config.set('ticket-workflow', 'reassign.operations',
                            reassign_operations)
@@ -317,7 +320,7 @@ class MaySetOwnerOperationDefaultRestrictOwnerNone(FunctionalTwillTestCaseSetup)
             env.config.save()
 
 
-class MaySetOwnerOperationDefaultRestrictOwnerAnonymous(FunctionalTwillTestCaseSetup):
+class MaySetOwnerOperationDefaultRestrictOwnerAnonymous(FunctionalTestCaseSetup):
     def runTest(self):
         """When using the workflow operation `may_set_owner` with
         restrict_owner=true, the assign-to dropdown menu will contain a
@@ -356,9 +359,9 @@ class MaySetOwnerOperationDefaultRestrictOwnerAnonymous(FunctionalTwillTestCaseS
             env.config.save()
 
 
-class RegressionTestTicket11930(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket11930(FunctionalTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11930
+        """Test for regression of https://trac.edgewall.org/ticket/11930
         Workflow action labels are present on the ticket page.
         """
         self._tester.create_ticket()
@@ -368,6 +371,23 @@ class RegressionTestTicket11930(FunctionalTwillTestCaseSetup):
         tc.find('<label for="action_accept">accept</label>')
 
 
+class RegressionTestTicket13087(FunctionalTestCaseSetup):
+    def runTest(self):
+        """Workflow controls for transition to * not created by
+        ConfigurableTicketWorkflow.
+        """
+        env = self._testenv.get_trac_environment()
+        try:
+            env.config.set('ticket-workflow', 'triage', 'new -> *')
+            env.config.save()
+
+            tkt = self._tester.create_ticket()
+            tc.notfind('<label for="action_triage">triage</label>')
+        finally:
+            env.config.remove('ticket-workflow', 'triage')
+            env.config.save()
+
+
 def functionalSuite(suite=None):
     if not suite:
         import trac.tests.functional
@@ -378,6 +398,7 @@ def functionalSuite(suite=None):
     suite.addTest(MaySetOwnerOperationDefaultRestrictOwnerNone())
     suite.addTest(MaySetOwnerOperationDefaultRestrictOwnerAnonymous())
     suite.addTest(RegressionTestTicket11930())
+    suite.addTest(RegressionTestTicket13087())
 
     return suite
 
