@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2018 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 from datetime import timedelta
@@ -18,13 +18,14 @@ from trac.test import locale_en
 from trac.ticket.query import QueryModule
 from trac.ticket.report import ReportModule
 from trac.ticket.roadmap import RoadmapModule
-from trac.ticket.model import Milestone, Ticket
+from trac.ticket.model import Milestone
+from trac.ticket.test import insert_ticket
 from trac.util.datefmt import (datetime_now, format_datetime, pretty_timedelta,
                                utc)
 from trac.wiki.tests import formatter
 
 
-TICKET_TEST_CASES = u"""
+TICKET_TEST_CASES = """
 ============================== ticket: link resolver
 bug:1
 issue:1
@@ -106,10 +107,10 @@ trac:ticket:2041
 #trac2041
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in Trac's Trac"><span class="icon"></span>trac:ticket:2041</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in Trac's Trac"><span class="icon"></span>Trac #2041</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in Trac's Trac"><span class="icon"></span>#T2041</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in Trac's Trac"><span class="icon"></span>#trac2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in The Trac Project"><span class="icon"></span>trac:ticket:2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in The Trac Project"><span class="icon"></span>Trac #2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in The Trac Project"><span class="icon"></span>#T2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in The Trac Project"><span class="icon"></span>#trac2041</a>
 </p>
 ------------------------------
 ============================== Ticket InterTrac shorthands
@@ -117,8 +118,8 @@ T:#2041
 trac:#2041
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/%232041" title="#2041 in Trac's Trac"><span class="icon"></span>T:#2041</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/%232041" title="#2041 in Trac's Trac"><span class="icon"></span>trac:#2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/%232041" title="#2041 in The Trac Project"><span class="icon"></span>T:#2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/%232041" title="#2041 in The Trac Project"><span class="icon"></span>trac:#2041</a>
 </p>
 ------------------------------
 ============================== ticket syntax with unicode digits
@@ -143,12 +144,8 @@ def ticket_setup(tc):
     config = tc.env.config
     config.set('ticket-custom', 'custom1', 'text')
     config.save()
-    ticket = Ticket(tc.env)
-
-    ticket.populate({'reporter': 'santa',
-                     'summary': 'This is the summary',
-                     'status': 'new'})
-    ticket.insert()
+    insert_ticket(tc.env, reporter='santa', summary='This is the summary',
+                  status='new')
 
 def ticket_teardown(tc):
     config = tc.env.config
@@ -158,7 +155,7 @@ def ticket_teardown(tc):
 
 
 
-REPORT_TEST_CASES = u"""
+REPORT_TEST_CASES = """
 ============================== report link shorthand form
 {1}, {2}
 {12}, {abc}
@@ -200,11 +197,11 @@ trac:report:1
 {trac 1}
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>trac:report:1</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>Trac r1</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>{T1}</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>{trac1}</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>{trac 1}</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>trac:report:1</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>Trac r1</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>{T1}</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>{trac1}</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>{trac 1}</a>
 </p>
 ------------------------------
 ============================== report syntax with unicode digits
@@ -233,7 +230,7 @@ dt_past = datetime_now(utc) - timedelta(days=1)
 dt_future = datetime_now(utc) + timedelta(days=1)
 
 
-MILESTONE_TEST_CASES = u"""
+MILESTONE_TEST_CASES = """
 ============================== milestone: link resolver
 milestone:foo
 [milestone:boo Milestone Boo]
@@ -290,7 +287,7 @@ def milestone_teardown(tc):
 
 
 
-QUERY_TEST_CASES = u"""
+QUERY_TEST_CASES = """
 ============================== query: link resolver
 query:?order=priority
 
@@ -410,7 +407,7 @@ New tickets: <span><a class="new" href="/ticket/1" title="This is the summary">#
 ------------------------------
 """
 
-QUERY2_TEST_CASES = u"""
+QUERY2_TEST_CASES = """
 ============================== TicketQuery macro: two results, list form
 New tickets: [[TicketQuery(status=new, order=reporter)]]
 ------------------------------
@@ -442,22 +439,16 @@ New tickets: <span><a class="new" href="/ticket/2" title="This is another summar
 """
 
 def query2_setup(tc):
-    ticket = Ticket(tc.env)
-    ticket.populate({'reporter': 'santa',
-                     'summary': 'This is the summary',
-                     'status': 'new'})
-    ticket.insert()
-    ticket = Ticket(tc.env)
-    ticket.populate({'reporter': 'claus',
-                     'summary': 'This is another summary',
-                     'status': 'new'})
-    ticket.insert()
+    insert_ticket(tc.env, reporter='santa', summary='This is the summary',
+                  status='new')
+    insert_ticket(tc.env, reporter='claus', summary='This is another summary',
+                  status='new')
 
 def query2_teardown(tc):
     tc.env.reset_db()
 
 
-COMMENT_TEST_CASES = u"""
+COMMENT_TEST_CASES = """
 ============================== comment: link resolver (deprecated)
 comment:ticket:1:1 (deprecated)
 [comment:ticket:1:1 see above] (deprecated)
@@ -563,17 +554,13 @@ comment::ticket:
 # to fix it for now.
 
 def comment_setup(tc):
-    ticket1 = Ticket(tc.env)
-    ticket1.populate({'reporter': 'santa',
-                      'summary': 'This is the summary for ticket 1',
-                      'status': 'new'})
-    ticket1.insert()
+    ticket1 = insert_ticket(tc.env, reporter='santa',
+                            summary='This is the summary for ticket 1',
+                            status='new')
     ticket1.save_changes(comment='This is the comment for ticket 1')
-    ticket2 = Ticket(tc.env)
-    ticket2.populate({'reporter': 'claws',
-                      'summary': 'This is the summary for ticket 2',
-                      'status': 'closed'})
-    ticket2.insert()
+    ticket2 = insert_ticket(tc.env, reporter='claws',
+                            summary='This is the summary for ticket 2',
+                            status='closed')
     ticket2.save_changes(comment='This is the comment for ticket 2')
 
 def comment_teardown(tc):
