@@ -1,19 +1,19 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015 Edgewall Software
+# Copyright (C) 2015-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 import unittest
 
-from trac.upgrades.tests import db31, db32, db39, db41
+from trac.upgrades.tests import db31, db32, db39, db41, db42, db44, db45
 
 
 def test_suite():
@@ -22,6 +22,9 @@ def test_suite():
     suite.addTest(db32.test_suite())
     suite.addTest(db39.test_suite())
     suite.addTest(db41.test_suite())
+    suite.addTest(db42.test_suite())
+    suite.addTest(db44.test_suite())
+    suite.addTest(db45.test_suite())
     return suite
 
 
