@@ -1,17 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C)2006-2009 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
-from __future__ import with_statement
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import atexit
 import errno
@@ -21,14 +19,14 @@ import sys
 
 
 def daemonize(pidfile=None, progname=None, stdin='/dev/null',
-              stdout='/dev/null', stderr='/dev/null', umask=022):
+              stdout='/dev/null', stderr='/dev/null', umask=0o22):
     """Fork a daemon process."""
     if pidfile:
         # Check whether the pid file already exists and refers to a still
         # process running
         pidfile = os.path.abspath(pidfile)
         if os.path.exists(pidfile):
-            with open(pidfile) as fileobj:
+            with open(pidfile, 'rb') as fileobj:
                 try:
                     pid = int(fileobj.read())
                 except ValueError:
@@ -40,15 +38,15 @@ def daemonize(pidfile=None, progname=None, stdin='/dev/null',
                 if not progname:
                     progname = os.path.basename(sys.argv[0])
                 sys.exit('%s is already running with pid %s' % (progname, pid))
-            except OSError, e:
+            except OSError as e:
                 if e.errno != errno.ESRCH:
                     raise
 
         # The pid file must be writable
         try:
-            fileobj = open(pidfile, 'a+')
-            fileobj.close()
-        except IOError, e:
+            with open(pidfile, 'a+'):
+                pass
+        except IOError as e:
             from trac.util.text import exception_to_unicode
             sys.exit('Error writing to pid file: %s' % exception_to_unicode(e))
 
@@ -70,9 +68,9 @@ def daemonize(pidfile=None, progname=None, stdin='/dev/null',
     # The process is now daemonized, redirect standard file descriptors
     for stream in sys.stdout, sys.stderr:
         stream.flush()
-    stdin = open(stdin, 'r')
-    stdout = open(stdout, 'a+')
-    stderr = open(stderr, 'a+', 0)
+    stdin = os.open(stdin, 'rb')
+    stdout = os.open(stdout, 'ab+')
+    stderr = os.open(stderr, 'ab+', 0)
     os.dup2(stdin.fileno(), sys.stdin.fileno())
     os.dup2(stdout.fileno(), sys.stdout.fileno())
     os.dup2(stderr.fileno(), sys.stderr.fileno())
@@ -88,8 +86,8 @@ def daemonize(pidfile=None, progname=None, stdin='/dev/null',
             if os.path.exists(pidfile):
                 os.remove(pidfile)
         atexit.register(remove_pidfile)
-        with open(pidfile, 'w') as fileobj:
-            fileobj.write(str(os.getpid()))
+        with open(pidfile, 'wb') as fileobj:
+            fileobj.write(b'%d' % os.getpid())
 
 
 def handle_signal(signum, frame):
