@@ -1,86 +1,89 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2018 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
+import io
 import os
+import re
 import socket
+import textwrap
 import unittest
-from StringIO import StringIO
 
-import trac.tests.compat
+import jinja2
+
 from trac.util.text import (
-    _get_default_ambiwidth, empty, expandtabs, fix_eol, javascript_quote,
-    levenshtein_distance, normalize_whitespace, print_table,
-    quote_query_string, shorten_line, strip_line_ws, stripws, sub_vars,
-    text_width, to_js_string, to_unicode, to_utf8, unicode_from_base64,
-    unicode_quote, unicode_quote_plus, unicode_to_base64, unicode_unquote,
-    unicode_urlencode, wrap)
+    _get_default_ambiwidth, empty, exception_to_unicode, expandtabs, fix_eol,
+    javascript_quote, jinja2template, levenshtein_distance,
+    normalize_whitespace, print_table, quote_query_string, shorten_line,
+    strip_line_ws, stripws, sub_vars, text_width, to_js_string, to_unicode,
+    to_utf8, unicode_from_base64, unicode_quote, unicode_quote_plus,
+    unicode_to_base64, unicode_unquote, unicode_urlencode, wrap)
 
 
 class ToUnicodeTestCase(unittest.TestCase):
     def test_explicit_charset(self):
-        uc = to_unicode('\xc3\xa7', 'utf-8')
-        self.assertIsInstance(uc, unicode)
-        self.assertEqual(u'\xe7', uc)
+        uc = to_unicode(b'\xc3\xa7', 'utf-8')
+        self.assertIsInstance(uc, str)
+        self.assertEqual('\xe7', uc)
 
     def test_explicit_charset_with_replace(self):
-        uc = to_unicode('\xc3', 'utf-8')
-        self.assertIsInstance(uc, unicode)
-        self.assertEqual(u'\xc3', uc)
+        uc = to_unicode(b'\xc3', 'utf-8')
+        self.assertIsInstance(uc, str)
+        self.assertEqual('\xc3', uc)
 
     def test_implicit_charset(self):
-        uc = to_unicode('\xc3\xa7')
-        self.assertIsInstance(uc, unicode)
-        self.assertEqual(u'\xe7', uc)
+        uc = to_unicode(b'\xc3\xa7')
+        self.assertIsInstance(uc, str)
+        self.assertEqual('\xe7', uc)
 
     def test_from_exception_using_unicode_args(self):
-        u = u'\uB144'
+        u = '\uB144'
         try:
-            raise ValueError, '%s is not a number.' % u
+            raise ValueError('%s is not a number.' % u)
         except ValueError as e:
-            self.assertEqual(u'\uB144 is not a number.', to_unicode(e))
-
-    def test_from_exception_using_str_args(self):
-        u = u'Das Ger\xe4t oder die Ressource ist belegt'
-        try:
-            raise ValueError, u.encode('utf-8')
-        except ValueError as e:
-            self.assertEqual(u, to_unicode(e))
+            self.assertEqual('\uB144 is not a number.', to_unicode(e))
 
+    @unittest.skipIf(os.name != 'nt', 'For OSError on Windows')
     def test_from_windows_error(self):
         try:
-            os.stat('non/existent/file.txt')
+            os.stat(r'non\existent\file.txt')
         except OSError as e:
             uc = to_unicode(e)
-            self.assertIsInstance(uc, unicode, uc)
-            self.assertTrue(uc.startswith('[Error '), uc)
-            self.assertIn(e.strerror.decode('mbcs'), uc)
+            self.assertIsInstance(uc, str, uc)
+            self.assertIsInstance(e.strerror, str)
+            self.assertIn(r": 'non\existent\file.txt'", uc)
 
+    @unittest.skipIf(os.name != 'nt', 'For OSError on Windows')
+    def test_from_windows_error_with_unicode_path(self):
+        try:
+            os.stat(r'nön\existént\file.txt')
+        except OSError as e:
+            uc = to_unicode(e)
+            self.assertIsInstance(uc, str, uc)
+            self.assertIsInstance(e.strerror, str)
+            self.assertIn(r": 'nön\existént\file.txt'", uc)
+
+    @unittest.skipIf(os.name != 'nt', 'For OSError on Windows')
     def test_from_socket_error(self):
         for res in socket.getaddrinfo('127.0.0.1', 65536, 0,
                                       socket.SOCK_STREAM):
             af, socktype, proto, canonname, sa = res
-            s = socket.socket(af, socktype, proto)
-            try:
-                s.connect(sa)
-            except socket.error as e:
-                uc = to_unicode(e)
-                self.assertIsInstance(uc, unicode, uc)
-                if hasattr(e, 'strerror'):
-                    self.assertIn(e.strerror.decode('mbcs'), uc)
-
-    if os.name != 'nt':
-        del test_from_windows_error
-        del test_from_socket_error
+            with socket.socket(af, socktype, proto) as s:
+                try:
+                    s.connect(sa)
+                except socket.error as e:
+                    uc = to_unicode(e)
+                    self.assertIsInstance(uc, str, uc)
+                    self.assertIsInstance(e.strerror, str)
 
 
 class ExpandtabsTestCase(unittest.TestCase):
@@ -106,7 +109,7 @@ class JavascriptQuoteTestCase(unittest.TestCase):
         self.assertEqual(r'\u0026\u003c\u003e',
                          javascript_quote('&<>'))
         self.assertEqual(r'\u2028\u2029',
-                         javascript_quote(u'\u2028\u2029'))
+                         javascript_quote('\u2028\u2029'))
 
 
 class ToJsStringTestCase(unittest.TestCase):
@@ -124,73 +127,75 @@ class ToJsStringTestCase(unittest.TestCase):
         self.assertEqual('""',
                          to_js_string(None))
         self.assertEqual(r'"\u2028\u2029"',
-                         to_js_string(u'\u2028\u2029'))
+                         to_js_string('\u2028\u2029'))
 
 
 class UnicodeQuoteTestCase(unittest.TestCase):
     def test_unicode_quote(self):
-        self.assertEqual(u'the%20%C3%9C%20thing',
-                         unicode_quote(u'the Ü thing'))
-        self.assertEqual(u'%2520%C3%9C%20%2520',
-                         unicode_quote(u'%20Ü %20'))
+        self.assertEqual('the%20%C3%9C%20thing',
+                         unicode_quote('the Ü thing'))
+        self.assertEqual('%2520%C3%9C%20%2520',
+                         unicode_quote('%20Ü %20'))
 
     def test_unicode_quote_plus(self):
-        self.assertEqual(u'the+%C3%9C+thing',
-                         unicode_quote_plus(u'the Ü thing'))
-        self.assertEqual(u'%2520%C3%9C+%2520',
-                         unicode_quote_plus(u'%20Ü %20'))
+        self.assertEqual('the+%C3%9C+thing',
+                         unicode_quote_plus('the Ü thing'))
+        self.assertEqual('%2520%C3%9C+%2520',
+                         unicode_quote_plus('%20Ü %20'))
 
     def test_unicode_unquote(self):
-        u = u'the Ü thing'
-        up = u'%20Ü %20'
+        u = 'the Ü thing'
+        up = '%20Ü %20'
         self.assertEqual(u, unicode_unquote(unicode_quote(u)))
         self.assertEqual(up, unicode_unquote(unicode_quote(up)))
 
     def test_unicode_urlencode(self):
         self.assertEqual('thing=%C3%9C&%C3%9C=thing&%C3%9Cthing',
-                         unicode_urlencode({u'Ü': 'thing',
-                                            'thing': u'Ü',
-                                            u'Üthing': empty}))
+                         unicode_urlencode({'Ü': 'thing',
+                                            'thing': 'Ü',
+                                            'Üthing': empty}))
 
 
 class QuoteQueryStringTestCase(unittest.TestCase):
     def test_quote(self):
-        text = u'type=the Ü thing&component=comp\x7fonent'
+        text = 'type=the Ü thing&component=comp\x7fonent'
         self.assertEqual('type=the+%C3%9C+thing&component=comp%7Fonent',
                          quote_query_string(text))
 
 
 class ToUtf8TestCase(unittest.TestCase):
     def test_unicode(self):
-        self.assertEqual('à', to_utf8('à'))
-        self.assertEqual('ç', to_utf8('ç'))
+        self.assertEqual('à'.encode('utf-8'), to_utf8('à'))
+        self.assertEqual('ç'.encode('utf-8'), to_utf8('ç'))
 
     def test_boolean(self):
-        self.assertEqual('True', to_utf8(True))
-        self.assertEqual('False', to_utf8(False))
+        self.assertEqual(b'True', to_utf8(True))
+        self.assertEqual(b'False', to_utf8(False))
 
     def test_int(self):
-        self.assertEqual('-1', to_utf8(-1))
-        self.assertEqual('0', to_utf8(0))
-        self.assertEqual('1', to_utf8(1))
+        self.assertEqual(b'-1', to_utf8(-1))
+        self.assertEqual(b'0', to_utf8(0))
+        self.assertEqual(b'1', to_utf8(1))
 
     def test_utf8(self):
-        self.assertEqual('à', to_utf8(u'à'))
-        self.assertEqual('ç', to_utf8(u'ç'))
+        self.assertEqual('à'.encode('utf-8'), to_utf8('à'))
+        self.assertEqual('ç'.encode('utf-8'), to_utf8('ç'))
 
     def test_exception_with_utf8_message(self):
-        self.assertEqual('thė mèssägē', to_utf8(Exception('thė mèssägē')))
+        self.assertEqual('thė mèssägē'.encode('utf-8'),
+                         to_utf8(Exception('thė mèssägē')))
 
     def test_exception_with_unicode_message(self):
-        self.assertEqual('thė mèssägē', to_utf8(Exception(u'thė mèssägē')))
+        self.assertEqual('thė mèssägē'.encode('utf-8'),
+                         to_utf8(Exception('thė mèssägē')))
 
 
 class WhitespaceTestCase(unittest.TestCase):
     def test_default(self):
-        self.assertEqual(u'This is text ',
-            normalize_whitespace(u'Th\u200bis\u00a0is te\u200bxt\u00a0'))
-        self.assertEqual(u'Some other text',
-            normalize_whitespace(u'Some\tother\ntext\r', to_space='\t\n',
+        self.assertEqual('This is text ',
+            normalize_whitespace('Th\u200bis\u00a0is te\u200bxt\u00a0'))
+        self.assertEqual('Some other text',
+            normalize_whitespace('Some\tother\ntext\r', to_space='\t\n',
                                  remove='\r'))
 
 
@@ -198,18 +203,18 @@ class TextWidthTestCase(unittest.TestCase):
     def test_single(self):
         def tw1(text):
             return text_width(text, ambiwidth=1)
-        self.assertEqual(8, tw1(u'Alphabet'))
+        self.assertEqual(8, tw1('Alphabet'))
         self.assertEqual(16, tw1('east asian width'))
-        self.assertEqual(16, tw1(u'ひらがなカタカナ'))
-        self.assertEqual(21, tw1(u'色は匂えど…酔ひもせず'))
+        self.assertEqual(16, tw1('ひらがなカタカナ'))
+        self.assertEqual(21, tw1('色は匂えど…酔ひもせず'))
 
     def test_double(self):
         def tw2(text):
             return text_width(text, ambiwidth=2)
-        self.assertEqual(8, tw2(u'Alphabet'))
+        self.assertEqual(8, tw2('Alphabet'))
         self.assertEqual(16, tw2('east asian width'))
-        self.assertEqual(16, tw2(u'ひらがなカタカナ'))
-        self.assertEqual(22, tw2(u'色は匂えど…酔ひもせず'))
+        self.assertEqual(16, tw2('ひらがなカタカナ'))
+        self.assertEqual(22, tw2('色は匂えど…酔ひもせず'))
 
 
 class PrintTableTestCase(unittest.TestCase):
@@ -225,20 +230,20 @@ class PrintTableTestCase(unittest.TestCase):
             ('Trac 0.5',  '2004-02-23', 'Incognito'),
         )
         headers = ('Version', 'Date', 'Name')
-        expected = """\
+        expected = textwrap.dedent("""\
 
-Version     Date         Name
-----------------------------------
-Trac 0.12 | 2010-06-13 | Babel
-Trac 0.11 | 2008-06-22 | Genshi
-Trac 0.10 | 2006-09-28 | Zengia
-Trac 0.9  | 2005-10-31 | Vodun
-Trac 0.8  | 2004-11-15 | Qualia
-Trac 0.7  | 2004-05-18 | Fulci
-Trac 0.6  | 2004-03-23 | Solanum
-Trac 0.5  | 2004-02-23 | Incognito
+            Version     Date         Name
+            ----------------------------------
+            Trac 0.12 | 2010-06-13 | Babel
+            Trac 0.11 | 2008-06-22 | Genshi
+            Trac 0.10 | 2006-09-28 | Zengia
+            Trac 0.9  | 2005-10-31 | Vodun
+            Trac 0.8  | 2004-11-15 | Qualia
+            Trac 0.7  | 2004-05-18 | Fulci
+            Trac 0.6  | 2004-03-23 | Solanum
+            Trac 0.5  | 2004-02-23 | Incognito
 
-"""
+            """)
         self._validate_print_table(expected, data, headers=headers, sep=' | ',
                                    ambiwidth=1)
 
@@ -250,48 +255,48 @@ Trac 0.5  | 2004-02-23 | Incognito
             ('int',      '0',     0),
             ('float',    '0.0',   0.0),
         )
-        expected = u"""\
+        expected = textwrap.dedent("""\
 
-NoneType | None  |
-bool     | True  | True
-bool     | False | False
-int      | 0     | 0
-float    | 0.0   | 0.0
+            NoneType | None  |
+            bool     | True  | True
+            bool     | False | False
+            int      | 0     | 0
+            float    | 0.0   | 0.0
 
-"""
+            """)
         self._validate_print_table(expected, data, sep=' | ', ambiwidth=1)
 
     def test_ambiwidth_1(self):
         data = (
             ('foo@localhost', 'foo@localhost'),
-            (u'bar@….com', 'bar@example.com'),
+            ('bar@….com', 'bar@example.com'),
         )
         headers = ('Obfuscated', 'Email')
-        expected = u"""\
+        expected = textwrap.dedent("""\
 
-Obfuscated      Email
--------------------------------
-foo@localhost | foo@localhost
-bar@….com     | bar@example.com
+            Obfuscated      Email
+            -------------------------------
+            foo@localhost | foo@localhost
+            bar@….com     | bar@example.com
 
-"""
+            """)
         self._validate_print_table(expected, data, headers=headers, sep=' | ',
                                    ambiwidth=1)
 
     def test_ambiwidth_2(self):
         data = (
             ('foo@localhost', 'foo@localhost'),
-            (u'bar@….com', 'bar@example.com'),
+            ('bar@….com', 'bar@example.com'),
         )
         headers = ('Obfuscated', 'Email')
-        expected = u"""\
+        expected = textwrap.dedent("""\
 
-Obfuscated      Email
--------------------------------
-foo@localhost | foo@localhost
-bar@….com    | bar@example.com
+            Obfuscated      Email
+            -------------------------------
+            foo@localhost | foo@localhost
+            bar@….com    | bar@example.com
 
-"""
+            """)
         self._validate_print_table(expected, data, headers=headers, sep=' | ',
                                    ambiwidth=2)
 
@@ -302,80 +307,80 @@ bar@….com    | bar@example.com
             (43, 'alfa\r\nbravo\r\n', 'zero\r\none\r\ntwo'),
         )
         headers = ('Id', 'Column 1', 'Column 2')
-        expected = u"""\
+        expected = textwrap.dedent("""\
 
-Id   Column 1   Column 2
---------------------------
-41 | Trac     | Trac-Hacks
-42 | blah     | foo
-   |          | bar
-43 | alfa     | zero
-   | bravo    | one
-   |          | two
+            Id   Column 1   Column 2
+            --------------------------
+            41 | Trac     | Trac-Hacks
+            42 | blah     | foo
+               |          | bar
+            43 | alfa     | zero
+               | bravo    | one
+               |          | two
 
-"""
+            """)
         self._validate_print_table(expected, data, headers=headers, sep=' | ')
 
     def _validate_print_table(self, expected, data, **kwargs):
-        out = StringIO()
+        out = io.StringIO()
         kwargs['out'] = out
         print_table(data, **kwargs)
-        self.assertEqual(expected.encode('utf-8'),
+        self.assertEqual(expected,
                          strip_line_ws(out.getvalue(), leading=False))
 
 
 class WrapTestCase(unittest.TestCase):
     def test_wrap_ambiwidth_single(self):
-        text = u'Lorem ipsum dolor sit amet, consectetur adipisicing ' + \
-               u'elit, sed do eiusmod tempor incididunt ut labore et ' + \
-               u'dolore magna aliqua. Ut enim ad minim veniam, quis ' + \
-               u'nostrud exercitation ullamco laboris nisi ut aliquip ex ' + \
-               u'ea commodo consequat. Duis aute irure dolor in ' + \
-               u'reprehenderit in voluptate velit esse cillum dolore eu ' + \
-               u'fugiat nulla pariatur. Excepteur sint occaecat ' + \
-               u'cupidatat non proident, sunt in culpa qui officia ' + \
-               u'deserunt mollit anim id est laborum.'
-        wrapped = u"""\
-> Lorem ipsum dolor sit amet, consectetur adipisicing elit,
-| sed do eiusmod tempor incididunt ut labore et dolore
-| magna aliqua. Ut enim ad minim veniam, quis nostrud
-| exercitation ullamco laboris nisi ut aliquip ex ea
-| commodo consequat. Duis aute irure dolor in reprehenderit
-| in voluptate velit esse cillum dolore eu fugiat nulla
-| pariatur. Excepteur sint occaecat cupidatat non proident,
-| sunt in culpa qui officia deserunt mollit anim id est
-| laborum."""
+        text = 'Lorem ipsum dolor sit amet, consectetur adipisicing ' + \
+               'elit, sed do eiusmod tempor incididunt ut labore et ' + \
+               'dolore magna aliqua. Ut enim ad minim veniam, quis ' + \
+               'nostrud exercitation ullamco laboris nisi ut aliquip ex ' + \
+               'ea commodo consequat. Duis aute irure dolor in ' + \
+               'reprehenderit in voluptate velit esse cillum dolore eu ' + \
+               'fugiat nulla pariatur. Excepteur sint occaecat ' + \
+               'cupidatat non proident, sunt in culpa qui officia ' + \
+               'deserunt mollit anim id est laborum.'
+        wrapped = textwrap.dedent("""\
+            > Lorem ipsum dolor sit amet, consectetur adipisicing elit,
+            | sed do eiusmod tempor incididunt ut labore et dolore
+            | magna aliqua. Ut enim ad minim veniam, quis nostrud
+            | exercitation ullamco laboris nisi ut aliquip ex ea
+            | commodo consequat. Duis aute irure dolor in reprehenderit
+            | in voluptate velit esse cillum dolore eu fugiat nulla
+            | pariatur. Excepteur sint occaecat cupidatat non proident,
+            | sunt in culpa qui officia deserunt mollit anim id est
+            | laborum.""")
         self.assertEqual(wrapped, wrap(text, 59, '> ', '| ', '\n'))
 
     def test_wrap_ambiwidth_double(self):
-        text = u'Trac は BSD ライセンスのもとで配布されて' + \
-               u'います。[1:]このライセンスの全文は、𠀋' + \
-               u'配布ファイルに含まれている[3:CОPYING]ファ' + \
-               u'イルと同じものが[2:オンライン]で参照でき' \
-               u'ます。'
-        wrapped = u"""\
-> Trac は BSD ライセンスのもとで配布されています。[1:]この
-| ライセンスの全文は、𠀋配布ファイルに含まれている
-| [3:CОPYING]ファイルと同じものが[2:オンライン]で参照でき
-| ます。"""
+        text = 'Trac は BSD ライセンスのもとで配布されて' + \
+               'います。[1:]このライセンスの全文は、𠀋' + \
+               '配布ファイルに含まれている[3:CОPYING]ファ' + \
+               'イルと同じものが[2:オンライン]で参照でき' \
+               'ます。'
+        wrapped = textwrap.dedent("""\
+            > Trac は BSD ライセンスのもとで配布されています。[1:]この
+            | ライセンスの全文は、𠀋配布ファイルに含まれている
+            | [3:CОPYING]ファイルと同じものが[2:オンライン]で参照でき
+            | ます。""")
         self.assertEqual(wrapped, wrap(text, 59, '> ', '| ', '\n',
                                        ambiwidth=2))
 
 
 class FixEolTestCase(unittest.TestCase):
     def test_mixed_eol(self):
-        text = u'\nLine 2\rLine 3\r\nLine 4\n\r'
-        self.assertEqual(u'\nLine 2\nLine 3\nLine 4\n\n',
+        text = '\nLine 2\rLine 3\r\nLine 4\n\r'
+        self.assertEqual('\nLine 2\nLine 3\nLine 4\n\n',
                          fix_eol(text, '\n'))
-        self.assertEqual(u'\rLine 2\rLine 3\rLine 4\r\r',
+        self.assertEqual('\rLine 2\rLine 3\rLine 4\r\r',
                          fix_eol(text, '\r'))
-        self.assertEqual(u'\r\nLine 2\r\nLine 3\r\nLine 4\r\n\r\n',
+        self.assertEqual('\r\nLine 2\r\nLine 3\r\nLine 4\r\n\r\n',
                          fix_eol(text, '\r\n'))
 
 
 class UnicodeBase64TestCase(unittest.TestCase):
     def test_to_and_from_base64_unicode(self):
-        text = u'Trac は ØÆÅ'
+        text = 'Trac は ØÆÅ'
         text_base64 = unicode_to_base64(text)
         self.assertEqual('VHJhYyDjga8gw5jDhsOF', text_base64)
         self.assertEqual(text, unicode_from_base64(text_base64))
@@ -400,19 +405,47 @@ class UnicodeBase64TestCase(unittest.TestCase):
 
 class StripwsTestCase(unittest.TestCase):
     def test_stripws(self):
-        self.assertEqual(u'stripws',
-                         stripws(u' \u200b\t\u3000stripws \u200b\t\u2008'))
-        self.assertEqual(u'stripws \u3000\t',
-                         stripws(u'\u200b\t\u2008 stripws \u3000\t',
+        self.assertEqual('stripws',
+                         stripws(' \u200b\t\u3000stripws \u200b\t\u2008'))
+        self.assertEqual('stripws \u3000\t',
+                         stripws('\u200b\t\u2008 stripws \u3000\t',
                                  trailing=False))
-        self.assertEqual(u' \t\u3000stripws',
-                         stripws(u' \t\u3000stripws \u200b\t\u2008',
+        self.assertEqual(' \t\u3000stripws',
+                         stripws(' \t\u3000stripws \u200b\t\u2008',
                                  leading=False))
-        self.assertEqual(u' \t\u3000stripws \u200b\t\u2008',
-                         stripws(u' \t\u3000stripws \u200b\t\u2008',
+        self.assertEqual(' \t\u3000stripws \u200b\t\u2008',
+                         stripws(' \t\u3000stripws \u200b\t\u2008',
                                  leading=False, trailing=False))
 
 
+class Jinja2TemplateTestCase(unittest.TestCase):
+    def test_html_template(self):
+        self.assertEqual("<h1>Hell&amp;O</h1>",
+                         jinja2template("<h1>${hell}O</h1>")
+                         .render({'hell': 'Hell&'}))
+
+    def test_text_template(self):
+        self.assertEqual("<h1>Hell&O</h1>",
+                         jinja2template("<h1>${hell}O</h1>", text=True)
+                         .render({'hell': 'Hell&'}))
+
+    def test_text_template_line_statement_prefix_none(self):
+        with self.assertRaises(jinja2.TemplateSyntaxError):
+            self.assertEqual("",
+                jinja2template("#${id}", text=True).render({'id': 10}))
+        self.assertEqual("#10",
+             jinja2template("#${id}", text=True,
+                            line_statement_prefix=None).render({'id': 10}))
+
+    def test_text_template_line_comment_prefix_none(self):
+        self.assertEqual("",
+             jinja2template("##${id}", text=True,
+                            line_statement_prefix=None).render({'id': 10}))
+        self.assertEqual("##10",
+             jinja2template("##${id}", text=True, line_statement_prefix=None,
+                            line_comment_prefix=None).render({'id': 10}))
+
+
 class LevenshteinDistanceTestCase(unittest.TestCase):
     def test_distance(self):
         self.assertEqual(5, levenshtein_distance('kitten', 'sitting'))
@@ -447,6 +480,7 @@ class ShortenLineTestCase(unittest.TestCase):
         self.assertEqual('abcde ...', shorten_line(text, 9))
 
 
+@unittest.skipIf(os.name == 'nt', 'POSIX locale is not available')
 class DefaultAmbiwidthTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -512,6 +546,28 @@ class DefaultAmbiwidthTestCase(unittest.TestCase):
         self.assertEqual(1, _get_default_ambiwidth())
 
 
+class ExceptionToUnicodeTestCase(unittest.TestCase):
+
+    def test_without_traceback(self):
+        try:
+            raise ValueError('test')
+        except ValueError as e:
+            self.assertEqual('ValueError: test', exception_to_unicode(e))
+
+    def test_with_traceback(self):
+        try:
+            raise ValueError('test')
+        except ValueError as e:
+            result = exception_to_unicode(e, traceback=True)
+            result = re.sub(r'\n  File "[^"]+", line [0-9]+, ',
+                            '\n  File "<file>", line <line>, ', result)
+            self.assertEqual("""
+Traceback (most recent call last):
+  File "<file>", line <line>, in test_with_traceback
+    raise ValueError('test')
+ValueError: test""", result)
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(ToUnicodeTestCase))
@@ -528,11 +584,12 @@ def test_suite():
     suite.addTest(unittest.makeSuite(FixEolTestCase))
     suite.addTest(unittest.makeSuite(UnicodeBase64TestCase))
     suite.addTest(unittest.makeSuite(StripwsTestCase))
+    suite.addTest(unittest.makeSuite(Jinja2TemplateTestCase))
     suite.addTest(unittest.makeSuite(LevenshteinDistanceTestCase))
     suite.addTest(unittest.makeSuite(SubVarsTestCase))
     suite.addTest(unittest.makeSuite(ShortenLineTestCase))
-    if os.name != 'nt':
-        suite.addTest(unittest.makeSuite(DefaultAmbiwidthTestCase))
+    suite.addTest(unittest.makeSuite(DefaultAmbiwidthTestCase))
+    suite.addTest(unittest.makeSuite(ExceptionToUnicodeTestCase))
     return suite
 
 
