@@ -1,17 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013 Edgewall Software
+# Copyright (C) 2013-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
-import tempfile
 import unittest
 from pkg_resources import resource_exists, resource_filename
 try:
@@ -25,7 +24,7 @@ else:
     except ImportError:
         from babel.localedata import list as locale_identifiers
 
-from trac.test import EnvironmentStub
+from trac.test import EnvironmentStub, mkdtemp
 from trac.util import translation
 
 
@@ -33,7 +32,7 @@ class TranslationsProxyTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub()
-        self.env.path = tempfile.mkdtemp(prefix='trac-tempenv-')
+        self.env.path = mkdtemp()
 
     def tearDown(self):
         translation.deactivate()
