@@ -1,22 +1,21 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2020 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 from datetime import datetime
 
-import trac.tests.compat
 from trac.test import EnvironmentStub, Mock
 from trac.util.datefmt import to_utimestamp, utc
 from trac.versioncontrol import Repository, Changeset, Node, NoSuchChangeset
@@ -51,8 +50,8 @@ class CacheTestCase(unittest.TestCase):
                     get_oldest_rev=lambda: 0,
                     get_youngest_rev=lambda: youngest_rev,
                     normalize_rev=lambda x: get_changeset(x).rev,
-                    next_rev=(lambda x: int(x) < youngest_rev and x + 1 \
-                              or None))
+                    next_rev=(lambda x: int(x) < youngest_rev and x + 1
+                                        or None))
 
     def preset_cache(self, *args):
         """Each arg is a (rev tuple, changes list of tuples) pair."""
@@ -273,10 +272,10 @@ class CacheTestCase(unittest.TestCase):
         cset_changes = cset.get_changes()
         self.assertEqual(('branches', Node.DIRECTORY, Changeset.ADD, None,
                           None),
-                         cset_changes.next())
+                         next(cset_changes))
         self.assertEqual(('tags', Node.DIRECTORY, Changeset.ADD, None, None),
-                         cset_changes.next())
-        self.assertRaises(StopIteration, cset_changes.next)
+                         next(cset_changes))
+        self.assertRaises(StopIteration, next, cset_changes)
 
         rows = self.env.db_query(
                 "SELECT time,author,message FROM revision ORDER BY rev")
@@ -306,15 +305,15 @@ class CacheTestCase(unittest.TestCase):
         cache = MockCachedRepository(self.env, repos, self.log)
 
         cache.sync_changeset('0')   # not cached yet
-        cache.sync_changeset(u'1')  # not cached yet
+        cache.sync_changeset('1')  # not cached yet
         rows = self.env.db_query(
             "SELECT rev,author FROM revision ORDER BY rev")
         self.assertEqual(2, len(rows))
         self.assertEqual(('0000000000', 'joe'), rows[0])
         self.assertEqual(('0000000001', 'joe'), rows[1])
 
-        cache.sync_changeset(u'0')  # cached
-        cache.sync_changeset('1')   # cached
+        cache.sync_changeset('0')  # cached
+        cache.sync_changeset('1')  # cached
         rows = self.env.db_query(
             "SELECT rev,author FROM revision ORDER BY rev")
         self.assertEqual(2, len(rows))
@@ -339,10 +338,10 @@ class CacheTestCase(unittest.TestCase):
         self.assertEqual(t2, changeset.date)
         changes = changeset.get_changes()
         self.assertEqual(('trunk', Node.DIRECTORY, Changeset.ADD, None, None),
-                         changes.next())
+                         next(changes))
         self.assertEqual(('trunk/RDME', Node.FILE, Changeset.ADD, None, None),
-                         changes.next())
-        self.assertRaises(StopIteration, changes.next)
+                         next(changes))
+        self.assertRaises(StopIteration, next, changes)
 
 
 def test_suite():
