@@ -1,67 +1,80 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os.path
-import tempfile
+import textwrap
 import unittest
 
 from trac.config import ConfigurationError
 from trac.resource import Resource
-from trac.test import EnvironmentStub, Mock
+from trac.test import EnvironmentStub, Mock, mkdtemp, rmtree
 from trac.util import create_file
 from trac.versioncontrol.api import RepositoryManager
-from trac.versioncontrol.svn_authz import AuthzSourcePolicy, ParseError, \
-                                          parse
+from trac.versioncontrol.svn_authz import AuthzSourcePolicy, parse
 
 
 class AuthzParserTestCase(unittest.TestCase):
 
+    def setUp(self):
+        self.tmpdir = mkdtemp()
+        self.authz_file = os.path.join(self.tmpdir, 'trac-authz')
+
+    def tearDown(self):
+        rmtree(self.tmpdir)
+
     def test_parse_file(self):
-        authz = parse("""\
-[groups]
-developers = foo, bar
-users = @developers, &baz
+        create_file(self.authz_file, textwrap.dedent("""\
+            [groups]
+            developers = foo, bar
+            users = @developers, &baz
 
-[aliases]
-baz = CN=Hàröld Hacker,OU=Enginéers,DC=red-bean,DC=com
+            [aliases]
+            baz = CN=Hàröld Hacker,OU=Enginéers,DC=red-bean,DC=com
 
-# Applies to all repositories
-[/]
-* = r
+            # Applies to all repositories
+            [/]
+            * = r
 
-[/trunk]
-@developers = rw
-&baz =
-@users = r
+            [/trunk]
+            @developers = rw
+            &baz =
+            @users = r
 
-[/branches]
-bar = rw
+            [/branches]
+            bar = rw
 
-; Applies only to module
-[module:/trunk]
-foo = rw
-&baz = r
+            ; Applies only to module
+            [module:/trunk]
+            foo = rw
+            &baz = r
 
-; Unicode module names
-[module:/c/résumé]
-bar = rw
-Foo = rw
-BAZ = r
+            ; Unicode module names
+            [module:/c/résumé]
+            bar = rw
+            Foo = rw
+            BAZ = r
 
-; Unused module, not parsed
-[unused:/some/path]
-foo = r
-""", set(['', 'module']))
+            ; Unicode module names
+            [module:/c/résumé]
+            bar = rw
+            Foo = rw
+            BAZ = r
+
+            ; Unused module, not parsed
+            [unused:/some/path]
+            foo = r
+            """))
+        authz = parse(self.authz_file, {'', 'module'})
         self.assertEqual({
             '': {
                 '/': {
@@ -70,7 +83,7 @@ foo = r
                 '/trunk': {
                     'foo': True,
                     'bar': True,
-                    u'CN=Hàröld Hacker,OU=Enginéers,DC=red-bean,DC=com': True,
+                    'CN=Hàröld Hacker,OU=Enginéers,DC=red-bean,DC=com': True,
                 },
                 '/branches': {
                     'bar': True,
@@ -79,190 +92,177 @@ foo = r
             'module': {
                 '/trunk': {
                     'foo': True,
-                    u'CN=Hàröld Hacker,OU=Enginéers,DC=red-bean,DC=com': True,
+                    'CN=Hàröld Hacker,OU=Enginéers,DC=red-bean,DC=com': True,
                 },
-                u'/c/résumé': {
-                    u'bar': True,
-                    u'Foo': True,
-                    u'BAZ': True,
+                '/c/résumé': {
+                    'bar': True,
+                    'Foo': True,
+                    'BAZ': True,
                 },
             },
         }, authz)
 
-    def test_parse_errors(self):
-        self.assertRaises(ParseError, parse, """\
-user = r
-
-[module:/trunk]
-user = r
-""", set(['', 'module']))
-        self.assertRaises(ParseError, parse, """\
-[module:/trunk]
-user
-""", set(['', 'module']))
-
 
 class AuthzSourcePolicyTestCase(unittest.TestCase):
 
     def setUp(self):
-        tmpdir = tempfile.mkdtemp(prefix='trac-')
-        self.authz = os.path.join(tmpdir, 'trac-authz')
-        create_file(self.authz, """\
-[groups]
-group1 = user
-group2 = @group1
+        tmpdir = mkdtemp()
+        self.authz_file = os.path.join(tmpdir, 'trac-authz')
+        create_file(self.authz_file, textwrap.dedent("""\
+            [groups]
+            group1 = user
+            group2 = @group1
 
-cycle1 = @cycle2
-cycle2 = @cycle3
-cycle3 = @cycle1, user
+            cycle1 = @cycle2
+            cycle2 = @cycle3
+            cycle3 = @cycle1, user
 
-alias1 = &jekyll
-alias2 = @alias1
+            alias1 = &jekyll
+            alias2 = @alias1
 
-[aliases]
-jekyll = Mr Hyde
+            [aliases]
+            jekyll = Mr Hyde
 
-# Read / write permissions
-[/readonly]
-user = r
-[/writeonly]
-user = w
-[/readwrite]
-user = rw
-[/empty]
-user =
+            # Read / write permissions
+            [/readonly]
+            user = r
+            [/writeonly]
+            user = w
+            [/readwrite]
+            user = rw
+            [/empty]
+            user =
 
-# Trailing slashes
-[/trailing_a]
-user = r
-[/trailing_b/]
-user = r
+            # Trailing slashes
+            [/trailing_a]
+            user = r
+            [/trailing_b/]
+            user = r
 
-# Sub-paths
-[/sub/path]
-user = r
+            # Sub-paths
+            [/sub/path]
+            user = r
 
-# Module usage
-[module:/module_a]
-user = r
-[other:/module_b]
-user = r
-[/module_c]
-user = r
-[module:/module_d]
-user =
-[/module_d]
-user = r
+            # Module usage
+            [module:/module_a]
+            user = r
+            [other:/module_b]
+            user = r
+            [/module_c]
+            user = r
+            [module:/module_d]
+            user =
+            [/module_d]
+            user = r
 
-# Wildcards
-[/wildcard]
-* = r
+            # Wildcards
+            [/wildcard]
+            * = r
 
-# Special tokens
-[/special/anonymous]
-$anonymous = r
-[/special/authenticated]
-$authenticated = r
+            # Special tokens
+            [/special/anonymous]
+            $anonymous = r
+            [/special/authenticated]
+            $authenticated = r
 
-# Groups
-[/groups_a]
-@group1 = r
-[/groups_b]
-@group2 = r
-[/cyclic]
-@cycle1 = r
+            # Groups
+            [/groups_a]
+            @group1 = r
+            [/groups_b]
+            @group2 = r
+            [/cyclic]
+            @cycle1 = r
 
-# Precedence
-[module:/precedence_a]
-user =
-[/precedence_a]
-user = r
-[/precedence_b]
-user = r
-[/precedence_b/sub]
-user =
-[/precedence_b/sub/test]
-user = r
-[/precedence_c]
-user =
-@group1 = r
-[/precedence_d]
-@group1 = r
-user =
+            # Precedence
+            [module:/precedence_a]
+            user =
+            [/precedence_a]
+            user = r
+            [/precedence_b]
+            user = r
+            [/precedence_b/sub]
+            user =
+            [/precedence_b/sub/test]
+            user = r
+            [/precedence_c]
+            user =
+            @group1 = r
+            [/precedence_d]
+            @group1 = r
+            user =
 
-# Aliases
-[/aliases_a]
-&jekyll = r
-[/aliases_b]
-@alias2 = r
+            # Aliases
+            [/aliases_a]
+            &jekyll = r
+            [/aliases_b]
+            @alias2 = r
 
-# Scoped repository
-[scoped:/scope/dir1]
-joe = r
-[scoped:/scope/dir2]
-Jane = r
+            # Scoped repository
+            [scoped:/scope/dir1]
+            joe = r
+            [scoped:/scope/dir2]
+            Jane = r
 
-# multiple entries
-[/multiple]
-$authenticated = r
-[/multiple/foo]
-joe =
-$authenticated =
-* = r
-[/multiple/bar]
-* =
-john = r
-Jane = r
-$anonymous = r
-[/multiple/baz]
-$anonymous = r
-* =
-Jane = r
-[module:/multiple/bar]
-joe = r
-john =
+            # multiple entries
+            [/multiple]
+            $authenticated = r
+            [/multiple/foo]
+            joe =
+            $authenticated =
+            * = r
+            [/multiple/bar]
+            * =
+            john = r
+            Jane = r
+            $anonymous = r
+            [/multiple/baz]
+            $anonymous = r
+            * =
+            Jane = r
+            [module:/multiple/bar]
+            joe = r
+            john =
 
-# multiple entries with module and parent directory
-[/multiple/1]
-user = r
-@group1 = r
-$authenticated = r
-* = r
-[module:/multiple/1/user]
-user =
-[module:/multiple/1/group]
-@group1 =
-[module:/multiple/1/auth]
-$authenticated =
-[module:/multiple/1/star]
-* =
-[/multiple/2]
-user =
-@group1 =
-$authenticated =
-* =
-[module:/multiple/2/user]
-user = r
-[module:/multiple/2/group]
-@group1 = r
-[module:/multiple/2/auth]
-$authenticated = r
-[module:/multiple/2/star]
-* = r
-""")
+            # multiple entries with module and parent directory
+            [/multiple/1]
+            user = r
+            @group1 = r
+            $authenticated = r
+            * = r
+            [module:/multiple/1/user]
+            user =
+            [module:/multiple/1/group]
+            @group1 =
+            [module:/multiple/1/auth]
+            $authenticated =
+            [module:/multiple/1/star]
+            * =
+            [/multiple/2]
+            user =
+            @group1 =
+            $authenticated =
+            * =
+            [module:/multiple/2/user]
+            user = r
+            [module:/multiple/2/group]
+            @group1 = r
+            [module:/multiple/2/auth]
+            $authenticated = r
+            [module:/multiple/2/star]
+            * = r
+            """))
         self.env = EnvironmentStub(enable=[AuthzSourcePolicy], path=tmpdir)
         self.env.config.set('trac', 'permission_policies',
                             'AuthzSourcePolicy, DefaultPermissionPolicy')
-        self.env.config.set('svn', 'authz_file', self.authz)
+        self.env.config.set('svn', 'authz_file', self.authz_file)
 
         # Monkey-subclass RepositoryManager to serve mock repositories
         rm = RepositoryManager(self.env)
 
         class TestRepositoryManager(rm.__class__):
             def get_real_repositories(self):
-                return set([Mock(reponame='module'),
-                            Mock(reponame='other'),
-                            Mock(reponame='scoped')])
+                return {Mock(reponame='module'), Mock(reponame='other'),
+                        Mock(reponame='scoped')}
 
             def get_repository(self, reponame):
                 if reponame == 'scoped':
@@ -313,17 +313,17 @@ $authenticated = r
     def test_get_authz_file_removed_raises(self):
         """ConfigurationError exception is raised if file is removed."""
         policy = AuthzSourcePolicy(self.env)
-        os.remove(self.authz)
+        os.remove(self.authz_file)
         self.assertRaises(ConfigurationError, policy.check_permission,
                           'BROWSER_VIEW', 'user', None, None)
 
     def test_parse_error_raises(self):
         """ConfigurationError exception is raised when exception occurs
         parsing the `[svn authz_file`."""
-        create_file(self.authz, """\
-[/somepath
-joe = r
-""")
+        create_file(self.authz_file, textwrap.dedent("""\
+            [/somepath
+            joe = r
+            """))
         policy = AuthzSourcePolicy(self.env)
         self.assertRaises(ConfigurationError, policy.check_permission,
                           'BROWSER_VIEW', 'user', None, None)
@@ -363,16 +363,16 @@ joe = r
         self.assertRevPerm(True, 'joe')
         # Granted if at least one fine permission is granted
         policy._mtime = 0
-        create_file(self.authz, """\
-[/somepath]
-joe = r
-denied =
-[module:/otherpath]
-Jane = r
-$anonymous = r
-[inactive:/not-in-this-instance]
-unknown = r
-""")
+        create_file(self.authz_file, textwrap.dedent("""\
+            [/somepath]
+            joe = r
+            denied =
+            [module:/otherpath]
+            Jane = r
+            $anonymous = r
+            [inactive:/not-in-this-instance]
+            unknown = r
+            """))
         self.assertPathPerm(None, 'unknown')
         self.assertRevPerm(None, 'unknown')
         self.assertPathPerm(None, 'denied')
