@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2014 Edgewall Software
+# Copyright (C) 2003-2020 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
@@ -8,11 +8,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
@@ -24,17 +24,16 @@ import os
 import posixpath
 import re
 
-from genshi.builder import tag
-
 from trac.config import BoolOption, IntOption, Option
 from trac.core import *
 from trac.mimeview.api import Mimeview
 from trac.perm import IPermissionRequestor
-from trac.resource import Resource, ResourceNotFound
+from trac.resource import ResourceNotFound
 from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import as_bool, content_disposition, embedded_numbers, pathjoin
 from trac.util.datefmt import from_utimestamp, pretty_timedelta
+from trac.util.html import tag
 from trac.util.presentation import to_json
 from trac.util.text import CRLF, exception_to_unicode, shorten_line, \
                            to_unicode, unicode_urlencode
@@ -44,7 +43,7 @@ from trac.versioncontrol.api import Changeset, NoSuchChangeset, Node, \
 from trac.versioncontrol.diff import diff_blocks, get_diff_options, \
                                      unified_diff
 from trac.versioncontrol.web_ui.browser import BrowserModule
-from trac.versioncontrol.web_ui.util import render_zip
+from trac.versioncontrol.web_ui.util import content_closing, render_zip
 from trac.web import IRequestHandler, RequestDone
 from trac.web.chrome import (Chrome, INavigationContributor, add_ctxtnav,
                              add_link, add_script, add_stylesheet,
@@ -75,11 +74,10 @@ class IPropertyDiffRenderer(Interface):
         `options` are the current diffs options.
 
         The rendered result can be one of the following:
-        - `None`: the property change will be shown the normal way
-          (''changed from `old` to `new`'')
-        - an `unicode` value: the change will be shown as textual content
-        - `Markup` or other Genshi content: the change will shown as block
-          markup
+         - `None`: the property change will be shown the normal way
+           (''changed from `old` to `new`'')
+         - an `str` value: the change will be shown as textual content
+         - `Markup` or `Fragment`: the change will shown as block markup
         """
 
 
@@ -374,7 +372,7 @@ class ChangesetModule(Component):
                                           old=new, old_path=full_new_path)
             add_ctxtnav(req, _('Reverse Diff'), href=rev_href)
 
-        return 'changeset.html', data, None
+        return 'changeset.html', data
 
     # Internal methods
 
@@ -508,7 +506,7 @@ class ChangesetModule(Component):
             if old_props != new_props:
                 for k, v in sorted(old_props.items()):
                     new = old = diff = None
-                    if not k in new_props:
+                    if k not in new_props:
                         old = v  # won't be displayed, no need to render it
                     elif v != new_props[k]:
                         diff = self.render_property_diff(
@@ -522,7 +520,7 @@ class ChangesetModule(Component):
                         changed_properties.append({'name': k, 'old': old,
                                                    'new': new, 'diff': diff})
                 for k, v in sorted(new_props.items()):
-                    if not k in old_props:
+                    if k not in old_props:
                         new = browser.render_property(k, 'changeset',
                                                       new_ctx, new_props)
                         if new is not None:
@@ -547,10 +545,10 @@ class ChangesetModule(Component):
                 return None
             if mview.is_binary(new_node.content_type, new_node.path):
                 return None
-            old_content = old_node.get_content().read()
+            old_content = _read_content(old_node)
             if mview.is_binary(content=old_content):
                 return None
-            new_content = new_node.get_content().read()
+            new_content = _read_content(new_node)
             if mview.is_binary(content=new_content):
                 return None
 
@@ -561,8 +559,7 @@ class ChangesetModule(Component):
                 context = options.get('contextlines', 3)
                 if context < 0 or options.get('contextall'):
                     context = None
-                tabwidth = self.config['diff'].getint('tab_width') or \
-                           self.config['mimeviewer'].getint('tab_width', 8)
+                tabwidth = self.config.getint('mimeviewer', 'tab_width', 8)
                 ignore_blank_lines = options.get('ignoreblanklines')
                 ignore_case = options.get('ignorecase')
                 ignore_space = options.get('ignorewhitespace')
@@ -576,26 +573,27 @@ class ChangesetModule(Component):
                 return []
 
         diff_changes = list(get_changes())
-        diff_bytes = diff_files = 0
-        if self.max_diff_bytes or self.max_diff_files:
-            for old_node, new_node, kind, change in diff_changes:
-                if change in Changeset.DIFF_CHANGES and kind == Node.FILE \
-                        and old_node.is_viewable(req.perm) \
-                        and new_node.is_viewable(req.perm):
-                    diff_files += 1
-                    diff_bytes += _estimate_changes(old_node, new_node)
-        show_diffs = (not self.max_diff_files or
-                      0 < diff_files <= self.max_diff_files) and \
-                     (not self.max_diff_bytes or
-                      diff_bytes <= self.max_diff_bytes or
-                      diff_files == 1)
-
         # XHR is used for blame support: display the changeset view without
         # the navigation and with the changes concerning the annotated file
-        annotated = False
+        diff_bytes = diff_files = 0
+        annotated = None
         if req.is_xhr:
-            show_diffs = False
+            show_diffs = None
             annotated = repos.normalize_path(req.args.get('annotate'))
+        else:
+            if self.max_diff_bytes or self.max_diff_files:
+                for old_node, new_node, kind, change in diff_changes:
+                    if change in Changeset.DIFF_CHANGES and \
+                            kind == Node.FILE and \
+                            old_node.is_viewable(req.perm) and \
+                            new_node.is_viewable(req.perm):
+                        diff_files += 1
+                        diff_bytes += _estimate_changes(old_node, new_node)
+            show_diffs = (not self.max_diff_files or
+                          0 < diff_files <= self.max_diff_files) and \
+                         (not self.max_diff_bytes or
+                          diff_bytes <= self.max_diff_bytes or
+                          diff_files == 1)
 
         has_diffs = False
         filestats = self._prepare_filestats()
@@ -657,30 +655,33 @@ class ChangesetModule(Component):
                 info = None
             changes.append(info)  # the sequence should be immutable
 
-        data.update({'has_diffs': has_diffs, 'changes': changes,
-                     'xhr': req.is_xhr,  # Remove in 1.3.1
-                     'filestats': filestats,
-                     'annotated': annotated, 'files': files,
-                     'location': self._get_parent_location(files),
-                     'longcol': 'Revision', 'shortcol': 'r'})
+        data.update({
+            'has_diffs': has_diffs,
+            'show_diffs': show_diffs,
+            'diff_files': diff_files,
+            'diff_bytes': diff_bytes,
+            'max_diff_files': self.max_diff_files,
+            'max_diff_bytes': self.max_diff_bytes,
+            'changes': changes,
+            'filestats': filestats,
+            'annotated': annotated,
+            'files': files,
+            'location': self._get_parent_location(files),
+            'longcol': 'Revision',
+            'shortcol': 'r'
+        })
 
         if req.is_xhr:  # render and return the content only
-            chrome = Chrome(self.env)
-            stream = chrome.render_template(req, 'changeset.html', data,
-                                            fragment=True)
-            stream = stream.select('//div[@id="content"]')
-            if chrome.use_chunked_encoding:
-                output = chrome.iterable_content(stream, 'xhtml')
-            else:
-                output = stream.render('xhtml', encoding='utf-8')
-            req.send(output)
+            stream = Chrome(self.env).generate_fragment(
+                req, 'changeset_content.html', data)
+            req.send(stream)
 
         return data
 
     def _render_diff(self, req, filename, repos, data):
         """Raw Unified Diff version"""
 
-        output = (line.encode('utf-8') if isinstance(line, unicode) else line
+        output = (line.encode('utf-8') if isinstance(line, str) else line
                   for line in self._iter_diff_lines(req, repos, data))
         if Chrome(self.env).use_chunked_encoding:
             length = None
@@ -718,7 +719,7 @@ class ChangesetModule(Component):
                     continue
                 if mimeview.is_binary(old_node.content_type, old_node.path):
                     continue
-                old_content = old_node.get_content().read()
+                old_content = _read_content(old_node)
                 if mimeview.is_binary(content=old_content):
                     continue
                 old_node_info = (old_node.path, old_node.rev)
@@ -729,7 +730,7 @@ class ChangesetModule(Component):
                     continue
                 if mimeview.is_binary(new_node.content_type, new_node.path):
                     continue
-                new_content = new_node.get_content().read()
+                new_content = _read_content(new_node)
                 if mimeview.is_binary(content=new_content):
                     continue
                 new_node_info = (new_node.path, new_node.rev)
@@ -851,7 +852,7 @@ class ChangesetModule(Component):
             if len(repositories) > 1:
                 filters = [
                     ('repo-' + repos.reponame,
-                     u"\xa0\xa0-\xa0" + (repos.reponame or _('(default)')))
+                     "\xa0\xa0-\xa0" + (repos.reponame or _('(default)')))
                     for repos in repositories
                     if not as_bool(repos.params.get('hidden'))
                     and repos.is_viewable(req.perm)]
@@ -867,7 +868,7 @@ class ChangesetModule(Component):
 
     def get_timeline_events(self, req, start, stop, filters):
         all_repos = 'changeset' in filters
-        repo_filters = set(f for f in filters if f.startswith('repo-'))
+        repo_filters = {f for f in filters if f.startswith('repo-')}
         if all_repos or repo_filters:
             show_files = self.timeline_show_files
             show_location = show_files == 'location'
@@ -889,8 +890,6 @@ class ChangesetModule(Component):
                                              key=collapse_changesets):
                     viewable_changesets = []
                     for cset in changesets:
-                        cset_resource = Resource(self.realm, cset.rev,
-                                                 parent=repos.resource)
                         if cset.is_viewable(req.perm):
                             repos_for_uid = [repos.reponame]
                             uid = repos.get_changeset_uid(cset.rev)
@@ -900,7 +899,7 @@ class ChangesetModule(Component):
                                     uids_seen[uid].append(repos.reponame)
                                     continue  # already viewable, just append
                                 uids_seen[uid] = repos_for_uid
-                            viewable_changesets.append((cset, cset_resource,
+                            viewable_changesets.append((cset, cset.resource,
                                                         repos_for_uid))
                     if viewable_changesets:
                         cset = viewable_changesets[-1][0]
@@ -954,7 +953,7 @@ class ChangesetModule(Component):
                             resource = c.resource.parent.child('source',
                                                                chg[0] or '/',
                                                                r.id)
-                            if not 'FILE_VIEW' in context.perm(resource):
+                            if 'FILE_VIEW' not in context.perm(resource):
                                 continue
                             filestats[chg[2]] += 1
                             files.append(chg[0])
@@ -976,7 +975,7 @@ class ChangesetModule(Component):
                             resource = c.resource.parent.child('source',
                                                                chg[0] or '/',
                                                                r.id)
-                            if not 'FILE_VIEW' in context.perm(resource):
+                            if 'FILE_VIEW' not in context.perm(resource):
                                 continue
                             if 0 < show_files < len(files):
                                 break
@@ -984,7 +983,7 @@ class ChangesetModule(Component):
                     files = [tag.li(tag.div(class_=mod), path or '/')
                              for path, mod in sorted(unique_files)]
                     if 0 < show_files < len(files):
-                        files = files[:show_files] + [tag.li(u'\u2026')]
+                        files = files[:show_files] + [tag.li('\u2026')]
                     markup = tag(tag.ul(files, class_="changes"), markup)
             if message:
                 markup += format_to(self.env, None,
@@ -1021,8 +1020,8 @@ class ChangesetModule(Component):
                 labels.append(tag.span(name, class_='trac-bookmark'))
             return title if not labels else tag(title, labels)
         elif field == 'summary':
-            return _("%(title)s: %(message)s",
-                     title=title, message=shorten_line(message))
+            return tag_("%(title)s: %(message)s",
+                        title=title, message=shorten_line(message))
 
     # IWikiSyntaxProvider methods
 
@@ -1130,11 +1129,11 @@ class ChangesetModule(Component):
             yield ('changeset', _('Changesets'))
 
     def get_search_results(self, req, terms, filters):
-        if not 'changeset' in filters:
+        if 'changeset' not in filters:
             return
         rm = RepositoryManager(self.env)
-        repositories = dict((repos.params['id'], repos)
-                            for repos in rm.get_real_repositories())
+        repositories = {repos.params['id']: repos
+                        for repos in rm.get_real_repositories()}
         uids_seen = set()
         with self.env.db_query as db:
             sql, args = search_to_sql(db, ['rev', 'message', 'author'], terms)
@@ -1179,7 +1178,7 @@ class AnyDiffModule(Component):
             reponame, repos, path = rm.get_repository_by_path(dirname)
             # an entry is a (isdir, name, path) tuple
             def kind_order(entry):
-                return (not entry[0], embedded_numbers(entry[1]))
+                return not entry[0], embedded_numbers(entry[1])
 
             entries = []
             if repos:
@@ -1226,4 +1225,9 @@ class AnyDiffModule(Component):
 
         Chrome(self.env).add_jquery_ui(req)
         add_stylesheet(req, 'common/css/diff.css')
-        return 'diff_form.html', data, None
+        return 'diff_form.html', data
+
+
+def _read_content(node):
+    with content_closing(node.get_content()) as content:
+        return content.read()
