@@ -1,23 +1,22 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014 Edgewall Software
+# Copyright (C) 2014-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
+import io
 import posixpath
 import unittest
 import zipfile
 from datetime import datetime
-from cStringIO import StringIO
 
-import trac.tests.compat
 from trac.core import Component, TracError, implements
 from trac.perm import PermissionError
 from trac.resource import ResourceNotFound
@@ -29,7 +28,6 @@ from trac.versioncontrol.api import (
     Repository, RepositoryManager)
 from trac.versioncontrol.web_ui.browser import BrowserModule
 from trac.web.api import RequestDone
-from trac.web.chrome import Chrome
 from trac.web.tests.api import RequestHandlerPermissionsTestCaseBase
 
 
@@ -53,8 +51,6 @@ class MockRepositoryConnector(Component):
             if 'file' in basename:
                 kind = Node.FILE
                 entries = ()
-                content = 'Contents for %s' % to_utf8(path)
-                length = len(content)
             else:
                 kind = Node.DIRECTORY
                 if 'dir' in basename:
@@ -62,14 +58,14 @@ class MockRepositoryConnector(Component):
                 else:
                     entries = ['dir1', 'dir2']
                 entries = [posixpath.join(path, entry) for entry in entries]
-                content = length = None
+            content = b'Contents for %s' % to_utf8(path)
             node = Mock(Node, repos, path, rev, kind,
                         created_path=path, created_rev=rev,
                         get_entries=lambda: iter(get_node(entry, rev)
                                                  for entry in entries),
                         get_properties=lambda: {},
-                        get_content=lambda: content and StringIO(content),
-                        get_content_length=lambda: length,
+                        get_content=lambda: io.BytesIO(content),
+                        get_content_length=lambda: len(content),
                         get_content_type=lambda: 'application/octet-stream',
                         get_last_modified=lambda: t)
             return node
@@ -82,8 +78,7 @@ class MockRepositoryConnector(Component):
                          get_changeset=get_changeset,
                          get_node=get_node,
                          previous_rev=lambda rev, path='': None,
-                         next_rev=lambda rev, path='': None,
-                         display_rev=lambda rev: str(rev))
+                         next_rev=lambda rev, path='': None)
         return repos
 
 
@@ -120,13 +115,13 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
         self.grant_perm('anonymous', 'BROWSER_VIEW')
         provider = DbRepositoryProvider(self.env)
         req = MockRequest(self.env, path_info='/')
-        self.assertEqual('browser', self.get_navigation_items(req).next()[1])
+        self.assertEqual('browser', next(self.get_navigation_items(req))[1])
 
         provider.remove_repository('allow')
-        self.assertEqual('browser', self.get_navigation_items(req).next()[1])
+        self.assertEqual('browser', next(self.get_navigation_items(req))[1])
 
         provider.remove_repository('deny')
-        self.assertEqual('browser', self.get_navigation_items(req).next()[1])
+        self.assertEqual('browser', next(self.get_navigation_items(req))[1])
 
         provider.remove_repository('(default)')
         self.assertEqual([], list(self.get_navigation_items(req)))
@@ -134,13 +129,13 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
     def test_get_navigation_items_without_browser_view(self):
         provider = DbRepositoryProvider(self.env)
         req = MockRequest(self.env, path_info='/')
-        self.assertEqual('browser', self.get_navigation_items(req).next()[1])
+        self.assertEqual('browser', next(self.get_navigation_items(req))[1])
 
         provider.remove_repository('(default)')
-        self.assertEqual('browser', self.get_navigation_items(req).next()[1])
+        self.assertEqual('browser', next(self.get_navigation_items(req))[1])
 
         provider.remove_repository('deny')
-        self.assertEqual('browser', self.get_navigation_items(req).next()[1])
+        self.assertEqual('browser', next(self.get_navigation_items(req))[1])
 
         provider.remove_repository('allow')
         self.assertEqual([], list(self.get_navigation_items(req)))
@@ -173,21 +168,21 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
         DbRepositoryProvider(self.env).remove_repository('(default)')
         req = MockRequest(self.env, path_info='/browser/')
         rv = self.process_request(req)
-        self.assertEqual(None, rv[1]['repos'])
+        self.assertIsNone(rv[1]['repos'])
 
         req = MockRequest(self.env, path_info='/browser/blah-blah-file')
         try:
             self.process_request(req)
             self.fail('ResourceNotFound not raised')
         except ResourceNotFound as e:
-            self.assertEqual('No node blah-blah-file', unicode(e))
+            self.assertEqual('No node blah-blah-file', str(e))
 
     def test_repository_without_browser_view(self):
         req = MockRequest(self.env, authname='anonymous',
                           path_info='/browser/')
         rv = self.process_request(req)
         # cannot view default repository but don't raise PermissionError
-        self.assertEqual(None, rv[1]['repos'])
+        self.assertIsNone(rv[1]['repos'])
 
         req = MockRequest(self.env, path_info='/browser/allow')
         rv = self.process_request(req)
@@ -208,7 +203,7 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
         DbRepositoryProvider(self.env).remove_repository('(default)')
         req = MockRequest(self.env, path_info='/browser/')
         rv = self.process_request(req)
-        self.assertEqual(None, rv[1]['repos'])
+        self.assertIsNone(rv[1]['repos'])
 
         req = MockRequest(self.env, authname='anonymous',
                           path_info='/browser/blah-blah-file')
@@ -217,7 +212,7 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
             self.fail('PermissionError not raised')
         except PermissionError as e:
             self.assertEqual('BROWSER_VIEW', e.action)
-            self.assertEqual(None, e.resource)
+            self.assertIsNone(e.resource)
 
     def test_node_with_file_view(self):
         self.grant_perm('anonymous', 'BROWSER_VIEW', 'FILE_VIEW')
@@ -318,8 +313,8 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
         provider.modify_repository('(default)', {'hidden': 'enabled'})
         req = MockRequest(self.env, authname='anonymous',
                           path_info='/browser/')
-        template, data, content_type = self.process_request(req)
-        self.assertEqual(None, data['repos'])
+        template, data = self.process_request(req)
+        self.assertIsNone(data['repos'])
         repo_data = data['repo']  # for repository index
         self.assertEqual('allow', repo_data['repositories'][0][0])
         self.assertEqual('raise', repo_data['repositories'][1][0])
@@ -330,7 +325,7 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
         provider = DbRepositoryProvider(self.env)
         provider.modify_repository('(default)', {'hidden': 'enabled'})
         req = MockRequest(self.env, path_info='/browser/blah-blah-file')
-        template, data, content_type = self.process_request(req)
+        template, data = self.process_request(req)
         self.assertEqual('', data['reponame'])
         self.assertEqual('blah-blah-file', data['path'])
 
@@ -348,13 +343,13 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
             self.process_request(req)
             self.fail('ResourceNotFound not raised')
         except ResourceNotFound as e:
-            self.assertEqual('No viewable repositories', unicode(e))
+            self.assertEqual('No viewable repositories', str(e))
         req = MockRequest(self.env, path_info='/browser/allow/')
         try:
             self.process_request(req)
             self.fail('ResourceNotFound not raised')
         except ResourceNotFound as e:
-            self.assertEqual('No node allow', unicode(e))
+            self.assertEqual('No node allow', str(e))
         req = MockRequest(self.env, authname='anonymous',
                           path_info='/browser/deny/')
         try:
@@ -373,13 +368,13 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
             self.process_request(req)
             self.fail('ResourceNotFound not raised')
         except ResourceNotFound as e:
-            self.assertEqual('No viewable repositories', unicode(e))
+            self.assertEqual('No viewable repositories', str(e))
         req = MockRequest(self.env, path_info='/browser/deny/')
         try:
             self.process_request(req)
             self.fail('ResourceNotFound not raised')
         except ResourceNotFound as e:
-            self.assertEqual('No node deny', unicode(e))
+            self.assertEqual('No node deny', str(e))
 
     def test_no_viewable_repositories_without_browser_view(self):
         provider = DbRepositoryProvider(self.env)
@@ -391,7 +386,7 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
             self.fail('PermissionError not raised')
         except PermissionError as e:
             self.assertEqual('BROWSER_VIEW', e.action)
-            self.assertEqual(None, e.resource)
+            self.assertIsNone(e.resource)
         provider.remove_repository('deny')
         provider.remove_repository('(default)')
         req = MockRequest(self.env, authname='anonymous',
@@ -401,7 +396,7 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
             self.fail('PermissionError not raised')
         except PermissionError as e:
             self.assertEqual('BROWSER_VIEW', e.action)
-            self.assertEqual(None, e.resource)
+            self.assertIsNone(e.resource)
 
     def test_zip_archive(self):
         req = MockRequest(self.env, path_info='/browser/trunk',
@@ -414,28 +409,20 @@ anonymous = !BROWSER_VIEW, !FILE_VIEW
                          sorted(i.filename for i in z.infolist()))
 
         zi = z.getinfo('trunk/dir1/')
-        self.assertEqual((040755 << 16) | 0x10, zi.external_attr)
+        self.assertEqual((0o40755 << 16) | 0x10, zi.external_attr)
 
         zi = z.getinfo('trunk/dir1/file.txt')
-        self.assertEqual(0644 << 16, zi.external_attr)
-        self.assertEqual('Contents for trunk/dir1/file.txt',
+        self.assertEqual(0o644 << 16, zi.external_attr)
+        self.assertEqual(b'Contents for trunk/dir1/file.txt',
                          z.read('trunk/dir1/file.txt'))
         self.assertEqual((2017, 3, 31, 12, 34, 56), zi.date_time)
 
         zi = z.getinfo('trunk/dir2/file.txt')
-        self.assertEqual(0644 << 16, zi.external_attr)
-        self.assertEqual('Contents for trunk/dir2/file.txt',
+        self.assertEqual(0o644 << 16, zi.external_attr)
+        self.assertEqual(b'Contents for trunk/dir2/file.txt',
                          z.read('trunk/dir2/file.txt'))
         self.assertEqual((2017, 3, 31, 12, 34, 56), zi.date_time)
 
-    def test_directory_content_length_in_browser(self):
-        req = MockRequest(self.env, path_info='/browser')
-        rv = self.process_request(req)
-        rendered = Chrome(self.env).render_template(req, *rv)
-        self.assertIn('>dir1</', rendered)
-        self.assertIn('>dir2</', rendered)
-        self.assertNotIn(' title="None bytes"', rendered)
-
 
 def test_suite():
     suite = unittest.TestSuite()
