@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2018 Edgewall Software
+# Copyright (C) 2014-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 from datetime import datetime, timedelta
 import unittest
@@ -26,7 +26,7 @@ from trac.versioncontrol.web_ui.log import LogModule
 from trac.web.api import parse_arg_list
 from trac.web.chrome import web_context
 from trac.web.tests.api import RequestHandlerPermissionsTestCaseBase
-from trac.wiki.formatter import format_to_oneliner
+from trac.wiki.formatter import format_to_html
 
 
 mock_repotype = 'mock:' + __name__
@@ -48,7 +48,13 @@ class TestLogModulePermissionPolicy(Component):
     implements(IPermissionPolicy)
 
     def check_permission(self, action, username, resource, perm):
-        if action == 'LOG_VIEW' and resource.realm == 'repository':
+        if action == 'LOG_VIEW':
+            if resource.realm != 'source':
+                raise AssertionError('Invalid realm %r' % resource.realm)
+            if resource.id == 'deny' or resource.id.endswith('/deny'):
+                return False
+            if resource.id == 'allow' or resource.id.endswith('/allow'):
+                return True
             return username != 'anonymous'
 
 
@@ -118,7 +124,7 @@ class MockRepository(Repository):
 class MockChangeset(Changeset):
 
     def get_changes(self):
-        raise StopIteration
+        return iter(())
 
 
 class MockNode(Node):
@@ -171,14 +177,14 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
         provider.add_repository('mock', '/', mock_repotype)
 
     def test_default_repository_not_configured(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11599."""
+        """Test for regression of https://trac.edgewall.org/ticket/11599."""
         req = MockRequest(self.env, path_info='/log/', args={'new_path': '/'})
         self.assertRaises(TracError, self.process_request, req)
 
     def test_without_rev(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         self.assertEqual('revisionlog.html', template)
         items = data['items']
         self.assertEqual(5, len(items))
@@ -187,14 +193,14 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
         self.assertEqual(['delete'] + ['edit'] * 3 + [None],
                          [item['change'] for item in items])
         links = req.chrome['links']['next']
-        self.assertEqual('/trac.cgi/log/mock/file?limit=4&rev=88&'
-                         'mode=stop_on_copy', links[0]['href'])
+        self.assertEqual('/trac.cgi/log/mock/file?limit=4&'
+                         'mode=stop_on_copy&rev=88', links[0]['href'])
         self.assertEqual(1, len(links))
 
     def test_with_rev(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'rev': '49'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([49, 46, 43, 40, 37],
@@ -208,7 +214,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_with_rev_and_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'rev': '49', 'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([49, 46, 43, 40, 37],
@@ -221,14 +227,14 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
         self.assertEqual(['edit'] * 3 + ['move', None],
                          [item['change'] for item in items])
         links = req.chrome['links']['next']
-        self.assertEqual('/trac.cgi/log/mock/file-old?limit=4&rev=37&'
-                         'mode=stop_on_copy', links[0]['href'])
+        self.assertEqual('/trac.cgi/log/mock/file-old?limit=4&'
+                         'mode=stop_on_copy&rev=37', links[0]['href'])
         self.assertEqual(1, len(links))
 
     def test_with_rev_on_start(self):
         req = MockRequest(self.env, path_info='/log/mock/file-old',
                           args={'rev': '10'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(4, len(items))
         self.assertEqual([10, 7, 4, 1],
@@ -244,7 +250,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_with_rev_and_limit_on_start(self):
         req = MockRequest(self.env, path_info='/log/mock/file-old',
                           args={'rev': '10', 'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(4, len(items))
         self.assertEqual([10, 7, 4, 1],
@@ -263,7 +269,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
             try:
                 self.process_request(req)
             except NoSuchChangeset as e:
-                self.assertEqual(message, unicode(e))
+                self.assertEqual(message, str(e))
 
         fn('No changeset 101 in the repository', args={'rev': '101'})
         fn('No changeset 0 in the repository', args={'rev': '0'})
@@ -272,7 +278,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_1(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '70,79-82,94-100'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(9, len(items))
         self.assertEqual([100, 97, 94, 91, 82, 79, 76, 70, 67],
@@ -290,7 +296,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_2(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '22-49'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([49, 46, 43, 40, 37],
@@ -307,7 +313,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_3(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '22-46,55-61'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(8, len(items))
         self.assertEqual([61, 58, 55, 52, 46, 43, 40, 37],
@@ -325,7 +331,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_4(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '40-46,55-61'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(8, len(items))
         self.assertEqual([61, 58, 55, 52, 46, 43, 40, 37],
@@ -343,7 +349,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_1_with_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '70,79-82,94-100', 'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(6, len(items))
         self.assertEqual([100, 97, 94, 91, 82, 79],
@@ -357,8 +363,8 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
                          [item['change'] for item in items])
         self.assertIn('next', req.chrome['links'])
         links = req.chrome['links']['next']
-        self.assertEqual('/trac.cgi/log/mock/file?limit=4&revs=70%2C79&'
-                         'rev=79&mode=stop_on_copy', links[0]['href'])
+        self.assertEqual('/trac.cgi/log/mock/file?limit=4&mode=stop_on_copy&'
+                         'rev=79&revs=70%2C79', links[0]['href'])
         self.assertEqual(1, len(links))
 
     def test_revranges_1_next_link_with_limits(self):
@@ -366,7 +372,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
             req = MockRequest(self.env, path_info='/log/mock/file',
                               args={'revs': '70,79-82,94-100',
                                     'limit': str(limit)})
-            template, data, ctype = self.process_request(req)
+            template, data = self.process_request(req)
             links = req.chrome['links']
             if 'next' in links:
                 link = links['next'][0]['href']
@@ -385,12 +391,12 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
                           'mode': 'stop_on_copy'}, next_link_args(4))
         self.assertEqual({'limit': '5', 'rev': '76', 'revs': '70',
                           'mode': 'stop_on_copy'}, next_link_args(5))
-        self.assertEqual(None, next_link_args(6))
+        self.assertIsNone(next_link_args(6))
 
     def test_revranges_2_with_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '22-49', 'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([49, 46, 43, 40, 37],
@@ -404,14 +410,14 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
                          [item['change'] for item in items])
         self.assertIn('next', req.chrome['links'])
         links = req.chrome['links']['next']
-        self.assertEqual('/trac.cgi/log/mock/file-old?limit=4&revs=22-37&'
-                         'rev=37&mode=stop_on_copy', links[0]['href'])
+        self.assertEqual('/trac.cgi/log/mock/file-old?limit=4&mode='
+                         'stop_on_copy&rev=37&revs=22-37', links[0]['href'])
         self.assertEqual(1, len(links))
 
     def test_revranges_3_with_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '22-46,55-61', 'limit': '7'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(8, len(items))
         self.assertEqual([61, 58, 55, 52, 46, 43, 40, 37],
@@ -429,7 +435,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_4_with_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '40-46,55-61', 'limit': '7'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(8, len(items))
         self.assertEqual([61, 58, 55, 52, 46, 43, 40, 37],
@@ -450,7 +456,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
             try:
                 self.process_request(req)
             except NoSuchChangeset as e:
-                self.assertEqual(message, unicode(e))
+                self.assertEqual(message, str(e))
 
         fn('No changeset 101 in the repository', args={'revs': '101'})
         fn('No changeset 0 in the repository', args={'revs': '0'})
@@ -465,7 +471,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'rev': '43', 'limit': '4',
                                 'mode': 'follow_copy'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([43, 40, 37, 34, 31],
@@ -478,14 +484,14 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
         self.assertEqual(['edit', 'move', 'edit', 'edit', None],
                          [item['change'] for item in items])
         links = req.chrome['links']['next']
-        self.assertEqual('/trac.cgi/log/mock/file-old?limit=4&rev=31&'
-                         'mode=follow_copy', links[0]['href'])
+        self.assertEqual('/trac.cgi/log/mock/file-old?limit=4&'
+                         'mode=follow_copy&rev=31', links[0]['href'])
         self.assertEqual(1, len(links))
 
     def test_path_history(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'mode': 'path_history'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(3, len(items))
         self.assertEqual(['delete', 'move', 'add'],
@@ -495,19 +501,21 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_log_link_checking_repository_resource(self):
         self.env.config.set('trac', 'permission_policies',
             'TestLogModulePermissionPolicy, DefaultPermissionPolicy')
+
+        self.assertIn(' title="No permission to view change log"',
+                      self._format_to_html('anonymous', 'log:mock@42-43'))
+        self.assertIn(' href="/trac.cgi/log/mock/allow?revs=42-43"',
+                      self._format_to_html('anonymous',
+                                           'log:mock/allow@42-43'))
+        self.assertIn(' href="/trac.cgi/log/mock/?revs=42-43"',
+                      self._format_to_html('blah', 'log:mock@42-43'))
+        self.assertIn(' title="No permission to view change log"',
+                      self._format_to_html('blah', 'log:mock/deny'))
+
+    def _format_to_html(self, authname, wiki):
         resource = Resource('wiki', 'WikiStart')
-
-        req = MockRequest(self.env, authname='anonymous')
-        rendered = unicode(format_to_oneliner(self.env,
-                                              web_context(req, resource),
-                                              'log:mock@42-43'))
-        self.assertIn(' title="No permission to view change log"', rendered)
-
-        req = MockRequest(self.env, authname='blah')
-        rendered = unicode(format_to_oneliner(self.env,
-                                              web_context(req, resource),
-                                              'log:mock@42-43'))
-        self.assertIn(' href="/trac.cgi/log/mock/?revs=42-43"', rendered)
+        req = MockRequest(self.env, authname=authname)
+        return str(format_to_html(self.env, web_context(req, resource), wiki))
 
 
 def test_suite():
