@@ -1,19 +1,18 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014 Edgewall Software
+# Copyright (C) 2014-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 import unittest
 
-import trac.tests.compat
 from trac.test import Mock
 from trac.versioncontrol.api import EmptyChangeset, NoSuchChangeset
 from trac.versioncontrol.web_ui import util
