@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2018 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.com/license.html.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 import unittest
 
@@ -59,11 +59,11 @@ class GitRepositoryStub(object):
     _revs = [
         ('ffffffffffffffffffffffffffffffffffffffff', ('HEAD',)),
         ('deadbef222222222222222222222222222222222', ('1.0-stable',
-                                                      u'1.0-stáblé')),
-        ('deadbef111111111111111111111111111111111', ('v1.0.1', u'vér1.0.1')),
-        ('deadbef000000000000000000000000000000000', ('v1.0', u'vér1.0')),
+                                                      '1.0-stáblé')),
+        ('deadbef111111111111111111111111111111111', ('v1.0.1', 'vér1.0.1')),
+        ('deadbef000000000000000000000000000000000', ('v1.0', 'vér1.0')),
         ('deadbeefffffffffffffffffffffffffffffffff', ('0.12-stable',
-                                                      u'0.12-stáblé')),
+                                                      '0.12-stáblé')),
         ('0000009876543210987654321098765432109876', ()),  # only digits
         ('0000001234567890123456789012345678901234', ()),
         ('1111111111111111111111111111111111111111', ()),  # oldest rev
@@ -116,13 +116,12 @@ def _get_all_repositories():
 
 
 def repository_setup(tc):
-    setattr(tc.env, 'get_repository', _get_repository)
     setattr(RepositoryManager(tc.env), 'get_repository', _get_repository)
     setattr(RepositoryManager(tc.env), 'get_all_repositories',
             _get_all_repositories)
 
 
-CHANGESET_TEST_CASES = u"""
+CHANGESET_TEST_CASES = """
 ============================== changeset: link resolver
 changeset:1
 changeset:12
@@ -203,8 +202,8 @@ trac:changeset:2081
 [trac:changeset:2081 Trac r2081]
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon"></span>trac:changeset:2081</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon"></span>Trac r2081</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in The Trac Project"><span class="icon"></span>trac:changeset:2081</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in The Trac Project"><span class="icon"></span>Trac r2081</a>
 </p>
 ------------------------------
 ============================== Changeset InterTrac shorthands
@@ -214,16 +213,16 @@ trac:changeset:2081
 T:r2081
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon"></span>[T2081]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon"></span>[trac 2081]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081/trunk" title="changeset:2081/trunk in Trac\'s Trac"><span class="icon"></span>[trac 2081/trunk]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/r2081" title="r2081 in Trac's Trac"><span class="icon"></span>T:r2081</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in The Trac Project"><span class="icon"></span>[T2081]</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in The Trac Project"><span class="icon"></span>[trac 2081]</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/changeset%3A2081/trunk" title="changeset:2081/trunk in The Trac Project"><span class="icon"></span>[trac 2081/trunk]</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/r2081" title="r2081 in The Trac Project"><span class="icon"></span>T:r2081</a>
 </p>
 ------------------------------
 """ #"
 
 
-LOG_TEST_CASES = u"""
+LOG_TEST_CASES = """
 ============================== Log range TracLinks
 [1:2], r1:2, [12:23], r12:23
 [1:2/trunk], r1:2/trunk
@@ -416,9 +415,9 @@ rfc:4180 should not be a log link
 [trac 3317:3318/trunk]
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/%403317%3A3318" title="log:/@3317:3318 in Trac\'s Trac"><span class="icon"></span>[T3317:3318]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/%403317%3A3318" title="log:/@3317:3318 in Trac\'s Trac"><span class="icon"></span>[trac 3317:3318]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/trunk%403317%3A3318" title="log:/trunk@3317:3318 in Trac\'s Trac"><span class="icon"></span>[trac 3317:3318/trunk]</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/log%3A/%403317%3A3318" title="log:/@3317:3318 in The Trac Project"><span class="icon"></span>[T3317:3318]</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/log%3A/%403317%3A3318" title="log:/@3317:3318 in The Trac Project"><span class="icon"></span>[trac 3317:3318]</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/log%3A/trunk%403317%3A3318" title="log:/trunk@3317:3318 in The Trac Project"><span class="icon"></span>[trac 3317:3318/trunk]</a>
 </p>
 ------------------------------
 ============================== Log range with unicode digits
@@ -437,7 +436,7 @@ r₁₂:₂₀,₂₅,₃₀-₃₅
 """
 
 
-DIFF_TEST_CASES = u"""
+DIFF_TEST_CASES = """
 ============================== diff: link resolver
 diff:trunk//branch
 diff:trunk@12//branch@23
@@ -468,7 +467,7 @@ diff://
 """
 
 
-SOURCE_TEST_CASES = u"""
+SOURCE_TEST_CASES = """
 ============================== source: link resolver
 source:/foo/bar
 source:/foo/bar#42   # no long works as rev spec
