@@ -1,31 +1,27 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2020 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
-from __future__ import print_function
-
 from abc import ABCMeta, abstractmethod
 from base64 import b64decode, b64encode
 from hashlib import md5, sha1
 import os
 import re
 import sys
-import urllib2
-import urlparse
-
-from genshi.builder import tag
+import urllib.parse
+import urllib.request
 
 from trac.config import BoolOption, IntOption, Option
 from trac.core import *
@@ -35,6 +31,7 @@ from trac.util import hex_entropy, md5crypt
 from trac.util.compat import crypt
 from trac.util.concurrency import threading
 from trac.util.datefmt import time_now
+from trac.util.html import tag
 from trac.util.translation import _, tag_
 
 
@@ -79,12 +76,14 @@ class LoginModule(Component):
         authentication information, and therefore, after how much
         inactivity a user will have to log in again. The value
         of 0 makes the cookie expire at the end of the browsing
-        session. (''since 0.12'')""")
+        session.
+        """)
 
     auth_cookie_path = Option('trac', 'auth_cookie_path', '',
         """Path for the authentication cookie. Set this to the common
         base path of several Trac instances if you want them to share
-        the cookie.  (''since 0.12'')""")
+        the cookie.
+        """)
 
     # IAuthenticator methods
 
@@ -110,15 +109,20 @@ class LoginModule(Component):
         return 'login'
 
     def get_navigation_items(self, req):
-        if req.authname and req.authname != 'anonymous':
+        if req.is_authenticated:
             yield ('metanav', 'login',
                    tag_("logged in as %(user)s",
                         user=Chrome(self.env).authorinfo(req, req.authname)))
             yield ('metanav', 'logout',
-                   tag.form(tag.div(tag.button(_("Logout"),
-                                               name='logout', type='submit')),
-                            action=req.href.logout(), method='post',
-                            id='logout', class_='trac-logout'))
+                   tag.form(
+                       tag.div(
+                           tag.button(_("Logout"), name='logout',
+                                      type='submit'),
+                           tag.input(type='hidden', name='__FORM_TOKEN',
+                                     value=req.form_token)
+                       ),
+                       action=req.href.logout(), method='post',
+                       id='logout', class_='trac-logout'))
         else:
             yield ('metanav', 'login',
                    tag.a(_("Login"), href=req.href.login()))
@@ -204,10 +208,7 @@ class LoginModule(Component):
         Simply deletes the corresponding record from the auth_cookie
         table.
         """
-        if req.method != 'POST':
-            return
-        if req.authname == 'anonymous':
-            # Not logged in
+        if req.method != 'POST' or not req.is_authenticated:
             return
 
         if 'trac_auth' in req.incookie:
@@ -264,9 +265,9 @@ class LoginModule(Component):
         if referer:
             if not referer.startswith(('http://', 'https://')):
                 # Make URL absolute
-                scheme, host = urlparse.urlparse(req.base_url)[:2]
-                referer = urlparse.urlunparse((scheme, host, referer, None,
-                                               None, None))
+                scheme, host = urllib.parse.urlparse(req.base_url)[:2]
+                referer = urllib.parse.urlunparse((scheme, host, referer, None,
+                                                   None, None))
             pos = req.base_url.find(':')
             base_scheme = req.base_url[:pos]
             base_noscheme = req.base_url[pos:]
@@ -285,9 +286,7 @@ class LoginModule(Component):
         return req.args.get('referer') or req.get_header('Referer')
 
 
-class HTTPAuthentication(object):
-
-    __metaclass__ = ABCMeta
+class HTTPAuthentication(object, metaclass=ABCMeta):
 
     @abstractmethod
     def do_auth(self, environ, start_response):
@@ -321,23 +320,23 @@ class BasicAuthentication(PasswordFileAuthentication):
     def load(self, filename):
         # FIXME use a logger
         self.hash = {}
-        fd = open(filename, 'r')
-        for line in fd:
-            line = line.strip()
-            if not line:
-                continue
-            try:
-                u, h = line.split(':')
-            except ValueError:
-                print("Warning: invalid password line in %s: %s"
-                      % (filename, line), file=sys.stderr)
-                continue
-            if '$' in h or h.startswith('{SHA}') or self.crypt:
-                self.hash[u] = h
-            else:
-                print('Warning: cannot parse password for user "%s" '
-                      'without the "crypt" module. Install the passlib '
-                      'package from PyPI' % u, file=sys.stderr)
+        with open(filename, encoding='utf-8') as fd:
+            for line in fd:
+                line = line.split('#')[0].strip()
+                if not line:
+                    continue
+                try:
+                    u, h = line.split(':')[:2]
+                except ValueError:
+                    print("Warning: invalid password line in %s: %s"
+                          % (filename, line), file=sys.stderr)
+                    continue
+                if '$' in h or h.startswith('{SHA}') or self.crypt:
+                    self.hash[u] = h
+                else:
+                    print('Warning: cannot parse password for user "%s" '
+                          'without the "crypt" module. Install the passlib '
+                          'package from PyPI' % u, file=sys.stderr)
 
         if self.hash == {}:
             print("Warning: found no users in file:", filename,
@@ -350,7 +349,8 @@ class BasicAuthentication(PasswordFileAuthentication):
             return False
 
         if the_hash.startswith('{SHA}'):
-            return b64encode(sha1(password).digest()) == the_hash[5:]
+            return str(b64encode(sha1(password.encode('utf-8')).digest()),
+                       'ascii') == the_hash[5:]
 
         if '$' not in the_hash:
             return self.crypt(password, the_hash[:2]) == the_hash
@@ -362,15 +362,16 @@ class BasicAuthentication(PasswordFileAuthentication):
     def do_auth(self, environ, start_response):
         header = environ.get('HTTP_AUTHORIZATION')
         if header and header.startswith('Basic'):
-            auth = b64decode(header[6:]).split(':')
+            auth = str(b64decode(header[6:]), 'utf-8').split(':')
             if len(auth) == 2:
                 user, password = auth
                 if self.test(user, password):
                     return user
 
-        start_response('401 Unauthorized',
-                       [('WWW-Authenticate', 'Basic realm="%s"' % self.realm),
-                        ('Content-Length', '0')])('')
+        headers = [('WWW-Authenticate', 'Basic realm="%s"' % self.realm),
+                   ('Content-Length', '0')]
+        write = start_response('401 Unauthorized', headers)
+        write(b'')
 
 
 class DigestAuthentication(PasswordFileAuthentication):
@@ -392,26 +393,26 @@ class DigestAuthentication(PasswordFileAuthentication):
         """
         # FIXME use a logger
         self.hash = {}
-        fd = open(filename, 'r')
-        for line in fd.readlines():
-            line = line.strip()
-            if not line:
-                continue
-            try:
-                u, r, a1 = line.split(':')
-            except ValueError:
-                print("Warning: invalid digest line in %s: %s"
-                      % (filename, line), file=sys.stderr)
-                continue
-            if r == self.realm:
-                self.hash[u] = a1
+        with open(filename, encoding='utf-8') as fd:
+            for line in fd:
+                line = line.split('#')[0].strip()
+                if not line:
+                    continue
+                try:
+                    u, r, a1 = line.split(':')[:3]
+                except ValueError:
+                    print("Warning: invalid digest line in %s: %s"
+                          % (filename, line), file=sys.stderr)
+                    continue
+                if r == self.realm:
+                    self.hash[u] = a1
         if self.hash == {}:
             print("Warning: found no users in realm:", self.realm,
                   file=sys.stderr)
 
     def parse_auth_header(self, authorization):
         values = {}
-        for value in urllib2.parse_http_list(authorization):
+        for value in urllib.request.parse_http_list(authorization):
             n, v = value.split('=', 1)
             if v[0] == '"' and v[-1] == '"':
                 values[n] = v[1:-1]
@@ -427,11 +428,12 @@ class DigestAuthentication(PasswordFileAuthentication):
         self.active_nonces.append(nonce)
         if len(self.active_nonces) > self.MAX_NONCES:
             self.active_nonces = self.active_nonces[-self.MAX_NONCES:]
-        start_response('401 Unauthorized',
-                       [('WWW-Authenticate',
-                        'Digest realm="%s", nonce="%s", qop="auth", stale="%s"'
-                        % (self.realm, nonce, stale)),
-                        ('Content-Length', '0')])('')
+        headers = [('WWW-Authenticate',
+                    'Digest realm="%s", nonce="%s", qop="auth", stale="%s"' %
+                    (self.realm, nonce, stale)),
+                   ('Content-Length', '0')]
+        write = start_response('401 Unauthorized', headers)
+        write(b'')
 
     def do_auth(self, environ, start_response):
         header = environ.get('HTTP_AUTHORIZATION')
@@ -453,7 +455,7 @@ class DigestAuthentication(PasswordFileAuthentication):
             self.send_auth_request(environ, start_response)
             return None
 
-        kd = lambda x: md5(':'.join(x)).hexdigest()
+        kd = lambda x: md5(b':'.join(v.encode('utf-8') for v in x)).hexdigest()
         a1 = self.hash[auth['username']]
         a2 = kd([environ['REQUEST_METHOD'], auth['uri']])
         # Is the response correct?
