@@ -1,21 +1,21 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2020 Edgewall Software
 # Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import doctest
 import unittest
 
-import trac.tests.compat
+from trac.util.text import jinja2template
 import trac.web.href
 
 
@@ -110,17 +110,32 @@ class HrefTestCase(unittest.TestCase):
     def test_add_unicode(self):
         href = trac.web.href.Href('/base')
         self.assertEqual('/base/p%C3%A4th/to/%20/file/',
-                         href + u'/päth/to/ /file/')
+                         href + '/päth/to/ /file/')
         self.assertEqual('/base/p%C3%A4th/to/%20/file',
-                         href + u'päth/to/ /file')
+                         href + 'päth/to/ /file')
         self.assertEqual('/base?type=def%C3%A9ct&or&type=abc%20def',
-                         href + u'?type=deféct&or&type=abc def')
+                         href + '?type=deféct&or&type=abc def')
         self.assertEqual('/base/p%C3%A4th/to/file/'
                          '?type=def%C3%A9ct&or&type=abc%20def',
-                         href + u'/päth/to/file/?type=deféct&or&type=abc def')
+                         href + '/päth/to/file/?type=deféct&or&type=abc def')
         self.assertEqual('/base/p%C3%A4th/to/file'
                          '?type=def%C3%A9ct&or&type=abc%20def',
-                         href + u'päth/to/file?type=deféct&or&type=abc def')
+                         href + 'päth/to/file?type=deféct&or&type=abc def')
+
+    def test_jinja2(self):
+        # https://trac.edgewall.org/ticket/13244
+
+        data = {'href': trac.web.href.Href('http://localhost/base')}
+        self.assertEqual(
+            '<a href="http://localhost/base">anchor</a>',
+            self._render_template('<a href="${href()}">anchor</a>', data))
+        self.assertEqual(
+            'URL: http://localhost/base',
+            self._render_template('URL: ${href()}', data, text=True))
+
+    def _render_template(self, template, data, text=False):
+        t = jinja2template(template, text=text)
+        return t.render(**data)
 
 
 def test_suite():
