@@ -1,29 +1,38 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2018 Edgewall Software
+# Copyright (C) 2004-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import unittest
 
-from genshi.template import MarkupTemplate
 from trac.core import Component, TracError, implements
 from trac.util.html import html
 from trac.util.translation import tag_
-from trac.wiki.api import IWikiSyntaxProvider
+from trac.wiki.api import IWikiMacroProvider, IWikiSyntaxProvider
 from trac.wiki.formatter import MacroError, ProcessorError
 from trac.wiki.macros import WikiMacroBase
 from trac.wiki.test import wikisyntax_test_suite
 
 
+# WikiProcessor code block tests should always use PlainTextRenderer
+# and not the PygmentsRenderer, as the latter renders quotes as &quot;
+
+from trac.mimeview.api import PlainTextRenderer
+_old_ratio = PlainTextRenderer.get_quality_ratio
+def override_pygments(self, mimetype):
+    return (10 if mimetype == 'text/plain' else _old_ratio(self, mimetype))
+PlainTextRenderer.get_quality_ratio = override_pygments
+
+
 # We need to supply our own macro because the real macros
 # can not be loaded using our 'fake' environment.
 
@@ -34,6 +43,42 @@ class HelloWorldMacro(WikiMacroBase):
         return 'Hello World, args = ' + content
 
 
+class NoDescriptionMacro(WikiMacroBase):
+    # Macro with no description string
+
+    def expand_macro(self, formatter, name, content):
+        return ''
+
+
+class NoDescriptionHiddenMacro(WikiMacroBase):
+    # Macro with no description string
+    # Hidden from the macro index (MacroList)
+
+    hide_from_macro_index = True
+
+    def expand_macro(self, formatter, name, content):
+        return ''
+
+
+class IWikiMacroProviderMacros(Component):
+    # Macro for testing MacroList output
+
+    implements(IWikiMacroProvider)
+
+    def get_macros(self):
+        yield 'ProviderMacro1'
+        yield 'ProviderMacro2'
+
+    def get_macro_description(self, name):
+        if name == 'ProviderMacro1':
+            return 'ProviderMacro1 description'
+        if name == 'ProviderMacro2':
+            return None  # Will be hidden from macro index
+
+    def expand_macro(self, formatter, name, content, args=None):
+        return 'The macro is named %s' % name
+
+
 class DivHelloWorldMacro(WikiMacroBase):
     """A dummy macro returning a div block, used by the unit test."""
 
@@ -58,20 +103,22 @@ class DivCodeMacro(WikiMacroBase):
 
 
 class DivCodeElementMacro(WikiMacroBase):
-    """A dummy macro returning a Genshi Element, used by the unit test."""
+    """A dummy macro returning an Element, used by the unit test."""
 
     def expand_macro(self, formatter, name, content):
-        return html.DIV('Hello World, args = ', content, class_="code")
+        return html.div('Hello World, args = ', content, class_="code")
 
 
 class DivCodeStreamMacro(WikiMacroBase):
-    """A dummy macro returning a Genshi Stream, used by the unit test."""
+    """A dummy macro returning a template, used by the unit test."""
 
     def expand_macro(self, formatter, name, content):
-        tmpl = MarkupTemplate("""
-        <div>Hello World, args = $args</div>
-        """)
-        return tmpl.generate(args=content)
+        template = """
+            <div>Hello World, args = ${args}</div>
+            """
+        from trac.util.text import jinja2template
+        tmpl = jinja2template(template.strip())
+        return tmpl.render(args=content)
 
 
 class NoneMacro(WikiMacroBase):
@@ -88,8 +135,8 @@ class WikiProcessorSampleMacro(WikiMacroBase):
             return 'Called as a macro: ' + content
         else:
             return 'Called as a processor with params: <dl>%s</dl>' % \
-                ''.join('<dt>%s</dt><dd>%s</dd>' % kv for kv in args.items()) \
-                + content
+                ''.join('<dt>%s</dt><dd>%s</dd>' % (k, args[k])
+                        for k in sorted(args)) + content
 
 
 class ValueErrorWithUtf8Macro(WikiMacroBase):
@@ -101,7 +148,7 @@ class ValueErrorWithUtf8Macro(WikiMacroBase):
 class TracErrorWithUnicodeMacro(WikiMacroBase):
 
     def expand_macro(self, formatter, name, content, args):
-        raise TracError(unicode(content))
+        raise TracError(str(content))
 
 
 class MacroErrorWithFragmentMacro(WikiMacroBase):
@@ -136,18 +183,10 @@ class SampleResolver(Component):
             module = 'thing'
         except ValueError:
             pass
-        return html.A(label, class_='%s resolver' % kind,
+        return html.a(label, class_='%s resolver' % kind,
                       href=formatter.href(module, target))
 
 
-def suite(data=None, setup=None, file=__file__, teardown=None, context=None):
-    """Retained for backward-compatibility.
-
-    :since 1.2: Deprecated and will be removed in 1.3.1
-    """
-    return test_suite(data, setup, file, teardown, context)
-
-
 def test_suite(data=None, setup=None, file=__file__, teardown=None,
                context=None):
     suite = unittest.TestSuite()
