@@ -1,25 +1,24 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2018 Edgewall Software
+# Copyright (C) 2006-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
-from StringIO import StringIO
+import io
 import os
-import tempfile
 import unittest
 
 from trac.attachment import Attachment
 from trac.config import BoolOption, ConfigSection, IntOption, ListOption, \
                         Option
-from trac.test import locale_en, rmtree
+from trac.test import locale_en, mkdtemp, rmtree
 from trac.util.datefmt import datetime_now, format_date, utc
 from trac.wiki.model import WikiPage
 from trac.wiki.tests import formatter
@@ -31,20 +30,20 @@ def add_pages(tc, names):
         w = WikiPage(tc.env)
         w.name = name
         w.text = '--'
-        w.save('joe', 'the page ' + name, '::1', now)
+        w.save('joe', 'the page ' + name, now)
 
 
 def add_attachment(tc, realm, id, file):
     attachment = Attachment(tc.env, realm, id)
     attachment.description = "image in %s" % id
-    attachment.insert(file, StringIO(''), 0, 2)
+    attachment.insert(file, io.BytesIO(), 0, 2)
 
 
 # == [[Image]]
 
 def image_setup(tc):
     add_pages(tc, ['page:fr', 'page'])
-    tc.env.path = tempfile.mkdtemp(prefix='trac-tempenv-')
+    tc.env.path = mkdtemp()
     add_attachment(tc, 'wiki', 'page:fr', 'img.png')
     add_attachment(tc, 'wiki', 'page', 'img.png')
     add_attachment(tc, 'wiki', 'page', '][img.png')
@@ -64,7 +63,7 @@ def image_teardown(tc):
 #       unicode robustness and whitespace support (first space is
 #       normal ASCII SPACE, second is Unicode NO-BREAK SPACE).
 
-IMAGE_MACRO_TEST_CASES = u"""
+IMAGE_MACRO_TEST_CASES = """
 ============================== Image, no arguments
 [[Image]]
 ------------------------------
@@ -89,7 +88,7 @@ IMAGE_MACRO_TEST_CASES = u"""
 [[Image(source:« test ».png)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB.png"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" alt="source:« test ».png" title="source:« test ».png" /></a>
+<a href="/browser/%C2%AB%20test%C2%A0%C2%BB.png" style="padding:0; border:none"><img alt="source:« test ».png" src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" title="source:« test ».png" /></a>
 </p>
 ------------------------------
 [[Image(...)]]
@@ -97,157 +96,157 @@ IMAGE_MACRO_TEST_CASES = u"""
 [[Image(source:« test ».png, inline)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB.png"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" alt="source:« test ».png" title="source:« test ».png" /></a>
+<a href="/browser/%C2%AB%20test%C2%A0%C2%BB.png" style="padding:0; border:none"><img alt="source:« test ».png" src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" title="source:« test ».png" /></a>
 </p>
 ------------------------------
-<a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB.png"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" alt="source:« test ».png" title="source:« test ».png" /></a>
+<a href="/browser/%C2%AB%20test%C2%A0%C2%BB.png" style="padding:0; border:none"><img alt="source:« test ».png" src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" title="source:« test ».png" /></a>
 ============================== intertrac:source: Image, no other arguments
 [[Image(trac:source:/trunk/doc/images/bkgnd_pattern_« test ».png)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png"><img src="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png%3Fformat%3Draw" alt="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" crossorigin="anonymous" title="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" /></a>
+<a href="https://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png" style="padding:0; border:none"><img alt="source:/trunk/doc/images/bkgnd_pattern_« test ».png in The Trac Project" crossorigin="anonymous" src="https://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png%3Fformat%3Draw" title="source:/trunk/doc/images/bkgnd_pattern_« test ».png in The Trac Project" /></a>
 </p>
 ============================== source: Image, nolink
 [[Image(source:« test », nolink)]]
 ------------------------------
 <p>
-<img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="source:« test »" title="source:« test »" />
+<img alt="source:« test »" src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" title="source:« test »" />
 </p>
 ============================== source: Image, normal args
 [[Image(source:« test », align=left, title=Test)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="source:« test »" style="float:left" title="Test" /></a>
+<a href="/browser/%C2%AB%20test%C2%A0%C2%BB" style="padding:0; border:none"><img alt="source:« test »" src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" style="float:left" title="Test" /></a>
 </p>
 ============================== source: Image, size arg
 [[Image(source:« test », 30%)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB"><img width="30%" alt="source:« test »" title="source:« test »" src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" /></a>
+<a href="/browser/%C2%AB%20test%C2%A0%C2%BB" style="padding:0; border:none"><img alt="source:« test »" src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" title="source:« test »" width="30%" /></a>
 </p>
 ============================== source: Image, keyword alignment
 [[Image(source:« test », right)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="source:« test »" style="float:right" title="source:« test »" /></a>
+<a href="/browser/%C2%AB%20test%C2%A0%C2%BB" style="padding:0; border:none"><img alt="source:« test »" src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" style="float:right" title="source:« test »" /></a>
 </p>
 ============================== http: Image, nolink
 [[Image(http://www.edgewall.com/gfx/shredder_« test ».png, nolink)]]
 ------------------------------
 <p>
-<img src="http://www.edgewall.com/gfx/shredder_« test ».png" alt="http://www.edgewall.com/gfx/shredder_« test ».png" crossorigin="anonymous" title="http://www.edgewall.com/gfx/shredder_« test ».png" />
+<img alt="http://www.edgewall.com/gfx/shredder_« test ».png" crossorigin="anonymous" src="http://www.edgewall.com/gfx/shredder_« test ».png" title="http://www.edgewall.com/gfx/shredder_« test ».png" />
 </p>
 ============================== http: Image, absolute, many ':'
 [[Image(http://chart.apis.google.com:80/chart?cht=p3&chd=s:hW&chs=250x100&chl=Héllo|Wôrld, title=Google & Charting, link=)]]
 ------------------------------
 <p>
-<img src="http://chart.apis.google.com:80/chart?cht=p3&amp;chd=s:hW&amp;chs=250x100&amp;chl=Héllo|Wôrld" alt="http://chart.apis.google.com:80/chart" crossorigin="anonymous" title="Google &amp; Charting" />
+<img alt="http://chart.apis.google.com:80/chart" crossorigin="anonymous" src="http://chart.apis.google.com:80/chart?cht=p3&amp;chd=s:hW&amp;chs=250x100&amp;chl=Héllo|Wôrld" title="Google &amp; Charting" />
 </p>
 ============================== // Image, server-relative
 [[Image(//browser/« test »?format=raw, link=)]]
 ------------------------------
 <p>
-<img src="/browser/« test »?format=raw" alt="/browser/« test »" title="/browser/« test »" />
+<img alt="/browser/« test »" src="/browser/« test »?format=raw" title="/browser/« test »" />
 </p>
 ============================== / Image, project-relative, link to WikiStart
 [[Image(/browser/« test »?format=raw, link=wiki:WikiStart)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="/wiki/WikiStart"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="/browser/« test »" title="/browser/« test »" /></a>
+<a href="/wiki/WikiStart" style="padding:0; border:none"><img alt="/browser/« test »" src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" title="/browser/« test »" /></a>
 </p>
 ============================== Strip unicode white-spaces and ZWSPs (#10668)
 [[Image(  ​source:« test ».png　 ​, nolink, 100%　 ​)]]
 ------------------------------
 <p>
-<img width="100%" alt="source:« test ».png" title="source:« test ».png" src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" />
+<img alt="source:« test ».png" src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" title="source:« test ».png" width="100%" />
 </p>
 ============================== Attachments on page with ':' characters (#10562)
 [[Image("page:fr":img.png​,nolink)]]
 ------------------------------
 <p>
-<img src="/raw-attachment/wiki/page%3Afr/img.png" alt="image in page:fr" title="image in page:fr" />
+<img alt="image in page:fr" src="/raw-attachment/wiki/page%3Afr/img.png" title="image in page:fr" />
 </p>
 ============================== htdocs: Image, nolink
 [[Image(htdocs:trac_logo.png, nolink)]]
 ------------------------------
 <p>
-<img src="/chrome/site/trac_logo.png" alt="trac_logo.png" title="trac_logo.png" />
+<img alt="trac_logo.png" src="/chrome/site/trac_logo.png" title="trac_logo.png" />
 </p>
 ============================== shared: Image, nolink
 [[Image(shared:trac_logo.png, nolink)]]
 ------------------------------
 <p>
-<img src="/chrome/shared/trac_logo.png" alt="trac_logo.png" title="trac_logo.png" />
+<img alt="trac_logo.png" src="/chrome/shared/trac_logo.png" title="trac_logo.png" />
 </p>
 ==============================
 [[Image("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=")]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII="><img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" alt="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" title="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" /></a>
+<a href="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" style="padding:0; border:none"><img alt="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" title="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" /></a>
 </p>
 ==============================
 [[Image("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=", nolink)]]
 ------------------------------
 <p>
-<img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" alt="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" title="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" />
+<img alt="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" title="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" />
 </p>
 ============================== InterWiki
 [[Image(shields:travis/edgewall/trac.svg, link=trac:source:/trunk)]]
 [[Image(travis:edgewall/trac.svg:1.0-stable, link=trac:source:/branches/1.0-stable)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk"><img src="https://img.shields.io/travis/edgewall/trac.svg" alt="travis/edgewall/trac.svg in shields" crossorigin="anonymous" title="travis/edgewall/trac.svg in shields" /></a>
-<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/branches/1.0-stable"><img src="https://travis-ci.org/edgewall/trac.svg?branch=1.0-stable" alt="edgewall/trac.svg:1.0-stable in travis" crossorigin="anonymous" title="edgewall/trac.svg:1.0-stable in travis" /></a>
+<a href="https://trac.edgewall.org/intertrac/source%3A/trunk" style="padding:0; border:none"><img alt="travis/edgewall/trac.svg in shields" crossorigin="anonymous" src="https://img.shields.io/travis/edgewall/trac.svg" title="travis/edgewall/trac.svg in shields" /></a>
+<a href="https://trac.edgewall.org/intertrac/source%3A/branches/1.0-stable" style="padding:0; border:none"><img alt="edgewall/trac.svg:1.0-stable in travis" crossorigin="anonymous" src="https://travis-ci.org/edgewall/trac.svg?branch=1.0-stable" title="edgewall/trac.svg:1.0-stable in travis" /></a>
 </p>
 ============================== InterWiki, nolink
 [[Image(shields:pypi/dm/trac.svg, nolink)]]
 ------------------------------
 <p>
-<img src="https://img.shields.io/pypi/dm/trac.svg" alt="pypi/dm/trac.svg in shields" crossorigin="anonymous" title="pypi/dm/trac.svg in shields" />
+<img alt="pypi/dm/trac.svg in shields" crossorigin="anonymous" src="https://img.shields.io/pypi/dm/trac.svg" title="pypi/dm/trac.svg in shields" />
 </p>
-==============================
+============================== No attachment, nolink
 [[Image(notfound.png, nolink)]]
 ------------------------------
 <p>
-<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;notfound.png&#34; attached to WikiStart" crossorigin="anonymous" title="No image &#34;notfound.png&#34; attached to WikiStart" />
+<img alt="No image &#34;notfound.png&#34; attached to WikiStart" crossorigin="anonymous" src="http://assets.example.org/common/attachment.png" title="No image &#34;notfound.png&#34; attached to WikiStart" />
 </p>
-==============================
+============================== No attachment, correct CSS
 [[Image(img.png, margin-bottom=-1)]]
 ------------------------------
 <p>
-<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;img.png&#34; attached to WikiStart" title="No image &#34;img.png&#34; attached to WikiStart" style="margin-bottom: 1px" crossorigin="anonymous" />
+<img alt="No image &#34;img.png&#34; attached to WikiStart" crossorigin="anonymous" src="http://assets.example.org/common/attachment.png" style="margin-bottom: 1px" title="No image &#34;img.png&#34; attached to WikiStart" />
 </p>
-==============================
+============================== No attachment, invalid arg
 [[Image(img.png, margin-bottom=--)]]
 ------------------------------
 <p>
 <div class="system-message"><strong>Macro Image(img.png, margin-bottom=--) failed</strong><pre>Invalid macro argument <code>margin-bottom=--</code></pre></div>
 </p>
-==============================
+============================== No attachment, invalid arg 2
 [[Image(img.png, margin-top=--)]]
 ------------------------------
 <p>
 <div class="system-message"><strong>Macro Image(img.png, margin-top=--) failed</strong><pre>Invalid macro argument <code>margin-top=--</code></pre></div>
 </p>
-==============================
+============================== No attachment, invalid arg 3
 [[Image(img.png, margin=--)]]
 ------------------------------
 <p>
 <div class="system-message"><strong>Macro Image(img.png, margin=--) failed</strong><pre>Invalid macro argument <code>margin=--</code></pre></div>
 </p>
-==============================
+============================== No attachment, invalid arg 3
 [[Image(img.png, margin-left=--)]]
 ------------------------------
 <p>
 <div class="system-message"><strong>Macro Image(img.png, margin-left=--) failed</strong><pre>Invalid macro argument <code>margin-left=--</code></pre></div>
 </p>
-==============================
+============================== No attachment, invalid arg 4
 [[Image(img.png, margin-right=--)]]
 ------------------------------
 <p>
 <div class="system-message"><strong>Macro Image(img.png, margin-right=--) failed</strong><pre>Invalid macro argument <code>margin-right=--</code></pre></div>
 </p>
-==============================
+============================== No attachment, invalid arg 5
 [[Image(img.png, border=--)]]
 ------------------------------
 <p>
@@ -256,7 +255,7 @@ IMAGE_MACRO_TEST_CASES = u"""
 ==============================  # Regression test for #12333
 = [[Image]]
 ------------------------------
-<h1 id="Image">[[Image]]</h1>
+<h1 class="section" id="Image">[[Image]]</h1>
 ==============================  Invalid use of attachment TracLink
 [[Image(attachment:img.png:wiki:page)]]
 ------------------------------
@@ -267,19 +266,19 @@ IMAGE_MACRO_TEST_CASES = u"""
 [[Image(wiki:page:img2.png)]]
 ------------------------------
 <p>
-<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;img2.png&#34; attached to page" crossorigin="anonymous" title="No image &#34;img2.png&#34; attached to page" />
+<img alt="No image &#34;img2.png&#34; attached to page" crossorigin="anonymous" src="http://assets.example.org/common/attachment.png" title="No image &#34;img2.png&#34; attached to page" />
 </p>
 ==============================  "[" and "]" characters - 1 (#12762)
 [[Image(wiki:page:][img.png,nolink)]]
 ------------------------------
 <p>
-<img src="/raw-attachment/wiki/page/%5D%5Bimg.png" alt="image in page" title="image in page" />
+<img alt="image in page" src="/raw-attachment/wiki/page/%5D%5Bimg.png" title="image in page" />
 </p>
 ==============================  "[" and "]" characters - 2 (#12762)
 [[Image(][img.png,nolink)]]
 ------------------------------
 <p>
-<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;][img.png&#34; attached to WikiStart" crossorigin="anonymous" title="No image &#34;][img.png&#34; attached to WikiStart" />
+<img alt="No image &#34;][img.png&#34; attached to WikiStart" crossorigin="anonymous" src="http://assets.example.org/common/attachment.png" title="No image &#34;][img.png&#34; attached to WikiStart" />
 </p>
 """
 
@@ -289,6 +288,23 @@ IMAGE_MACRO_TEST_CASES = u"""
 #       according to the W3C XHTML validator).
 
 
+# == [[InterTrac]]
+
+def intertrac_setup(tc):
+    tc.env.config.set('intertrac', 'example', 'example2')
+    # Definition conflict with alias, but must not raise a TypeError.
+    tc.env.config.set('intertrac', 'example.url', 'http://example.org')
+
+
+INTERTRAC_MACRO_TEST_CASES = """\
+==============================
+[[InterTrac]]
+------------------------------
+<p>
+</p><table class="wiki intertrac"><tr><th><em>Prefix</em></th><th><em>Trac Site</em></th></tr><tr><td><strong>example</strong></td><td>Alias for <strong>example2</strong></td></tr><tr><td><a href="https://genshi.edgewall.org/timeline"><strong>genshi</strong></a></td><td><a href="https://genshi.edgewall.org">Genshi\'s Trac</a></td></tr><tr><td><strong>t</strong></td><td>Alias for <strong>trac</strong></td></tr><tr><td><a href="http://trac-hacks.org/timeline"><strong>th</strong></a></td><td><a href="http://trac-hacks.org">Trac Hacks</a></td></tr><tr><td><a href="https://trac.edgewall.org/timeline"><strong>trac</strong></a></td><td><a href="https://trac.edgewall.org">The Trac Project</a></td></tr></table><p>
+</p>
+"""
+
 
 # == [[TitleIndex]]
 
@@ -297,7 +313,7 @@ def titleindex_teardown(tc):
 
 
 
-TITLEINDEX1_MACRO_TEST_CASES = u"""
+TITLEINDEX1_MACRO_TEST_CASES = """
 ============================== TitleIndex, default format
 [[TitleIndex()]]
 ------------------------------
@@ -329,7 +345,7 @@ TITLEINDEX1_MACRO_TEST_CASES = u"""
 ------------------------------
 """
 
-TITLEINDEX2_MACRO_TEST_CASES = u"""
+TITLEINDEX2_MACRO_TEST_CASES = """
 ============================== TitleIndex, default format
 [[TitleIndex()]]
 ------------------------------
@@ -384,7 +400,7 @@ def titleindex2_setup(tc):
     add_pages(tc, ['WikiEnd'])
 
 
-TITLEINDEX3_MACRO_TEST_CASES = u"""
+TITLEINDEX3_MACRO_TEST_CASES = """
 ============================== TitleIndex, group format
 [[TitleIndex(Wiki,format=group)]]
 ------------------------------
@@ -481,7 +497,7 @@ def titleindex3_setup(tc):
         ])
 
 
-TITLEINDEX4_MACRO_TEST_CASES = u"""
+TITLEINDEX4_MACRO_TEST_CASES = """
 ============================== TitleIndex group and page with numbers (#7919)
 [[TitleIndex(format=group)]]
 ------------------------------
@@ -537,7 +553,7 @@ def titleindex4_setup(tc):
         ])
 
 
-TITLEINDEX5_MACRO_TEST_CASES = u"""
+TITLEINDEX5_MACRO_TEST_CASES = """
 ============================== TitleIndex, hierarchy format with complex hierarchy
 [[TitleIndex(format=hierarchy)]]
 ------------------------------
@@ -572,7 +588,7 @@ def titleindex5_setup(tc):
         ])
 
 
-RECENTCHANGES_MACRO_TEST_CASES = u""""
+RECENTCHANGES_MACRO_TEST_CASES = """"
 ============================== RecentChanges, group option
 [[RecentChanges()]]
 [[RecentChanges(group=date)]]
@@ -582,26 +598,26 @@ RECENTCHANGES_MACRO_TEST_CASES = u""""
 [[RecentChanges(Wiki,1,group=none)]]
 ------------------------------
 <p>
-</p><div><h3>%(date)s</h3><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
+</p><div class="wikipage"><h3 class="section">%(date)s</h3><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
 </li><li><a href="/wiki/WikiMid">WikiMid</a>
 </li><li><a href="/wiki/WikiStart">WikiStart</a>
 </li></ul></div><p>
-</p><div><h3>%(date)s</h3><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
+</p><div class="wikipage"><h3 class="section">%(date)s</h3><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
 </li><li><a href="/wiki/WikiMid">WikiMid</a>
 </li><li><a href="/wiki/WikiStart">WikiStart</a>
 </li></ul></div><p>
-</p><div><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
+</p><div class="wikipage"><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
 </li><li><a href="/wiki/WikiMid">WikiMid</a>
 </li><li><a href="/wiki/WikiStart">WikiStart</a>
 </li></ul></div><p>
-</p><div><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
+</p><div class="wikipage"><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
 </li><li><a href="/wiki/WikiMid">WikiMid</a>
 </li></ul></div><p>
-</p><div><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
+</p><div class="wikipage"><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
 </li><li><a href="/wiki/WikiMid">WikiMid</a>
 </li><li><a href="/wiki/WikiStart">WikiStart</a>
 </li></ul></div><p>
-</p><div><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
+</p><div class="wikipage"><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
 </li></ul></div><p>
 </p>
 ==============================
@@ -620,19 +636,19 @@ def recentchanges_setup(tc):
             w = WikiPage(tc.env)
             w.name = name
             w.text = '--'
-            w.save('joe', 'the page ' + name, '::1', now)
+            w.save('joe', 'the page ' + name, now)
     add_pages(tc, [
         'WikiMid',
         'WikiEnd',
         ])
-    tc.correct = tc.correct % {'date': format_date(tzinfo=utc,
-                                                   locale=locale_en)}
+    tc.expected = tc.expected % {'date': format_date(tzinfo=utc,
+                                                     locale=locale_en)}
 
 def recentchanges_teardown(tc):
     tc.env.reset_db()
 
 
-PAGEOUTLINE_MACRO_TEST_CASES = u""""
+PAGEOUTLINE_MACRO_TEST_CASES = """"
 ==============================
 [[PageOutline(a)]]
 ------------------------------
@@ -659,8 +675,8 @@ PAGEOUTLINE_MACRO_TEST_CASES = u""""
 </ol>
 </div><p>
 </p>
-<h1 id="HeadingLevel1">Heading Level 1</h1>
-<h2 id="HeadingLevel2">Heading Level 2</h2>
+<h1 class="section" id="HeadingLevel1">Heading Level 1</h1>
+<h2 class="section" id="HeadingLevel2">Heading Level 2</h2>
 ==============================
 [[PageOutline(7)]]
 ===== Heading Level 5
@@ -675,8 +691,8 @@ PAGEOUTLINE_MACRO_TEST_CASES = u""""
                     </ol>
 </div><p>
 </p>
-<h5 id="HeadingLevel5">Heading Level 5</h5>
-<h6 id="HeadingLevel6">Heading Level 6</h6>
+<h5 class="section" id="HeadingLevel5">Heading Level 5</h5>
+<h6 class="section" id="HeadingLevel6">Heading Level 6</h6>
 ==============================
 [[PageOutline(0-7)]]
 = Heading Level 1
@@ -720,16 +736,16 @@ PAGEOUTLINE_MACRO_TEST_CASES = u""""
 </ol>
 </div><p>
 </p>
-<h1 id="HeadingLevel1">Heading Level 1</h1>
-<h2 id="HeadingLevel2">Heading Level 2</h2>
-<h3 id="HeadingLevel3">Heading Level 3</h3>
-<h4 id="HeadingLevel4">Heading Level 4</h4>
-<h5 id="HeadingLevel5">Heading Level 5</h5>
-<h6 id="HeadingLevel6">Heading Level 6</h6>
+<h1 class="section" id="HeadingLevel1">Heading Level 1</h1>
+<h2 class="section" id="HeadingLevel2">Heading Level 2</h2>
+<h3 class="section" id="HeadingLevel3">Heading Level 3</h3>
+<h4 class="section" id="HeadingLevel4">Heading Level 4</h4>
+<h5 class="section" id="HeadingLevel5">Heading Level 5</h5>
+<h6 class="section" id="HeadingLevel6">Heading Level 6</h6>
 """
 
 
-TRACINI_MACRO_TEST_CASES = u"""\
+TRACINI_MACRO_TEST_CASES = """\
 ============================== TracIni, option with empty doc (#10940)
 [[TracIni(section-42)]]
 ------------------------------
@@ -885,6 +901,7 @@ No options
 ------------------------------
 """
 
+
 def tracini_setup(tc):
     tc._orig_registries = ConfigSection.registry, Option.registry
     class Foo(object):
@@ -903,16 +920,96 @@ def tracini_setup(tc):
         option_d4 = (BoolOption)('section-def', 'option4', False)
         option_d5 = (ListOption)('section-def', 'option5', [])
 
+
 def tracini_teardown(tc):
     ConfigSection.registry, Option.registry = tc._orig_registries
 
 
+INTERWIKI_MACRO_TEST_CASES = """
+==============================
+[[InterWiki]]
+------------------------------
+<p>
+</p><table class="wiki interwiki">\
+<tr><th><em>Prefix</em></th><th><em>Site</em></th></tr>\
+<tr><td><a href="http://wikicreole.org/wiki/RecentChanges">CreoleWiki</a></td>\
+<td><a href="http://wikicreole.org/wiki/">http://wikicreole.org/wiki/</a></td></tr>\
+<tr><td><a href="https://img.shields.io/RecentChanges">shields</a></td>\
+<td><a href="https://img.shields.io/">https://img.shields.io/</a></td></tr>\
+<tr><td><a href="http://stackoverflow.com/questions/RecentChanges">SO</a></td>\
+<td><a href="http://stackoverflow.com/questions/">Question $1 in StackOverflow</a></td></tr>\
+<tr><td><a href="https://travis-ci.org/RecentChanges?branch=">travis</a></td>\
+<td><a href="https://travis-ci.org/$1?branch=$2">Travis CI</a></td></tr>\
+</table><p>
+</p>
+------------------------------
+"""
+
+
+def interwiki_setup(tc):
+    tc.env.config.set('interwiki', 'shields', 'https://img.shields.io/')
+    tc.env.config.set('interwiki', 'travis',
+                      'https://travis-ci.org/$1?branch=$2 Travis CI')
+    page = WikiPage(tc.env)
+    page.name = 'InterMapTxt'
+    page.text = """
+The InterWikiTxt page
+----
+{{{
+SO  http://stackoverflow.com/questions/ # Question $1 in StackOverflow
+CreoleWiki   http://wikicreole.org/wiki/
+}}}
+"""
+    page.save('admin', 'created page')
+
+
+def interwiki_teardown(tc):
+    tc.env.reset_db()
+
+
+def macrolist_setup(tc):
+    pass
+
+MACROLIST_MACRO_TEST_CASES = """
+==============================
+[[MacroList(HelloWorld)]]
+------------------------------
+<p>
+</p><div class="trac-macrolist"><h3 id="HelloWorld-macro"><code>[[HelloWorld]]</code></h3><p>
+A dummy macro used by the unit test.
+</p>
+</div><p>
+</p>
+------------------------------
+==============================
+[[MacroList(NoDescription)]]
+------------------------------
+<p>
+</p><div class="trac-macrolist"><h3 id="NoDescription-macro"><code>[[NoDescription]]</code></h3><em>No documentation found</em></div><p>
+</p>
+------------------------------
+==============================
+[[MacroList(ProviderMacro)]]
+------------------------------
+<p>
+</p><div class="trac-macrolist"><h3 id="ProviderMacro1-macro"><code>[[ProviderMacro1]]</code></h3><p>
+ProviderMacro1 description
+</p>
+</div><p>
+</p>
+------------------------------
+"""
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(formatter.test_suite(IMAGE_MACRO_TEST_CASES,
                                        file=__file__,
                                        setup=image_setup,
                                        teardown=image_teardown))
+    suite.addTest(formatter.test_suite(INTERTRAC_MACRO_TEST_CASES,
+                                       file=__file__,
+                                       setup=intertrac_setup))
     suite.addTest(formatter.test_suite(TITLEINDEX1_MACRO_TEST_CASES,
                                        file=__file__))
     suite.addTest(formatter.test_suite(TITLEINDEX2_MACRO_TEST_CASES,
@@ -942,6 +1039,10 @@ def test_suite():
                                        file=__file__,
                                        setup=tracini_setup,
                                        teardown=tracini_teardown))
+    suite.addTest(formatter.test_suite(INTERWIKI_MACRO_TEST_CASES,
+                                       file=__file__, setup=interwiki_setup))
+    suite.addTest(formatter.test_suite(MACROLIST_MACRO_TEST_CASES,
+                                       file=__file__, setup=macrolist_setup))
     return suite
 
 
