@@ -1,22 +1,22 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013 Edgewall Software
+# Copyright (C) 2013-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
+# history and logs, available at https://trac.edgewall.org/log/.
+import textwrap
 import unittest
 from datetime import datetime
 
 from trac.test import EnvironmentStub, Mock
 from trac.tests.contentgen import random_sentence
-from trac.ticket.model import Ticket
+from trac.ticket.test import insert_ticket
 from trac.util.datefmt import utc
 from trac.versioncontrol.api import Repository, RepositoryManager
 from trac.wiki.tests import formatter
@@ -38,11 +38,9 @@ class CommitTicketUpdaterTestCase(unittest.TestCase):
 
     def _make_tickets(self, num):
         self.tickets = []
-        for i in xrange(0, num):
-            ticket = Ticket(self.env)
-            ticket['reporter'] = 'someone'
-            ticket['summary'] = random_sentence()
-            ticket.insert()
+        for i in range(num):
+            ticket = insert_ticket(self.env, reporter='someone',
+                                   summary=random_sentence())
             self.tickets.append(ticket)
 
     def test_changeset_added(self):
@@ -51,12 +49,31 @@ class CommitTicketUpdaterTestCase(unittest.TestCase):
         chgset = Mock(repos=self.repos, rev=1, message=message, author='joe',
                       date=datetime(2001, 1, 1, 1, 1, 1, 0, utc))
         self.updater.changeset_added(self.repos, chgset)
-        self.assertEqual("""\
-In [changeset:"1/repos1" 1/repos1]:
-{{{
-#!CommitTicketReference repository="repos1" revision="1"
-This is the first comment. Refs #1.
-}}}""", self.tickets[0].get_change(cnum=1)['fields']['comment']['new'])
+        changes = self.tickets[0].get_change(cnum=1)
+        self.assertEqual(textwrap.dedent("""\
+            In [changeset:"1/repos1" 1/repos1]:
+            {{{#!CommitTicketReference repository="repos1" revision="1"
+            This is the first comment. Refs #1.
+            }}}"""), changes['fields']['comment']['new'])
+
+    def test_changeset_added_multiline_comment(self):
+        self._make_tickets(1)
+        message = ("This is a multiline comment.\n\n"
+                   "It is multiline.\n\n"
+                   "Refs #1.")
+        chgset = Mock(repos=self.repos, rev=1, message=message, author='joe',
+                      date=datetime(2001, 1, 1, 1, 1, 1, 0, utc))
+        self.updater.changeset_added(self.repos, chgset)
+        changes = self.tickets[0].get_change(cnum=1)
+        self.assertEqual(textwrap.dedent("""\
+            In [changeset:"1/repos1" 1/repos1]:
+            {{{#!CommitTicketReference repository="repos1" revision="1"
+            This is a multiline comment.
+
+            It is multiline.
+
+            Refs #1.
+            }}}"""), changes['fields']['comment']['new'])
 
     def test_changeset_modified(self):
         self._make_tickets(2)
@@ -70,18 +87,18 @@ This is the first comment. Refs #1.
                           date=datetime(2001, 1, 2, 1, 1, 1, 0, utc))
         self.updater.changeset_added(self.repos, old_chgset)
         self.updater.changeset_modified(self.repos, new_chgset, old_chgset)
-        self.assertEqual("""\
-In [changeset:"1/repos1" 1/repos1]:
-{{{
-#!CommitTicketReference repository="repos1" revision="1"
-This is the first comment. Refs #1.
-}}}""", self.tickets[0].get_change(cnum=1)['fields']['comment']['new'])
-        self.assertEqual("""\
-In [changeset:"1/repos1" 1/repos1]:
-{{{
-#!CommitTicketReference repository="repos1" revision="1"
-This is the first comment after an edit. Refs #1, #2.
-}}}""", self.tickets[1].get_change(cnum=1)['fields']['comment']['new'])
+        changes = self.tickets[0].get_change(cnum=1)
+        self.assertEqual(textwrap.dedent("""\
+            In [changeset:"1/repos1" 1/repos1]:
+            {{{#!CommitTicketReference repository="repos1" revision="1"
+            This is the first comment. Refs #1.
+            }}}"""), changes['fields']['comment']['new'])
+        changes = self.tickets[1].get_change(cnum=1)
+        self.assertEqual(textwrap.dedent("""\
+            In [changeset:"1/repos1" 1/repos1]:
+            {{{#!CommitTicketReference repository="repos1" revision="1"
+            This is the first comment after an edit. Refs #1, #2.
+            }}}"""), changes['fields']['comment']['new'])
 
     def test_commands_refs(self):
         commands ={(1,): 'Refs #1', (2,): 'refs #2',
@@ -109,10 +126,7 @@ This is the first comment after an edit. Refs #1, #2.
 def macro_setup(tc):
     tc.env = EnvironmentStub(enable=('trac.*',
                                      'tracopt.ticket.commit_updater.*',))
-    ticket = Ticket(tc.env)
-    ticket['summary'] = 'the summary'
-    ticket['status'] = 'new'
-    ticket.insert()
+    insert_ticket(tc.env, summary='the summary', status='new')
     def _get_repository(reponame):
         return Mock(get_changeset=_get_changeset, resource=None)
     def _get_changeset(rev=None):
@@ -120,7 +134,7 @@ def macro_setup(tc):
     setattr(RepositoryManager(tc.env), 'get_repository', _get_repository)
 
 
-COMMIT_TICKET_REF_MACRO_TEST_CASES = u"""\
+COMMIT_TICKET_REF_MACRO_TEST_CASES = """\
 ============================== No arguments
 [[CommitTicketReference]]
 ------------------------------
