@@ -1,29 +1,25 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014 Edgewall Software
+# Copyright (C) 2014-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
+import io
 import os
 import sys
-import shutil
-import tempfile
 import unittest
-from cStringIO import StringIO
 from datetime import datetime, timedelta
-from subprocess import Popen, PIPE
+from subprocess import DEVNULL, PIPE, Popen
 
-import trac.tests.compat
 from trac.core import TracError
-from trac.test import EnvironmentStub, MockRequest, locate
-from trac.tests.compat import rmtree
+from trac.test import EnvironmentStub, MockRequest, locate, mkdtemp, rmtree
 from trac.util import create_file
 from trac.util.compat import close_fds
 from trac.util.datefmt import to_timestamp, utc
@@ -53,29 +49,27 @@ class GitCommandMixin(object):
         return self._git(*args, **kwargs)
 
     def _spawn_git(self, *args, **kwargs):
-        args = map(to_utf8, (self.git_bin,) + args)
-        kwargs.setdefault('stdin', PIPE)
+        args = (self.git_bin,) + args
+        kwargs.setdefault('stdin', DEVNULL)
         kwargs.setdefault('stdout', PIPE)
         kwargs.setdefault('stderr', PIPE)
         kwargs.setdefault('cwd', self.repos_path)
         return Popen(args, close_fds=close_fds, **kwargs)
 
     def _git(self, *args, **kwargs):
-        proc = self._spawn_git(*args, **kwargs)
-        stdout, stderr = proc.communicate()
-        self._close_proc_pipes(proc)
+        with self._spawn_git(*args, **kwargs) as proc:
+            stdout, stderr = proc.communicate()
         self.assertEqual(0, proc.returncode,
                          'git exits with %r, args %r, kwargs %r, stdout %r, '
                          'stderr %r' %
                          (proc.returncode, args, kwargs, stdout, stderr))
         return proc
 
-    def _git_fast_import(self, data):
-        if isinstance(data, unicode):
+    def _git_fast_import(self, data, **kwargs):
+        if isinstance(data, str):
             data = data.encode('utf-8')
-        proc = self._spawn_git('fast-import', stdin=PIPE)
-        stdout, stderr = proc.communicate(input=data)
-        self._close_proc_pipes(proc)
+        with self._spawn_git('fast-import', stdin=PIPE, **kwargs) as proc:
+            stdout, stderr = proc.communicate(input=data)
         self.assertEqual(0, proc.returncode,
                          'git exits with %r, stdout %r, stderr %r' %
                          (proc.returncode, stdout, stderr))
@@ -87,27 +81,27 @@ class GitCommandMixin(object):
         secs = offset.days * 3600 * 24 + offset.seconds
         hours, rem = divmod(abs(secs), 3600)
         return '%d %c%02d:%02d' % (to_timestamp(dt), '-' if secs < 0 else '+',
-                                   hours, rem / 60)
+                                   hours, rem // 60)
 
     def _set_committer_date(self, env, dt):
-        if not isinstance(dt, basestring):
+        if not isinstance(dt, str):
             if dt.tzinfo is None:
                 dt = dt.replace(tzinfo=utc)
             dt = self._git_date_format(dt)
         env['GIT_COMMITTER_DATE'] = dt
         env['GIT_AUTHOR_DATE'] = dt
 
-    def _close_proc_pipes(self, proc):
-        for f in (proc.stdin, proc.stdout, proc.stderr):
-            if f:
-                f.close()
-
 
 class BaseTestCase(unittest.TestCase, GitCommandMixin):
 
+    gitignore_content = b'*.py[co]\n' \
+                        b'\312\326\267\347\307\331.txt\n'
+
     def setUp(self):
         self.env = EnvironmentStub()
-        self.repos_path = tempfile.mkdtemp(prefix='trac-gitrepos-')
+        self.tmpdir = mkdtemp()
+        self.repos_path = os.path.join(self.tmpdir, 'gitrepos')
+        os.mkdir(self.repos_path)
         if self.git_bin:
             self.env.config.set('git', 'git_bin', self.git_bin)
 
@@ -117,8 +111,8 @@ class BaseTestCase(unittest.TestCase, GitCommandMixin):
         self._repomgr.reload_repositories()
         StorageFactory._clean()
         self.env.reset_db()
-        if os.path.isdir(self.repos_path):
-            rmtree(self.repos_path)
+        if os.path.isdir(self.tmpdir):
+            rmtree(self.tmpdir)
 
     @property
     def _repomgr(self):
@@ -128,23 +122,26 @@ class BaseTestCase(unittest.TestCase, GitCommandMixin):
     def _dbrepoprov(self):
         return DbRepositoryProvider(self.env)
 
-    def _add_repository(self, reponame='gitrepos', bare=False):
-        path = self.repos_path \
-               if bare else os.path.join(self.repos_path, '.git')
+    def _add_repository(self, reponame='gitrepos', bare=False, path=None):
+        if path is None:
+            path = self.repos_path
+        if not bare:
+            path = os.path.join(path, '.git')
         self._dbrepoprov.add_repository(reponame, path, 'git')
 
-    def _git_init(self, data=True, bare=False):
+    def _git_init(self, data=True, bare=False, **kwargs):
         if bare:
-            self._git('init', '--bare')
+            self._git('init', '--bare', **kwargs)
         else:
-            self._git('init')
+            self._git('init', **kwargs)
         if not bare and data:
-            self._git('config', 'user.name', 'Joe')
-            self._git('config', 'user.email', 'joe@example.com')
-            create_file(os.path.join(self.repos_path, '.gitignore'))
-            self._git('add', '.gitignore')
+            self._git('config', 'user.name', 'Joe', **kwargs)
+            self._git('config', 'user.email', 'joe@example.com', **kwargs)
+            create_file(os.path.join(self.repos_path, '.gitignore'),
+                        self.gitignore_content, 'wb')
+            self._git('add', '.gitignore', **kwargs)
             self._git_commit('-a', '-m', 'test',
-                             date=datetime(2001, 1, 29, 16, 39, 56))
+                             date=datetime(2001, 1, 29, 16, 39, 56), **kwargs)
 
 
 class SanityCheckingTestCase(BaseTestCase):
@@ -235,7 +232,7 @@ class HistoryTimeRangeTestCase(BaseTestCase):
         filename = os.path.join(self.repos_path, '.gitignore')
         start = datetime(2000, 1, 1, 0, 0, 0, 0, utc)
         ts = datetime(2014, 2, 5, 15, 24, 6, 0, utc)
-        for idx in xrange(3):
+        for idx in range(3):
             create_file(filename, 'commit-%d.txt' % idx)
             self._git_commit('-a', '-m', 'commit %d' % idx, date=ts)
         self._add_repository()
@@ -268,13 +265,17 @@ class GitNormalTestCase(BaseTestCase):
         self._add_repository()
         repos = self._repomgr.get_repository('gitrepos')
         rev = repos.youngest_rev
-        self.assertNotEqual(None, rev)
+        self.assertIsInstance(rev, str)
         self.assertEqual(40, len(rev))
 
         self.assertEqual(rev, repos.get_node('/').rev)
         self.assertEqual(rev, repos.get_node('/', rev[:7]).rev)
         self.assertEqual(rev, repos.get_node('/.gitignore').rev)
         self.assertEqual(rev, repos.get_node('/.gitignore', rev[:7]).rev)
+        self.assertEqual(self.gitignore_content,
+                         repos.get_node('/.gitignore').get_content().read())
+        self.assertEqual(len(self.gitignore_content),
+                         repos.get_node('/.gitignore').content_length)
 
         self.assertRaises(NoSuchNode, repos.get_node, '/non-existent')
         self.assertRaises(NoSuchNode, repos.get_node, '/non-existent', rev[:7])
@@ -286,13 +287,10 @@ class GitNormalTestCase(BaseTestCase):
         self.assertRaises(NoSuchChangeset,
                           repos.get_node, '/non-existent', 'invalid-revision')
 
-        # git_fs doesn't support non-ANSI strings on Windows
-        if os.name != 'nt':
-            self._git('branch', u'tïckét10605', 'master')
-            repos.sync()
-            self.assertEqual(rev, repos.get_node('/', u'tïckét10605').rev)
-            self.assertEqual(rev, repos.get_node('/.gitignore',
-                                                 u'tïckét10605').rev)
+        self._git('branch', 'tïckét10605', 'master')
+        repos.sync()
+        self.assertEqual(rev, repos.get_node('/', 'tïckét10605').rev)
+        self.assertEqual(rev, repos.get_node('/.gitignore', 'tïckét10605').rev)
 
     def _test_on_empty_repos(self, cached_repository):
         self.env.config.set('git', 'persistent_cache', 'false')
@@ -304,7 +302,7 @@ class GitNormalTestCase(BaseTestCase):
         repos = self._repomgr.get_repository('gitrepos')
         if cached_repository:
             # call sync() thrice with empty repository (#11851)
-            for i in xrange(3):
+            for i in range(3):
                 repos.sync()
                 rows = self.env.db_query("SELECT value FROM repository "
                                          "WHERE id=%s AND name=%s",
@@ -313,14 +311,14 @@ class GitNormalTestCase(BaseTestCase):
         else:
             repos.sync()
         youngest_rev = repos.youngest_rev
-        self.assertEqual(None, youngest_rev)
-        self.assertEqual(None, repos.oldest_rev)
-        self.assertEqual(None, repos.normalize_rev(''))
-        self.assertEqual(None, repos.normalize_rev(None))
-        self.assertEqual(None, repos.display_rev(''))
-        self.assertEqual(None, repos.display_rev(None))
-        self.assertEqual(None, repos.short_rev(''))
-        self.assertEqual(None, repos.short_rev(None))
+        self.assertIsNone(youngest_rev)
+        self.assertIsNone(repos.oldest_rev)
+        self.assertIsNone(repos.normalize_rev(''))
+        self.assertIsNone(repos.normalize_rev(None))
+        self.assertIsNone(repos.display_rev(''))
+        self.assertIsNone(repos.display_rev(None))
+        self.assertIsNone(repos.short_rev(''))
+        self.assertIsNone(repos.short_rev(None))
 
         node = repos.get_node('/', youngest_rev)
         self.assertEqual([], list(node.get_entries()))
@@ -332,7 +330,7 @@ class GitNormalTestCase(BaseTestCase):
         self.assertTrue(browser_mod.match_request(req))
         rv = browser_mod.process_request(req)
         self.assertEqual('browser.html', rv[0])
-        self.assertEqual(None, rv[1]['rev'])
+        self.assertIsNone(rv[1]['rev'])
 
         req = MockRequest(self.env, path_info='/log/gitrepos')
         log_mod = LogModule(self.env)
@@ -359,7 +357,7 @@ class GitRepositoryTestCase(BaseTestCase):
     def _create_merge_commit(self):
         for idx, branch in enumerate(('alpha', 'beta')):
             self._git('checkout', '-b', branch, 'master')
-            for n in xrange(2):
+            for n in range(2):
                 filename = 'file-%s-%d.txt' % (branch, n)
                 create_file(os.path.join(self.repos_path, filename))
                 self._git('add', filename)
@@ -370,8 +368,20 @@ class GitRepositoryTestCase(BaseTestCase):
         self._git('merge', '-m', 'Merge branch "beta" to "alpha"', 'beta')
 
     def test_invalid_path_raises(self):
-        self.assertRaises(InvalidRepository, GitRepository, self.env,
-                          '/the/invalid/path', [], self.env.log)
+        def try_init(reponame):
+            params = {'name': reponame}
+            with self.assertRaises(InvalidRepository) as cm:
+                GitRepository(self.env, '/the/invalid/path', params,
+                              self.env.log)
+            return cm.exception
+
+        e = try_init('')
+        self.assertEqual('"(default)" is not readable or not a Git '
+                         'repository.', str(e))
+
+        e = try_init('therepos')
+        self.assertEqual('"therepos" is not readable or not a Git repository.',
+                         str(e))
 
     def test_repository_instance(self):
         self._git_init()
@@ -392,16 +402,26 @@ class GitRepositoryTestCase(BaseTestCase):
         entries = list(repos.get_node('').get_history())
         self.assertEqual(2, len(entries))
         self.assertEqual('', entries[0][0])
+        self.assertEqual(youngest_rev, entries[0][1])
         self.assertEqual(Changeset.EDIT, entries[0][2])
         self.assertEqual('', entries[1][0])
+        self.assertIsInstance(entries[1][1], str)
         self.assertEqual(Changeset.ADD, entries[1][2])
 
+        self.assertEqual(entries[1][1], repos.previous_rev(youngest_rev, ''))
+        self.assertEqual(None, repos.next_rev(youngest_rev, ''))
+        self.assertEqual(None, repos.previous_rev(entries[1][1], ''))
+        self.assertEqual(youngest_rev, repos.next_rev(entries[1][1], ''))
+
         self._git('reset', '--hard', 'HEAD~')
         repos.sync()
         new_entries = list(repos.get_node('').get_history())
         self.assertEqual(1, len(new_entries))
         self.assertEqual(new_entries[0], entries[1])
         self.assertNotEqual(youngest_rev, repos.youngest_rev)
+        self.assertEqual(new_entries[0][1], repos.youngest_rev)
+        self.assertEqual(None, repos.next_rev(repos.youngest_rev, ''))
+        self.assertEqual(None, repos.previous_rev(repos.youngest_rev, ''))
 
     def test_tags(self):
         self._git_init()
@@ -503,7 +523,7 @@ class GitRepositoryTestCase(BaseTestCase):
                                                       rev))),
                          sorted(cset.get_changes()))
 
-    _data_annotations = """\
+    _data_annotations = b"""\
 blob
 mark :1
 data 14
@@ -608,7 +628,7 @@ from :6
     # * | 998bf23843c8fd982bbc23f88ec33c4d08114557 Changed b1
     # |/
     # * c5b01c74e125aa034a1d4ae31dc16f1897a73779 First commit
-    _data_iter_nodes = """\
+    _data_iter_nodes = b"""\
 blob
 mark :1
 data 2
@@ -781,6 +801,155 @@ from :17
             (root_commit, 'B/b2.txt'),
             ], [(node.created_rev, node.path) for node in nodes])
 
+    def test_colon_character_in_filename(self):
+        self._git_init(data=False)
+        self._git_fast_import(self._data_colon_character_in_filename)
+        self._add_repository('gitrepos')
+        repos = self._repomgr.get_repository('gitrepos')
+        repos.sync()
+        rev1 = '382e1e6b85ba20ce8a84af1a875eaa50b8e1e092'  # root commit
+        rev2 = 'd8001832aad079f85a39a54a388a8b15fe31093d'
+        ADD = Changeset.ADD
+        MOVE = Changeset.MOVE
+        FILE = Node.FILE
+
+        cset = repos.get_changeset(rev1)
+        self.assertEqual({('0100644',     FILE, ADD, None, None),
+                          ('0100644.txt', FILE, ADD, None, None),
+                          (':100644',     FILE, ADD, None, None),
+                          (':100644.txt', FILE, ADD, None, None),
+                          ('a100644',     FILE, ADD, None, None),
+                          ('a100644.txt', FILE, ADD, None, None)},
+                         set(cset.get_changes()))
+
+        cset = repos.get_changeset(rev2)
+        self.assertEqual({(':100666', FILE, MOVE, ':100644', rev1)},
+                         set(cset.get_changes()))
+
+    _data_colon_character_in_filename = b"""\
+blob
+mark :1
+data 0
+
+blob
+mark :2
+data 16
+...............
+
+reset refs/heads/master
+commit refs/heads/master
+mark :3
+author Joe <joe@example.com> 1491387182 +0000
+committer Joe <joe@example.com> 1491387182 +0000
+data 9
+(#12758)
+M 100644 :1 0100644.txt
+M 100644 :1 0100644
+M 100644 :1 :100644.txt
+M 100644 :2 :100644
+M 100644 :1 a100644.txt
+M 100644 :1 a100644
+
+commit refs/heads/master
+mark :4
+author Joe <joe@example.com> 1491387183 +0000
+committer Joe <joe@example.com> 1491387183 +0000
+data 16
+(#12758) rename
+from :3
+D :100644
+M 100644 :2 :100666
+
+reset refs/heads/master
+from :4
+"""
+
+    def test_submodule(self):
+        subrepos_path = os.path.join(self.tmpdir, 'subrepos')
+        submodule_dir = os.path.join(self.repos_path, 'sub')
+        os.mkdir(subrepos_path)
+        self._git_init(data=False, bare=True, cwd=subrepos_path)
+        self._git_fast_import(self._data_submodule, cwd=subrepos_path)
+
+        submodule_rev1 = '3e733d786b3529d750ee39edacea2f1c4daadca4'
+        self._git_init()
+        self._git('submodule', 'add', subrepos_path, 'sub')
+        self._git('checkout', submodule_rev1, cwd=submodule_dir)
+        self._git('add', '.gitmodules')
+        self._git_commit('-a', '-m', 'init submodule')
+        self._git('tag', 'v1', 'master')
+        self._add_repository('gitrepos')
+        repos = self._repomgr.get_repository('gitrepos')
+        repos.sync()
+
+        ADD = Changeset.ADD
+        EDIT = Changeset.EDIT
+        FILE = Node.FILE
+        DIRECTORY = Node.DIRECTORY
+
+        rev1 = repos.normalize_rev('v1')
+        cset1 = repos.get_changeset(rev1)
+        self.assertEqual([('.gitmodules', FILE,      ADD, None, None),
+                          ('sub',         DIRECTORY, ADD, None, None)],
+                         sorted(cset1.get_changes()))
+        node1 = repos.get_node('sub', rev1)
+        self.assertIsNone(node1.get_content())
+        self.assertIsNone(node1.get_content_length())
+        self.assertEqual(DIRECTORY, node1.kind)
+        self.assertEqual({'mode': '160000', 'commit': submodule_rev1},
+                         node1.get_properties())
+        self.assertEqual([], list(node1.get_entries()))
+
+        submodule_rev2 = '409058dc98500b5685c52a091cc9f44f3975113e'
+        self._git('checkout', submodule_rev2, cwd=submodule_dir)
+        self._git_commit('-a', '-m', 'change rev of the submodule')
+        self._git('tag', 'v2', 'master')
+        repos.sync()
+        rev2 = repos.normalize_rev('v2')
+        cset2 = repos.get_changeset(rev2)
+        self.assertEqual([('sub', DIRECTORY, EDIT, 'sub', rev1)],
+                         sorted(cset2.get_changes()))
+        node2 = repos.get_node('sub', rev2)
+        self.assertEqual({'mode': '160000', 'commit': submodule_rev2},
+                         node2.get_properties())
+
+    _data_submodule = b"""\
+blob
+mark :1
+data 0
+
+blob
+mark :2
+data 16
+...............
+
+# <= 3e733d786b3529d750ee39edacea2f1c4daadca4
+reset refs/heads/master
+commit refs/heads/master
+mark :3
+author Joe <joe@example.com> 1512643825 +0000
+committer Joe <joe@example.com> 1512643825 +0000
+data 12
+root commit
+M 100644 :1 001-001.txt
+M 100644 :2 001-002.txt
+M 100644 :2 001-003.txt
+
+# <= 409058dc98500b5685c52a091cc9f44f3975113e
+commit refs/heads/master
+mark :4
+author Joe <joe@example.com> 1512643826 +0000
+committer Joe <joe@example.com> 1512643826 +0000
+data 10
+2nd commit
+from :3
+M 100644 :1 002-001.txt
+M 100644 :2 002-002.txt
+
+reset refs/heads/master
+from :4
+"""
+
     def _get_quickjump_names(self, repos):
         return list(name for type, name, path, rev
                          in repos.get_quickjump_entries('HEAD'))
@@ -798,7 +967,7 @@ class GitCachedRepositoryTestCase(GitRepositoryTestCase):
 
     def test_sync(self):
         self._git_init()
-        for idx in xrange(3):
+        for idx in range(3):
             filename = 'file%d.txt' % idx
             create_file(os.path.join(self.repos_path, filename))
             self._git('add', filename)
@@ -831,7 +1000,7 @@ class GitCachedRepositoryTestCase(GitRepositoryTestCase):
 
     def test_sync_file_with_invalid_byte_sequence(self):
         self._git_init(data=False)
-        self._git_fast_import("""\
+        self._git_fast_import(b"""\
 blob
 mark :1
 data 0
@@ -843,7 +1012,7 @@ author Ryan Ollos <rjollos@edgewall.com> 1463639119 +0200
 committer Ryan Ollos <rjollos@edgewall.com> 1463639119 +0200
 data 9
 (#12322)
-M 100644 :1 "\312\326\267\347\307\331.txt"
+M 100644 :1 "\\312\\326\\267\\347\\307\\331.txt"
 
 reset refs/heads/master
 from :2
@@ -859,7 +1028,7 @@ from :2
 
         changes = list(repos.repos.get_changeset(revs[0]).get_changes())
         self.assertEqual(1, len(changes))
-        self.assertEqual(u'\ufffd\u05b7\ufffd\ufffd\ufffd.txt', changes[0][0])
+        self.assertEqual('\ufffd\u05b7\ufffd\ufffd\ufffd.txt', changes[0][0])
 
     def test_sync_merge(self):
         self._git_init()
@@ -893,25 +1062,20 @@ from :2
         self.assertEqual(revs, revs2)
 
     def test_sync_too_many_merges(self):
-        data = self._generate_data_many_merges(100)
+        n_merges = 2000
+        data = self._generate_data_many_merges(n_merges)
         self._git_init(data=False, bare=True)
         self._git_fast_import(data)
         self._add_repository('gitrepos', bare=True)
         repos = self._repomgr.get_repository('gitrepos')
-
-        reclimit = sys.getrecursionlimit()
-        try:
-            sys.setrecursionlimit(80)
-            repos.sync()
-        finally:
-            sys.setrecursionlimit(reclimit)
+        repos.sync()
 
         rows = self.env.db_query("SELECT COUNT(*) FROM revision "
                                  "WHERE repos=%s", (repos.id,))
-        self.assertEqual(202, rows[0][0])
+        self.assertEqual(n_merges * 2 + 2, rows[0][0])
 
     def _generate_data_many_merges(self, n, timestamp=1400000000):
-        init = """\
+        init = b"""\
 blob
 mark :1
 data 0
@@ -935,7 +1099,7 @@ from :2
 M 100644 :1 master.txt
 
 """
-        merge = """\
+        merge = b"""\
 commit refs/heads/dev
 mark :%(dev)d
 author Joe <joe@example.com> %(timestamp)d +0000
@@ -956,13 +1120,66 @@ merge :%(dev)d
 M 100644 :1 dev%(dev)08d.txt
 
 """
-        data = StringIO()
-        data.write(init % {'timestamp': timestamp})
-        for idx in xrange(n):
-            data.write(merge % {'timestamp': timestamp,
-                                'dev': 4 + idx * 2,
-                                'merge': 5 + idx * 2,
-                                'from': 3 + idx * 2})
+        data = io.BytesIO()
+        data.write(init % {b'timestamp': timestamp})
+        for idx in range(n):
+            data.write(merge % {b'timestamp': timestamp,
+                                b'dev': 4 + idx * 2,
+                                b'merge': 5 + idx * 2,
+                                b'from': 3 + idx * 2})
+        return data.getvalue()
+
+    def test_sync_many_refs(self):
+        n_refs = 1500
+        data = self._generate_data_many_refs(n_refs)
+        self._git_init(data=False, bare=True)
+        self._git_fast_import(data)
+        self._add_repository('gitrepos', bare=True)
+        repos = self._repomgr.get_repository('gitrepos')
+
+        revs = []
+        def feedback(rev):
+            revs.append(rev)
+        repos.sync(feedback)  # create cache
+        self.assertEqual(n_refs + 1, len(revs))
+
+        revs[:] = ()
+        repos.sync(feedback)  # check whether all refs are cached
+        self.assertEqual(0, len(revs))
+
+        rows = self.env.db_query("SELECT COUNT(*) FROM revision "
+                                 "WHERE repos=%s", (repos.id,))
+        self.assertEqual(n_refs + 1, rows[0][0])
+
+    def _generate_data_many_refs(self, n, timestamp=1400000000):
+        root_commit = """\
+blob
+mark :1
+data 0
+
+reset refs/heads/master
+commit refs/heads/master
+mark :2
+author Joe <joe@example.com> %(timestamp)d +0000
+committer Joe <joe@example.com> %(timestamp)d +0000
+data 12
+root commit
+M 100644 :1 .gitignore
+"""
+        ref_commit = """
+commit refs/heads/ref-%(ref)08d
+mark :%(mark)d
+author Joe <joe@example.com> %(timestamp)d +0000
+committer Joe <joe@example.com> %(timestamp)d +0000
+data 13
+ref-%(ref)08d
+from :2
+"""
+        data = io.BytesIO()
+        data.write(to_utf8(root_commit % {'timestamp': timestamp}))
+        for idx in range(n):
+            data.write(to_utf8(ref_commit % {'timestamp': timestamp + idx,
+                                             'mark': idx + 3, 'ref': idx}))
         return data.getvalue()
 
 
@@ -970,9 +1187,9 @@ class GitwebProjectsRepositoryProviderTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub()
-        self.projects_base = os.path.realpath(tempfile.mkdtemp())
+        self.projects_base = mkdtemp()
         self.projects_list = os.path.join(self.projects_base, 'projects_list')
-        with open(self.projects_list, 'w') as f:
+        with open(self.projects_list, 'w', encoding='utf-8') as f:
             f.write("""
             repos1 user1
             repos2.git user+2+<user@example.com>
@@ -988,7 +1205,7 @@ class GitwebProjectsRepositoryProviderTestCase(unittest.TestCase):
 
     def tearDown(self):
         self.env.shutdown()
-        shutil.rmtree(self.projects_base)
+        rmtree(self.projects_base)
 
     def test_project_list_path_not_found(self):
         """Warning is logged when projects_list file is not found, but
@@ -1034,7 +1251,7 @@ class GitConnectorTestCase(BaseTestCase):
 
     def _git_version_from_system_info(self):
         git_version = None
-        for name, version in self.env.get_systeminfo():
+        for name, version in self.env.system_info:
             if name == 'GIT':
                 git_version = version
         return git_version
